@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -152,24 +154,26 @@ vbp_has_poked(struct vbp_target *vt)
 void
 VBP_Update_Backend(struct vbp_target *vt)
 {
+	const struct director *dir;
 	unsigned i = 0, chg;
 	char bits[10];
 
 	CHECK_OBJ_NOTNULL(vt, VBP_TARGET_MAGIC);
 
+#define BITMAP(n, c, t, b)			\
+	bits[i++] = (vt->n & 1) ? c : '-';
+#include "tbl/backend_poll.h"
+	bits[i] = '\0';
+	assert(i < sizeof bits);
+
 	Lck_Lock(&vbp_mtx);
 	if (vt->backend == NULL) {
 		Lck_Unlock(&vbp_mtx);
 		return;
 	}
 
-#define BITMAP(n, c, t, b) \
-	bits[i++] = (vt->n & 1) ? c : '-';
-#include "tbl/backend_poll.h"
-	bits[i] = '\0';
-	assert(i < sizeof bits);
-
-	if (vt->backend->director == NULL) {
+	dir = vt->backend->director;
+	if (dir == NULL) {
 		Lck_Unlock(&vbp_mtx);
 		return;
 	}
@@ -178,8 +182,9 @@ VBP_Update_Backend(struct vbp_target *vt)
 	chg = (i != vt->backend->sick);
 	vt->backend->sick = i;
 
-	VSL(SLT_Backend_health, 0, "%s %s %s %s %u %u %u %.6f %.6f %s",
-	    vt->backend->director->vcl_name, chg ? "Went" : "Still",
+	AN(dir->vcl_name);
+	VSL(SLT_Backend_health, 0, "%s %s %s %s %u %u %u %.6f %.6f \"%s\"",
+	    dir->vcl_name, chg ? "Went" : "Still",
 	    i ? "sick" : "healthy", bits,
 	    vt->good, vt->threshold, vt->window,
 	    vt->last, vt->avg, vt->resp_buf);
@@ -187,7 +192,7 @@ VBP_Update_Backend(struct vbp_target *vt)
 
 	if (chg) {
 		vt->backend->changed = VTIM_real();
-		VRT_SetChanged(vt->backend->director, vt->backend->changed);
+		VRT_SetChanged(dir, vt->backend->changed);
 	}
 	Lck_Unlock(&vbp_mtx);
 }
@@ -406,6 +411,17 @@ vbp_poke(struct vbp_target *vt)
 		vt->happy |= 1;
 }
 
+/*--------------------------------------------------------------------
+ */
+static void
+vbp_heap_insert(struct vbp_target *vt)
+{
+	// Lck_AssertHeld(&vbp_mtx);
+	binheap_insert(vbp_heap, vt);
+	if (binheap_root(vbp_heap) == vt)
+		AZ(pthread_cond_signal(&vbp_cond));
+}
+
 /*--------------------------------------------------------------------
  */
 
@@ -435,7 +451,7 @@ vbp_task(struct worker *wrk, void *priv)
 		if (vt->heap_idx != BINHEAP_NOIDX) {
 			vt->due = VTIM_real() + vt->interval;
 			binheap_delete(vbp_heap, vt->heap_idx);
-			binheap_insert(vbp_heap, vt);
+			vbp_heap_insert(vt);
 		}
 	}
 	Lck_Unlock(&vbp_mtx);
@@ -449,6 +465,7 @@ vbp_thread(struct worker *wrk, void *priv)
 {
 	vtim_real now, nxt;
 	struct vbp_target *vt;
+	int r;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	AZ(priv);
@@ -470,14 +487,17 @@ vbp_thread(struct worker *wrk, void *priv)
 				vt->running = 1;
 				vt->task.func = vbp_task;
 				vt->task.priv = vt;
-				if (Pool_Task_Any(&vt->task, TASK_QUEUE_REQ))
+				Lck_Unlock(&vbp_mtx);
+				r = Pool_Task_Any(&vt->task, TASK_QUEUE_REQ);
+				Lck_Lock(&vbp_mtx);
+				if (r)
 					vt->running = 0;
 			}
 			binheap_insert(vbp_heap, vt);
 		}
 	}
 	NEEDLESS(Lck_Unlock(&vbp_mtx));
-	NEEDLESS(return NULL);
+	NEEDLESS(return (NULL));
 }
 
 
@@ -491,7 +511,7 @@ vbp_bitmap(struct vsb *vsb, char c, uint64_t map, const char *lbl)
 	int i;
 	uint64_t u = (1ULL << 63);
 
-	VSB_printf(vsb, "  ");
+	VSB_cat(vsb, "  ");
 	for (i = 0; i < 64; i++) {
 		if (map & u)
 			VSB_putc(vsb, c);
@@ -525,7 +545,7 @@ VBP_Status(struct vsb *vsb, const struct backend *be, int details, int json)
 	}
 
 	if (json) {
-		VSB_printf(vsb, "{\n");
+		VSB_cat(vsb, "{\n");
 		VSB_indent(vsb, 2);
 #define BITMAP(nn, cc, tt, bb)					\
 		VSB_printf(vsb, "\"bits_%c\": %ju,\n", cc, vt->nn);
@@ -534,8 +554,8 @@ VBP_Status(struct vsb *vsb, const struct backend *be, int details, int json)
 		VSB_printf(vsb, "\"threshold\": %u,\n", vt->threshold);
 		VSB_printf(vsb, "\"window\": %u", vt->window);
 		VSB_indent(vsb, -2);
-		VSB_printf(vsb, "\n");
-		VSB_printf(vsb, "},\n");
+		VSB_cat(vsb, "\n");
+		VSB_cat(vsb, "},\n");
 		return;
 	}
 
@@ -544,7 +564,7 @@ VBP_Status(struct vsb *vsb, const struct backend *be, int details, int json)
 	    vt->good, vt->threshold, vt->window);
 	VSB_printf(vsb,
 	    "  Average response time of good probes: %.6f\n", vt->avg);
-	VSB_printf(vsb,
+	VSB_cat(vsb,
 	    "  Oldest ======================"
 	    "============================ Newest\n");
 
@@ -570,12 +590,14 @@ vbp_build_req(struct vbp_target *vt, const struct vrt_backend_probe *vbp,
 	if (vbp->request != NULL) {
 		VSB_cat(vsb, vbp->request);
 	} else {
-		VSB_printf(vsb, "GET %s HTTP/1.1\r\n",
-		    vbp->url != NULL ?  vbp->url : "/");
-		if (be->hosthdr != NULL)
-			VSB_printf(vsb, "Host: %s\r\n", be->hosthdr);
-		VSB_printf(vsb, "Connection: close\r\n");
-		VSB_printf(vsb, "\r\n");
+		AN(be->hosthdr);
+		VSB_printf(vsb,
+		    "GET %s HTTP/1.1\r\n"
+		    "Host: %s\r\n"
+		    "Connection: close\r\n"
+		    "\r\n",
+		    vbp->url != NULL ?  vbp->url : "/",
+		    be->hosthdr);
 	}
 	AZ(VSB_finish(vsb));
 	vt->req = strdup(VSB_data(vsb));
@@ -634,8 +656,7 @@ VBP_Control(const struct backend *be, int enable)
 	if (enable) {
 		assert(vt->heap_idx == BINHEAP_NOIDX);
 		vt->due = VTIM_real();
-		binheap_insert(vbp_heap, vt);
-		AZ(pthread_cond_signal(&vbp_cond));
+		vbp_heap_insert(vt);
 	} else {
 		assert(vt->heap_idx != BINHEAP_NOIDX);
 		binheap_delete(vbp_heap, vt->heap_idx);
@@ -703,13 +724,17 @@ static int v_matchproto_(binheap_cmp_t)
 vbp_cmp(void *priv, const void *a, const void *b)
 {
 	const struct vbp_target *aa, *bb;
+	int ar, br;
 
 	AZ(priv);
 	CAST_OBJ_NOTNULL(aa, a, VBP_TARGET_MAGIC);
 	CAST_OBJ_NOTNULL(bb, b, VBP_TARGET_MAGIC);
 
-	if (aa->running && !bb->running)
-		return (0);
+	ar = aa->running == 0;
+	br = bb->running == 0;
+
+	if (ar != br)
+		return (ar);
 
 	return (aa->due < bb->due);
 }
@@ -731,7 +756,7 @@ VBP_Init(void)
 {
 	pthread_t thr;
 
-	Lck_New(&vbp_mtx, lck_backend);
+	Lck_New(&vbp_mtx, lck_probe);
 	vbp_heap = binheap_new(NULL, vbp_cmp, vbp_update);
 	AN(vbp_heap);
 	AZ(pthread_cond_init(&vbp_cond, NULL));
