@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -113,7 +115,7 @@ ved_include(struct req *preq, const char *src, const char *host,
 	enum req_fsm_nxt s;
 
 	CHECK_OBJ_NOTNULL(preq, REQ_MAGIC);
-	CHECK_OBJ_NOTNULL(preq->topreq, REQ_MAGIC);
+	CHECK_OBJ_NOTNULL(preq->top, REQTOP_MAGIC);
 	sp = preq->sp;
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(ecx, ECX_MAGIC);
@@ -139,7 +141,8 @@ ved_include(struct req *preq, const char *src, const char *host,
 
 	req->esi_level = preq->esi_level + 1;
 
-	req->topreq = preq->topreq;
+	memset(req->top, 0, sizeof *req->top);
+	req->top = preq->top;
 
 	HTTP_Setup(req->http, req->ws, req->vsl, SLT_ReqMethod);
 	HTTP_Dup(req->http, preq->http0);
@@ -161,17 +164,19 @@ ved_include(struct req *preq, const char *src, const char *host,
 	http_Unset(req->http, H_Range);
 
 	/* Set Accept-Encoding according to what we want */
-	http_Unset(req->http, H_Accept_Encoding);
 	if (ecx->isgzip)
 		http_ForceHeader(req->http, H_Accept_Encoding, "gzip");
+	else
+		http_Unset(req->http, H_Accept_Encoding);
 
 	/* Client content already taken care of */
 	http_Unset(req->http, H_Content_Length);
-	req->req_body_status = REQ_BODY_NONE;
+	req->req_body_status = BS_NONE;
 
 	AZ(req->vcl);
-	if (req->topreq->vcl0)
-		req->vcl = req->topreq->vcl0;
+	AN(req->top);
+	if (req->top->vcl0)
+		req->vcl = req->top->vcl0;
 	else
 		req->vcl = preq->vcl;
 	VCL_Ref(req->vcl);
@@ -183,7 +188,7 @@ ved_include(struct req *preq, const char *src, const char *host,
 	req->transport_priv = ecx;
 
 	CNT_Embark(wrk, req);
-	VCL_TaskEnter(req->vcl, req->privs);
+	VCL_TaskEnter(req->privs);
 
 	while (1) {
 		ecx->woken = 0;
@@ -560,7 +565,7 @@ struct ved_foo {
 	unsigned		magic;
 #define VED_FOO_MAGIC		0x6a5a262d
 	struct ecx		*ecx;
-	ssize_t			start, last, stop, lpad;
+	uint64_t		start, last, stop, lpad;
 	ssize_t			ll;
 	uint64_t		olen;
 	uint8_t			dbits[8];
