@@ -5,6 +5,8 @@
  *
  * Author: Dag-Erling Smørgrav <des@des.no>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -29,7 +31,12 @@
 
 #include "config.h"
 
-#include <execinfo.h>
+#ifdef WITH_UNWIND
+#  include <libunwind.h>
+#else
+#  include <execinfo.h>
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -47,12 +54,13 @@
 #include "vcs.h"
 
 /*
- * The panic string is constructed in memory, then copied to the
+ * The panic string is constructed in a VSB, then copied to the
  * shared memory.
  *
  * It can be extracted post-mortem from a core dump using gdb:
  *
- * (gdb) printf "%s", panicstr
+ * (gdb) p *(char **)((char *)pan_vsb+8)
+ *
  */
 
 static struct vsb pan_vsb_storage, *pan_vsb;
@@ -63,32 +71,6 @@ static void pan_req(struct vsb *, const struct req *);
 
 /*--------------------------------------------------------------------*/
 
-const char *
-body_status_2str(enum body_status e)
-{
-	switch (e) {
-#define BODYSTATUS(U,l)	case BS_##U: return (#l);
-#include "tbl/body_status.h"
-	default:
-		return ("?");
-	}
-}
-
-/*--------------------------------------------------------------------*/
-
-static const char *
-reqbody_status_2str(enum req_body_state_e e)
-{
-	switch (e) {
-#define REQ_BODY(U) case REQ_BODY_##U: return("R_BODY_" #U);
-#include "tbl/req_body.h"
-	default:
-		return ("?");
-	}
-}
-
-/*--------------------------------------------------------------------*/
-
 static const char *
 boc_state_2str(enum boc_state_e e)
 {
@@ -127,13 +109,13 @@ PAN_already(struct vsb *vsb, const void *ptr)
 	int i;
 
 	if (ptr == NULL) {
-		VSB_printf(vsb, "},\n");
+		VSB_cat(vsb, "},\n");
 		return (1);
 	}
 	for (i = 0; i < already_idx; i++) {
 		if (already_list[i] == ptr) {
-			VSB_printf(vsb, "  [Already dumped, see above]\n");
-			VSB_printf(vsb, "},\n");
+			VSB_cat(vsb, "  [Already dumped, see above]\n");
+			VSB_cat(vsb, "},\n");
 			return (1);
 		}
 	}
@@ -154,7 +136,7 @@ pan_ws(struct vsb *vsb, const struct ws *ws)
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, ws, WS_MAGIC);
 	if (ws->id[0] != '\0' && (!(ws->id[0] & 0x20)))
-		VSB_printf(vsb, "OVERFLOWED ");
+		VSB_cat(vsb, "OVERFLOWED ");
 	VSB_printf(vsb, "id = \"%s\",\n", ws->id);
 	VSB_printf(vsb, "{s, f, r, e} = {%p", ws->s);
 	if (ws->f >= ws->s)
@@ -169,9 +151,9 @@ pan_ws(struct vsb *vsb, const struct ws *ws)
 		VSB_printf(vsb, ", +%ld", (long) (ws->e - ws->s));
 	else
 		VSB_printf(vsb, ", %p", ws->e);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -196,13 +178,13 @@ pan_htc(struct vsb *vsb, const struct http_conn *htc)
 	VSB_printf(vsb, "content_length = %jd,\n",
 	    (intmax_t)htc->content_length);
 	VSB_printf(vsb, "body_status = %s,\n",
-	    body_status_2str(htc->body_status));
+	    htc->body_status ? htc->body_status->name : "NULL");
 	VSB_printf(vsb, "first_byte_timeout = %f,\n",
 	    htc->first_byte_timeout);
 	VSB_printf(vsb, "between_bytes_timeout = %f,\n",
 	    htc->between_bytes_timeout);
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -218,7 +200,7 @@ pan_http(struct vsb *vsb, const char *id, const struct http *h)
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, h, HTTP_MAGIC);
 	pan_ws(vsb, h->ws);
-	VSB_printf(vsb, "hdrs {\n");
+	VSB_cat(vsb, "hdrs {\n");
 	VSB_indent(vsb, 2);
 	for (i = 0; i < h->nhd; ++i) {
 		if (h->hd[i].b == NULL && h->hd[i].e == NULL)
@@ -227,9 +209,9 @@ pan_http(struct vsb *vsb, const char *id, const struct http *h)
 		    (int)(h->hd[i].e - h->hd[i].b), h->hd[i].b);
 	}
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -246,7 +228,7 @@ pan_boc(struct vsb *vsb, const struct boc *boc)
 	VSB_printf(vsb, "vary = %p,\n", boc->vary);
 	VSB_printf(vsb, "stevedore_priv = %p,\n", boc->stevedore_priv);
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -262,21 +244,21 @@ pan_objcore(struct vsb *vsb, const char *typ, const struct objcore *oc)
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, oc, OBJCORE_MAGIC);
 	VSB_printf(vsb, "refcnt = %d,\n", oc->refcnt);
-	VSB_printf(vsb, "flags = {");
+	VSB_cat(vsb, "flags = {");
 
 /*lint -save -esym(438,p) -esym(838,p) -e539 */
 	p = "";
 #define OC_FLAG(U, l, v) \
 	if (oc->flags & v) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/oc_flags.h"
-	VSB_printf(vsb, "},\n");
-	VSB_printf(vsb, "exp_flags = {");
+	VSB_cat(vsb, "},\n");
+	VSB_cat(vsb, "exp_flags = {");
 	p = "";
 #define OC_EXP_FLAG(U, l, v) \
 	if (oc->exp_flags & v) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/oc_exp_flags.h"
 /*lint -restore */
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 
 	if (oc->boc != NULL)
 		pan_boc(vsb, oc->boc);
@@ -288,18 +270,18 @@ pan_objcore(struct vsb *vsb, const char *typ, const struct objcore *oc)
 		VSB_printf(vsb, " (%s", oc->stobj->stevedore->name);
 		if (strlen(oc->stobj->stevedore->ident))
 			VSB_printf(vsb, " %s", oc->stobj->stevedore->ident);
-		VSB_printf(vsb, ")");
+		VSB_cat(vsb, ")");
 		if (oc->stobj->stevedore->panic) {
-			VSB_printf(vsb, " {\n");
+			VSB_cat(vsb, " {\n");
 			VSB_indent(vsb, 2);
 			oc->stobj->stevedore->panic(vsb, oc);
 			VSB_indent(vsb, -2);
-			VSB_printf(vsb, "}");
+			VSB_cat(vsb, "}");
 		}
 	}
-	VSB_printf(vsb, ",\n");
+	VSB_cat(vsb, ",\n");
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -319,13 +301,13 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 	pan_ws(vsb, wrk->aws);
 
 	m = wrk->cur_method;
-	VSB_printf(vsb, "VCL::method = ");
+	VSB_cat(vsb, "VCL::method = ");
 	if (m == 0) {
-		VSB_printf(vsb, "none,\n");
+		VSB_cat(vsb, "none,\n");
 		return;
 	}
 	if (!(m & 1))
-		VSB_printf(vsb, "inside ");
+		VSB_cat(vsb, "inside ");
 	m &= ~1;
 	hand = VCL_Method_Name(m);
 	if (hand != NULL)
@@ -338,7 +320,7 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 		VSB_printf(vsb, "VCL::return = %s,\n", hand);
 	else
 		VSB_printf(vsb, "VCL::return = 0x%x,\n", wrk->handling);
-	VSB_printf(vsb, "VCL::methods = {");
+	VSB_cat(vsb, "VCL::methods = {");
 	m = wrk->seen_methods;
 	p = "";
 	for (u = 1; m ; u <<= 1) {
@@ -348,9 +330,9 @@ pan_wrk(struct vsb *vsb, const struct worker *wrk)
 			p = ", ";
 		}
 	}
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 static void
@@ -368,7 +350,7 @@ pan_vfp(struct vsb *vsb, const struct vfp_ctx *vfc)
 	VSB_printf(vsb, "oc = %p,\n", vfc->oc);
 
 	if (!VTAILQ_EMPTY(&vfc->vfp)) {
-		VSB_printf(vsb, "filters = {\n");
+		VSB_cat(vsb, "filters = {\n");
 		VSB_indent(vsb, 2);
 		VTAILQ_FOREACH(vfe, &vfc->vfp, list) {
 			VSB_printf(vsb, "%s = %p {\n", vfe->vfp->name, vfe);
@@ -377,15 +359,15 @@ pan_vfp(struct vsb *vsb, const struct vfp_ctx *vfc)
 			VSB_printf(vsb, "priv2 = %zd,\n", vfe->priv2);
 			VSB_printf(vsb, "closed = %d\n", vfe->closed);
 			VSB_indent(vsb, -2);
-			VSB_printf(vsb, "},\n");
+			VSB_cat(vsb, "},\n");
 		}
 		VSB_indent(vsb, -2);
-		VSB_printf(vsb, "},\n");
+		VSB_cat(vsb, "},\n");
 	}
 
 	VSB_printf(vsb, "obj_flags = 0x%x,\n", vfc->obj_flags);
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 static void
@@ -400,17 +382,17 @@ pan_vdp(struct vsb *vsb, const struct vdp_ctx *vdc)
 	VSB_printf(vsb, "retval = %d,\n", vdc->retval);
 
 	if (!VTAILQ_EMPTY(&vdc->vdp)) {
-		VSB_printf(vsb, "filters = {\n");
+		VSB_cat(vsb, "filters = {\n");
 		VSB_indent(vsb, 2);
 		VTAILQ_FOREACH(vde, &vdc->vdp, list)
 			VSB_printf(vsb, "%s = %p { priv = %p }\n",
 			    vde->vdp->name, vde, vde->priv);
 		VSB_indent(vsb, -2);
-		VSB_printf(vsb, "},\n");
+		VSB_cat(vsb, "},\n");
 	}
 
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 static void
@@ -424,7 +406,7 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 	VSB_indent(vsb, 2);
 	PAN_CheckMagic(vsb, bo, BUSYOBJ_MAGIC);
 	VSB_printf(vsb, "end = %p,\n", bo->end);
-	VSB_printf(vsb, "retries = %d,\n", bo->retries);
+	VSB_printf(vsb, "retries = %u,\n", bo->retries);
 
 	if (bo->req != NULL)
 		pan_req(vsb, bo->req);
@@ -456,25 +438,41 @@ pan_busyobj(struct vsb *vsb, const struct busyobj *bo)
 
 	// fetch_task left out
 
-	VSB_printf(vsb, "flags = {");
+	VSB_cat(vsb, "flags = {");
 	p = "";
 /*lint -save -esym(438,p) -e539 */
 #define BO_FLAG(l, r, w, d) \
 	if (bo->l) { VSB_printf(vsb, "%s" #l, p); p = ", "; }
 #include "tbl/bo_flags.h"
 /*lint -restore */
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 
 	// timeouts/timers/acct/storage left out
 
 	VDI_Panic(bo->director_req, vsb, "director_req");
 	if (bo->director_resp == bo->director_req)
-		VSB_printf(vsb, "director_resp = director_req,\n");
+		VSB_cat(vsb, "director_resp = director_req,\n");
 	else
 		VDI_Panic(bo->director_resp, vsb, "director_resp");
 	VCL_Panic(vsb, "vcl", bo->vcl);
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
+}
+
+/*--------------------------------------------------------------------*/
+
+static void
+pan_top(struct vsb *vsb, const struct reqtop *top)
+{
+	VSB_printf(vsb, "top = %p {\n", top);
+	if (PAN_already(vsb, top))
+		return;
+	VSB_indent(vsb, 2);
+	pan_req(vsb, top->topreq);
+	pan_privs(vsb, top->privs);
+	VCL_Panic(vsb, "vcl0", top->vcl0);
+	VSB_indent(vsb, -2);
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -495,13 +493,13 @@ pan_req(struct vsb *vsb, const struct req *req)
 	    xp == NULL ? "NULL" : xp->name);
 
 	if (xp != NULL && xp->req_panic != NULL) {
-		VSB_printf(vsb, " {\n");
+		VSB_cat(vsb, " {\n");
 		VSB_indent(vsb, 2);
 		xp->req_panic(vsb, req);
 		VSB_indent(vsb, -2);
-		VSB_printf(vsb, "}");
+		VSB_cat(vsb, "}");
 	}
-	VSB_printf(vsb, "\n");
+	VSB_cat(vsb, "\n");
 	switch (req->req_step) {
 #define REQ_STEP(l, u, arg) case R_STP_##u: stp = "R_STP_" #u; break;
 #include "tbl/steps.h"
@@ -513,14 +511,14 @@ pan_req(struct vsb *vsb, const struct req *req)
 		VSB_printf(vsb, "step = 0x%x,\n", req->req_step);
 
 	VSB_printf(vsb, "req_body = %s,\n",
-	    reqbody_status_2str(req->req_body_status));
+	    req->req_body_status ? req->req_body_status->name : "NULL");
 
 	if (req->err_code)
 		VSB_printf(vsb,
 		    "err_code = %d, err_reason = %s,\n", req->err_code,
 		    req->err_reason ? req->err_reason : "(null)");
 
-	VSB_printf(vsb, "restarts = %d, esi_level = %d,\n",
+	VSB_printf(vsb, "restarts = %u, esi_level = %u,\n",
 	    req->restarts, req->esi_level);
 
 	if (req->sp != NULL)
@@ -533,36 +531,32 @@ pan_req(struct vsb *vsb, const struct req *req)
 	if (VALID_OBJ(req->htc, HTTP_CONN_MAGIC))
 		pan_htc(vsb, req->htc);
 	pan_http(vsb, "req", req->http);
-	if (req->resp->ws != NULL)
+	if (req->resp != NULL && req->resp->ws != NULL)
 		pan_http(vsb, "resp", req->resp);
 	if (req->vdc != NULL)
 		pan_vdp(vsb, req->vdc);
 
 	VCL_Panic(vsb, "vcl", req->vcl);
-	VCL_Panic(vsb, "vcl0", req->vcl0);
 
 	if (req->body_oc != NULL)
 		pan_objcore(vsb, "BODY", req->body_oc);
 	if (req->objcore != NULL)
 		pan_objcore(vsb, "REQ", req->objcore);
 
-	VSB_printf(vsb, "flags = {\n");
+	VSB_cat(vsb, "flags = {\n");
 	VSB_indent(vsb, 2);
 #define REQ_FLAG(l, r, w, d) if (req->l) VSB_printf(vsb, #l ",\n");
 #include "tbl/req_flags.h"
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 
 	pan_privs(vsb, req->privs);
 
-	VSB_printf(vsb, "topreq = {\n");
-	VSB_indent(vsb, 2);
-	pan_req(vsb, req->topreq);
-	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	if (req->top != NULL)
+		pan_top(vsb, req->top);
 
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
@@ -588,13 +582,13 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 	VSB_printf(vsb, "transport = %s",
 	    xp == NULL ? "<none>" : xp->name);
 	if (xp != NULL && xp->sess_panic != NULL) {
-		VSB_printf(vsb, " {\n");
+		VSB_cat(vsb, " {\n");
 		VSB_indent(vsb, 2);
 		xp->sess_panic(vsb, sp);
 		VSB_indent(vsb, -2);
-		VSB_printf(vsb, "}");
+		VSB_cat(vsb, "}");
 	}
-	VSB_printf(vsb, "\n");
+	VSB_cat(vsb, "\n");
 	ci = SES_Get_String_Attr(sp, SA_CLIENT_IP);
 	cp = SES_Get_String_Attr(sp, SA_CLIENT_PORT);
 	if (VALID_OBJ(sp->listen_sock, LISTEN_SOCK_MAGIC))
@@ -604,11 +598,52 @@ pan_sess(struct vsb *vsb, const struct sess *sp)
 		VSB_printf(vsb, "client = %s %s <unknown>\n", ci, cp);
 
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
 
+#ifdef WITH_UNWIND
+
+static void
+pan_backtrace(struct vsb *vsb)
+{
+	unw_cursor_t cursor; unw_context_t uc;
+	unw_word_t ip, sp;
+	unw_word_t offp;
+	char fname[1024];
+	int ret;
+
+	VSB_cat(vsb, "Backtrace:\n");
+	VSB_indent(vsb, 2);
+
+	ret = unw_getcontext(&uc);
+	if (ret != 0) {
+		VSB_printf(vsb, "Backtrace not available "
+		    "(unw_getcontext returned %d)\n", ret);
+		return;
+	}
+	ret = unw_init_local(&cursor, &uc);
+	if (ret != 0) {
+		VSB_printf(vsb, "Backtrace not available "
+		    "(unw_init_local returned %d)\n", ret);
+		return;
+	}
+	while (unw_step(&cursor) > 0) {
+		fname[0] = '\0';
+		ip = sp = 0;
+		unw_get_reg(&cursor, UNW_REG_IP, &ip);
+		unw_get_reg(&cursor, UNW_REG_SP, &sp);
+		unw_get_proc_name(&cursor, fname, sizeof(fname), &offp);
+		VSB_printf(vsb, "ip=0x%lx, sp=0x%lx <%s+0x%lx>\n", (long) ip,
+		    (long) sp, fname[0] ? fname : "<unknown>", (long)offp);
+	}
+
+	VSB_indent(vsb, -2);
+}
+
+#else /* WITH_UNWIND */
+
 #define BACKTRACE_LEVELS	10
 
 static void
@@ -626,14 +661,14 @@ pan_backtrace(struct vsb *vsb)
 		VSB_printf(vsb, "Backtrace not available (ret=%zu)\n", size);
 		return;
 	}
-	VSB_printf(vsb, "Backtrace:\n");
+	VSB_cat(vsb, "Backtrace:\n");
 	VSB_indent(vsb, 2);
 	for (i = 0; i < size; i++) {
 		bprintf(buf, "%p", array[i]);
 		VSB_printf(vsb, "%s: ", buf);
 		strings = backtrace_symbols(&array[i], 1);
 		if (strings == NULL || strings[0] == NULL) {
-			VSB_printf(vsb, "(?)");
+			VSB_cat(vsb, "(?)");
 		} else {
 			p = strings[0];
 			if (!memcmp(buf, p, strlen(buf))) {
@@ -645,11 +680,14 @@ pan_backtrace(struct vsb *vsb)
 			}
 			VSB_printf(vsb, "%s", p);
 		}
-		VSB_printf (vsb, "\n");
+		VSB_cat(vsb, "\n");
+		free(strings);
 	}
 	VSB_indent(vsb, -2);
 }
 
+#endif /* WITH_UNWIND */
+
 #ifdef HAVE_PTHREAD_GETATTR_NP
 static void
 pan_threadattr(struct vsb *vsb)
@@ -725,8 +763,8 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		    "Incomplete code in %s(), %s line %d:\n",
 		    func, file, line);
 		break;
-	default:
 	case VAS_ASSERT:
+	default:
 		VSB_printf(pan_vsb,
 		    "Assert error in %s(), %s line %d:\n"
 		    "  Condition(%s) not true.\n",
@@ -734,7 +772,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 		break;
 	}
 	VSB_printf(pan_vsb, "version = %s, vrt api = %u.%u\n",
-	    VCS_version, VRT_MAJOR_VERSION, VRT_MINOR_VERSION);
+	    VCS_String("V"), VRT_MAJOR_VERSION, VRT_MINOR_VERSION);
 	VSB_printf(pan_vsb, "ident = %s,%s\n",
 	    heritage.ident, Waiter_GetName());
 	VSB_printf(pan_vsb, "now = %f (mono), %f (real)\n",
@@ -766,10 +804,9 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 			VSL_Flush(bo->vsl, 0);
 		VMOD_Panic(pan_vsb);
 	} else {
-		VSB_printf(pan_vsb,
-		    "Feature short panic supressed details.\n");
+		VSB_cat(pan_vsb, "Feature short panic supressed details.\n");
 	}
-	VSB_printf(pan_vsb, "\n");
+	VSB_cat(pan_vsb, "\n");
 	VSB_putc(pan_vsb, '\0');	/* NUL termination */
 
 	if (FEATURE(FEATURE_NO_COREDUMP))
@@ -780,7 +817,7 @@ pan_ic(const char *func, const char *file, int line, const char *cond,
 
 /*--------------------------------------------------------------------*/
 
-static void v_matchproto_(cli_func_t)
+static void v_noreturn_ v_matchproto_(cli_func_t)
 ccf_panic(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -788,6 +825,8 @@ ccf_panic(struct cli *cli, const char * const *av, void *priv)
 	(void)av;
 	AZ(priv);
 	AZ(strcmp("", "You asked for it"));
+	/* NOTREACHED */
+	abort();
 }
 
 /*--------------------------------------------------------------------*/
@@ -810,7 +849,7 @@ PAN_Init(void)
 	AN(heritage.panic_str_len);
 	AN(VSB_new(pan_vsb, heritage.panic_str, heritage.panic_str_len,
 	    VSB_FIXEDLEN));
-	VSB_printf(pan_vsb, "This is a test\n");
+	VSB_cat(pan_vsb, "This is a test\n");
 	AZ(VSB_finish(pan_vsb));
 	VSB_clear(pan_vsb);
 	heritage.panic_str[0] = '\0';
