@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -230,7 +232,7 @@ VRY_Prep(struct req *req)
 		AZ(req->vary_b);
 		AZ(req->vary_l);
 		AZ(req->vary_e);
-		(void)WS_Reserve(req->ws, 0);
+		(void)WS_ReserveAll(req->ws);
 	} else {
 		AN(req->ws->r);
 	}
