@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -44,15 +46,16 @@
 #include "cache_vcl.h"
 #include "vcli_serve.h"
 #include "vtim.h"
+#include "vcc_interface.h"
 
-const char * const VCL_TEMP_INIT = "init";
-const char * const VCL_TEMP_COLD = "cold";
-const char * const VCL_TEMP_WARM = "warm";
-const char * const VCL_TEMP_BUSY = "busy";
-const char * const VCL_TEMP_COOLING = "cooling";
+const struct vcltemp VCL_TEMP_INIT[1] = {{ .name = "init", .is_cold = 1 }};
+const struct vcltemp VCL_TEMP_COLD[1] = {{ .name = "cold", .is_cold = 1 }};
+const struct vcltemp VCL_TEMP_WARM[1] = {{ .name = "warm", .is_warm = 1 }};
+const struct vcltemp VCL_TEMP_BUSY[1] = {{ .name = "busy", .is_warm = 1 }};
+const struct vcltemp VCL_TEMP_COOLING[1] = {{ .name = "cooling" }};
 
-// not really a state
-static const char * const VCL_TEMP_LABEL = "label";
+// not really a temperature
+static const struct vcltemp VCL_TEMP_LABEL[1] = {{ .name = "label" }};
 
 /*
  * XXX: Presently all modifications to this list happen from the
@@ -68,6 +71,7 @@ static struct vrt_ctx ctx_cli;
 static unsigned handling_cli;
 static struct ws ws_cli;
 static uintptr_t ws_snapshot_cli;
+static struct vsl_log vsl_cli;
 
 /*--------------------------------------------------------------------*/
 
@@ -77,7 +81,9 @@ VCL_Bo2Ctx(struct vrt_ctx *ctx, struct busyobj *bo)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(bo, BUSYOBJ_MAGIC);
+	CHECK_OBJ_NOTNULL(bo->wrk, WORKER_MAGIC);
 	ctx->vcl = bo->vcl;
+	ctx->syntax = ctx->vcl->conf->syntax;
 	ctx->vsl = bo->vsl;
 	ctx->http_bereq = bo->bereq;
 	ctx->http_beresp = bo->beresp;
@@ -85,6 +91,8 @@ VCL_Bo2Ctx(struct vrt_ctx *ctx, struct busyobj *bo)
 	ctx->sp = bo->sp;
 	ctx->now = bo->t_prev;
 	ctx->ws = bo->ws;
+	ctx->handling = &bo->wrk->handling;
+	*ctx->handling = 0;
 }
 
 void
@@ -93,17 +101,20 @@ VCL_Req2Ctx(struct vrt_ctx *ctx, struct req *req)
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
-	CHECK_OBJ_ORNULL(req->topreq, REQ_MAGIC);
 
 	ctx->vcl = req->vcl;
+	ctx->syntax = ctx->vcl->conf->syntax;
 	ctx->vsl = req->vsl;
 	ctx->http_req = req->http;
-	ctx->http_req_top = req->topreq->http;
+	CHECK_OBJ_NOTNULL(req->top, REQTOP_MAGIC);
+	ctx->http_req_top = req->top->topreq->http;
 	ctx->http_resp = req->resp;
 	ctx->req = req;
 	ctx->sp = req->sp;
 	ctx->now = req->t_prev;
 	ctx->ws = req->ws;
+	ctx->handling = &req->wrk->handling;
+	*ctx->handling = 0;
 }
 
 /*--------------------------------------------------------------------*/
@@ -121,54 +132,99 @@ VCL_Get_CliCtx(int msg)
 	if (msg) {
 		ctx_cli.msg = VSB_new_auto();
 		AN(ctx_cli.msg);
+	} else {
+		ctx_cli.vsl = &vsl_cli;
 	}
 	ctx_cli.ws = &ws_cli;
 	WS_Assert(ctx_cli.ws);
 	return (&ctx_cli);
 }
 
-void
+/*
+ * releases CLI ctx
+ *
+ * returns finished error msg vsb if VCL_Get_CliCtx(1) was called
+ *
+ * caller needs to VSB_destroy a non-NULL return value
+ *
+ */
+struct vsb *
 VCL_Rel_CliCtx(struct vrt_ctx **ctx)
 {
+	struct vsb *r = NULL;
 
 	ASSERT_CLI();
 	assert(*ctx == &ctx_cli);
 	AN((*ctx)->handling);
-	if (ctx_cli.msg)
-		VSB_destroy(&ctx_cli.msg);
+	if (ctx_cli.msg) {
+		TAKE_OBJ_NOTNULL(r, &ctx_cli.msg, VSB_MAGIC);
+		AZ(VSB_finish(r));
+	}
+	if (ctx_cli.vsl)
+		VSL_Flush(ctx_cli.vsl, 0);
 	WS_Assert(ctx_cli.ws);
-	WS_Reset(&ws_cli, ws_snapshot_cli);
+	WS_Rollback(&ws_cli, ws_snapshot_cli);
 	INIT_OBJ(*ctx, VRT_CTX_MAGIC);
 	*ctx = NULL;
+
+	return (r);
 }
 
 /*--------------------------------------------------------------------*/
 
 static int
-vcl_send_event(VRT_CTX, enum vcl_event_e ev)
+vcl_send_event(struct vcl *vcl, enum vcl_event_e ev, struct vsb **msg)
 {
-	int r;
+	int r, havemsg;
+	unsigned method = 0;
+	struct vrt_ctx *ctx;
 
 	ASSERT_CLI();
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
-	assert(ev == VCL_EVENT_LOAD ||
-	       ev == VCL_EVENT_WARM ||
-	       ev == VCL_EVENT_COLD ||
-	       ev == VCL_EVENT_DISCARD);
-	assert(ev != VCL_EVENT_LOAD || ctx->method == VCL_MET_INIT);
-	assert(ev != VCL_EVENT_DISCARD || ctx->method == VCL_MET_FINI);
+
+	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	CHECK_OBJ_NOTNULL(vcl->conf, VCL_CONF_MAGIC);
+	AN(msg);
+	AZ(*msg);
+
+	switch (ev) {
+	case VCL_EVENT_LOAD:
+		method = VCL_MET_INIT;
+		/* FALLTHROUGH */
+	case VCL_EVENT_WARM:
+		havemsg = 1;
+		break;
+	case VCL_EVENT_DISCARD:
+		method = VCL_MET_FINI;
+		/* FALLTHROUGH */
+	case VCL_EVENT_COLD:
+		havemsg = 0;
+		break;
+	default:
+		WRONG("vcl_event");
+	}
+
+	ctx = VCL_Get_CliCtx(havemsg);
+
 	AN(ctx->handling);
-	*ctx->handling = 0;
+	AZ(*ctx->handling);
 	AN(ctx->ws);
 
-	if (ev == VCL_EVENT_LOAD || ev == VCL_EVENT_WARM)
-		AN(ctx->msg);
+	ctx->vcl = vcl;
+	ctx->syntax = ctx->vcl->conf->syntax;
+	ctx->method = method;
 
-	VCL_TaskEnter(ctx->vcl, cli_task_privs);
+	VCL_TaskEnter(cli_task_privs);
 	r = ctx->vcl->conf->event_vcl(ctx, ev);
-	VCL_TaskLeave(ctx->vcl, cli_task_privs);
+	VCL_TaskLeave(cli_task_privs);
+
+	/* if the warm event did not get to vcl_init, vcl_fini
+	 * won't be run, so handling may be zero */
+	if (method && *ctx->handling && *ctx->handling != VCL_RET_OK) {
+		assert(ev == VCL_EVENT_LOAD);
+		r = 1;
+	}
+
+	*msg = VCL_Rel_CliCtx(&ctx);
 
 	if (r && (ev == VCL_EVENT_COLD || ev == VCL_EVENT_DISCARD))
 		WRONG("A VMOD cannot fail COLD or DISCARD events");
@@ -198,6 +254,7 @@ vcl_find(const char *name)
 void
 VCL_Panic(struct vsb *vsb, const char *nm, const struct vcl *vcl)
 {
+	const struct vpi_ii *ii;
 	int i;
 
 	AN(vsb);
@@ -210,53 +267,72 @@ VCL_Panic(struct vsb *vsb, const char *nm, const struct vcl *vcl)
 	VSB_printf(vsb, "busy = %u,\n", vcl->busy);
 	VSB_printf(vsb, "discard = %u,\n", vcl->discard);
 	VSB_printf(vsb, "state = %s,\n", vcl->state);
-	VSB_printf(vsb, "temp = %s,\n", (const volatile char *)vcl->temp);
-	VSB_printf(vsb, "conf = {\n");
+	VSB_printf(vsb, "temp = %s,\n", vcl->temp ? vcl->temp->name : "(null)");
+	VSB_cat(vsb, "conf = {\n");
 	VSB_indent(vsb, 2);
 	if (vcl->conf == NULL) {
-		VSB_printf(vsb, "conf = NULL\n");
+		VSB_cat(vsb, "conf = NULL\n");
 	} else {
 		PAN_CheckMagic(vsb, vcl->conf, VCL_CONF_MAGIC);
 		VSB_printf(vsb, "syntax = \"%u\",\n", vcl->conf->syntax);
-		VSB_printf(vsb, "srcname = {\n");
+		VSB_cat(vsb, "srcname = {\n");
 		VSB_indent(vsb, 2);
 		for (i = 0; i < vcl->conf->nsrc; ++i)
 			VSB_printf(vsb, "\"%s\",\n", vcl->conf->srcname[i]);
 		VSB_indent(vsb, -2);
-		VSB_printf(vsb, "},\n");
+		VSB_cat(vsb, "instances = {\n");
+		VSB_indent(vsb, 2);
+		ii = vcl->conf->instance_info;
+		while (ii != NULL && ii->p != NULL) {
+			VSB_printf(vsb, "\"%s\" = %p,\n", ii->name,
+			    (const void *)*(const uintptr_t *)ii->p);
+			ii++;
+		}
+		VSB_indent(vsb, -2);
+		VSB_cat(vsb, "},\n");
 	}
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 	VSB_indent(vsb, -2);
-	VSB_printf(vsb, "},\n");
+	VSB_cat(vsb, "},\n");
 }
 
 /*--------------------------------------------------------------------*/
 
 void
-vcl_get(struct vcl **vcc, struct vcl *vcl)
+VCL_Update(struct vcl **vcc, struct vcl *vcl)
 {
+	struct vcl *old;
+
 	AN(vcc);
 
+	old = *vcc;
+	*vcc = NULL;
+
+	CHECK_OBJ_ORNULL(old, VCL_MAGIC);
+
 	Lck_Lock(&vcl_mtx);
+	if (old != NULL) {
+		assert(old->busy > 0);
+		old->busy--;
+	}
+
 	if (vcl == NULL)
 		vcl = vcl_active; /* Sample vcl_active under lock to avoid
 				   * race */
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
 	if (vcl->label == NULL) {
-		AN(strcmp(vcl->state, VCL_TEMP_LABEL));
+		AN(strcmp(vcl->state, VCL_TEMP_LABEL->name));
 		*vcc = vcl;
 	} else {
-		AZ(strcmp(vcl->state, VCL_TEMP_LABEL));
+		AZ(strcmp(vcl->state, VCL_TEMP_LABEL->name));
 		*vcc = vcl->label;
 	}
 	CHECK_OBJ_NOTNULL(*vcc, VCL_MAGIC);
 	AZ((*vcc)->discard);
 	(*vcc)->busy++;
 	Lck_Unlock(&vcl_mtx);
-	AZ(errno=pthread_rwlock_rdlock(&(*vcc)->temp_rwl));
-	assert(VCL_WARM(*vcc));
-	AZ(errno=pthread_rwlock_unlock(&(*vcc)->temp_rwl));
+	assert((*vcc)->temp->is_warm);
 }
 
 /*--------------------------------------------------------------------*/
@@ -268,6 +344,7 @@ vcl_iterdir(struct cli *cli, const char *pat, const struct vcl *vcl,
 	int i, found = 0;
 	struct vcldir *vdir;
 
+	Lck_AssertHeld(&vcl_mtx);
 	VTAILQ_FOREACH(vdir, &vcl->director_list, list) {
 		if (fnmatch(pat, vdir->cli_name, 0))
 			continue;
@@ -333,8 +410,10 @@ vcl_BackendEvent(const struct vcl *vcl, enum vcl_event_e e)
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
 	AZ(vcl->busy);
 
+	Lck_Lock(&vcl_mtx);
 	VTAILQ_FOREACH(vdir, &vcl->director_list, list)
 		VDI_Event(vdir->dir, e);
+	Lck_Unlock(&vcl_mtx);
 }
 
 static void
@@ -345,6 +424,7 @@ vcl_KillBackends(struct vcl *vcl)
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
 	AZ(vcl->busy);
 	assert(VTAILQ_EMPTY(&vcl->ref_list));
+	Lck_Lock(&vcl_mtx);
 	while (1) {
 		vdir = VTAILQ_FIRST(&vcl->director_list);
 		if (vdir == NULL)
@@ -356,6 +436,7 @@ vcl_KillBackends(struct vcl *vcl)
 		FREE_OBJ(vdir->dir);
 		FREE_OBJ(vdir);
 	}
+	Lck_Unlock(&vcl_mtx);
 }
 
 /*--------------------------------------------------------------------*/
@@ -377,22 +458,23 @@ VCL_Open(const char *fn, struct vsb *msg)
 #endif
 	dlh = dlopen(fn, RTLD_NOW | RTLD_LOCAL);
 	if (dlh == NULL) {
-		VSB_printf(msg, "Could not load compiled VCL.\n");
+		VSB_cat(msg, "Could not load compiled VCL.\n");
 		VSB_printf(msg, "\tdlopen() = %s\n", dlerror());
 		return (NULL);
 	}
 	cnf = dlsym(dlh, "VCL_conf");
 	if (cnf == NULL) {
-		VSB_printf(msg, "Compiled VCL lacks metadata.\n");
+		VSB_cat(msg, "Compiled VCL lacks metadata.\n");
 		(void)dlclose(dlh);
 		return (NULL);
 	}
 	if (cnf->magic != VCL_CONF_MAGIC) {
-		VSB_printf(msg, "Compiled VCL has mangled metadata.\n");
+		VSB_cat(msg, "Compiled VCL has mangled metadata.\n");
 		(void)dlclose(dlh);
 		return (NULL);
 	}
-	if (cnf->syntax < heritage.min_vcl || cnf->syntax > heritage.max_vcl) {
+	if (cnf->syntax < heritage.min_vcl_version ||
+	    cnf->syntax > heritage.max_vcl_version) {
 		VSB_printf(msg, "Compiled VCL version (%.1f) not supported.\n",
 		    .1 * cnf->syntax);
 		(void)dlclose(dlh);
@@ -400,7 +482,6 @@ VCL_Open(const char *fn, struct vsb *msg)
 	}
 	ALLOC_OBJ(vcl, VCL_MAGIC);
 	AN(vcl);
-	AZ(errno=pthread_rwlock_init(&vcl->temp_rwl, NULL));
 	vcl->dlh = dlh;
 	vcl->conf = cnf;
 	return (vcl);
@@ -417,12 +498,11 @@ VCL_Close(struct vcl **vclp)
 	assert(VTAILQ_EMPTY(&vcl->vfps));
 	assert(VTAILQ_EMPTY(&vcl->vdps));
 	AZ(dlclose(vcl->dlh));
-	AZ(errno=pthread_rwlock_destroy(&vcl->temp_rwl));
 	FREE_OBJ(vcl);
 }
 
 /*--------------------------------------------------------------------
- * NB: This function is called from the test-load subprocess.
+ * NB: This function is called in/from the test-load subprocess.
  */
 
 int
@@ -448,50 +528,48 @@ VCL_TestLoad(const char *fn)
 
 /*--------------------------------------------------------------------*/
 
-static void
-vcl_print_refs(VRT_CTX)
+static struct vsb *
+vcl_print_refs(const struct vcl *vcl)
 {
-	struct vcl *vcl;
+	struct vsb *msg;
 	struct vclref *ref;
 
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
-	AN(ctx->msg);
-	vcl = ctx->vcl;
-	VSB_printf(ctx->msg, "VCL %s is waiting for:", vcl->loaded_name);
+	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	msg = VSB_new_auto();
+
+	VSB_printf(msg, "VCL %s is waiting for:", vcl->loaded_name);
 	Lck_Lock(&vcl_mtx);
-	VTAILQ_FOREACH(ref, &ctx->vcl->ref_list, list)
-		VSB_printf(ctx->msg, "\n\t- %s", ref->desc);
+	VTAILQ_FOREACH(ref, &vcl->ref_list, list)
+		VSB_printf(msg, "\n\t- %s", ref->desc);
 	Lck_Unlock(&vcl_mtx);
+	AZ(VSB_finish(msg));
+	return (msg);
 }
 
 static int
-vcl_set_state(VRT_CTX, const char *state)
+vcl_set_state(struct vcl *vcl, const char *state, struct vsb **msg)
 {
-	struct vcl *vcl;
+	struct vsb *nomsg = NULL;
 	int i = 0;
 
 	ASSERT_CLI();
-	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->vcl, VCL_MAGIC);
-	CHECK_OBJ_NOTNULL(ctx->vcl->conf, VCL_CONF_MAGIC);
-	AN(ctx->handling);
-	AN(ctx->vcl);
-	AN(state);
-	assert(ctx->msg != NULL || *state == '0');
 
-	vcl = ctx->vcl;
-	AZ(errno=pthread_rwlock_wrlock(&vcl->temp_rwl));
+	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	AN(state);
+	AN(msg);
+	AZ(*msg);
+
 	AN(vcl->temp);
 
 	switch (state[0]) {
 	case '0':
 		if (vcl->temp == VCL_TEMP_COLD)
 			break;
-		if (vcl->busy == 0 && VCL_WARM(vcl)) {
+		if (vcl->busy == 0 && vcl->temp->is_warm) {
 			vcl->temp = VTAILQ_EMPTY(&vcl->ref_list) ?
 			    VCL_TEMP_COLD : VCL_TEMP_COOLING;
-			AZ(vcl_send_event(ctx, VCL_EVENT_COLD));
+			AZ(vcl_send_event(vcl, VCL_EVENT_COLD, msg));
+			AZ(*msg);
 			vcl_BackendEvent(vcl, VCL_EVENT_COLD);
 		}
 		else if (vcl->busy)
@@ -509,17 +587,18 @@ vcl_set_state(VRT_CTX, const char *state)
 			vcl->temp = VCL_TEMP_WARM;
 		/* The VCL must first reach a stable cold state */
 		else if (vcl->temp == VCL_TEMP_COOLING) {
-			vcl_print_refs(ctx);
+			*msg = vcl_print_refs(vcl);
 			i = -1;
 		}
 		else {
 			vcl->temp = VCL_TEMP_WARM;
-			i = vcl_send_event(ctx, VCL_EVENT_WARM);
+			i = vcl_send_event(vcl, VCL_EVENT_WARM, msg);
 			if (i == 0) {
 				vcl_BackendEvent(vcl, VCL_EVENT_WARM);
 				break;
 			}
-			AZ(vcl->conf->event_vcl(ctx, VCL_EVENT_COLD));
+			AZ(vcl_send_event(vcl, VCL_EVENT_COLD, &nomsg));
+			AZ(nomsg);
 			vcl->temp = VCL_TEMP_COLD;
 		}
 		break;
@@ -528,53 +607,56 @@ vcl_set_state(VRT_CTX, const char *state)
 	}
 	if (i == 0 && state[1])
 		bprintf(vcl->state, "%s", state + 1);
-	AZ(errno=pthread_rwlock_unlock(&vcl->temp_rwl));
 
 	return (i);
 }
 
 static void
-vcl_cancel_load(struct vrt_ctx *ctx, struct cli *cli,
+vcl_cancel_load(struct vcl *vcl, struct cli *cli, struct vsb *msg,
     const char *name, const char *step)
 {
-	struct vcl *vcl = ctx->vcl;
 
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
 	CHECK_OBJ_NOTNULL(vcl->conf, VCL_CONF_MAGIC);
 
-	AZ(VSB_finish(ctx->msg));
 	VCLI_SetResult(cli, CLIS_CANT);
 	VCLI_Out(cli, "VCL \"%s\" Failed %s", name, step);
-	if (VSB_len(ctx->msg))
-		VCLI_Out(cli, "\nMessage:\n\t%s", VSB_data(ctx->msg));
-	ctx->method = VCL_MET_FINI;
-	AZ(vcl_send_event(ctx, VCL_EVENT_DISCARD));
-	ctx->method = 0;
+	if (VSB_len(msg))
+		VCLI_Out(cli, "\nMessage:\n\t%s", VSB_data(msg));
+	VSB_destroy(&msg);
+
+	AZ(vcl_send_event(vcl, VCL_EVENT_DISCARD, &msg));
+	AZ(msg);
+
 	vcl_KillBackends(vcl);
 	free(vcl->loaded_name);
 	VCL_Close(&vcl);
 }
 
 static void
-vcl_load(struct cli *cli, struct vrt_ctx *ctx,
+vcl_load(struct cli *cli,
     const char *name, const char *fn, const char *state)
 {
 	struct vcl *vcl;
-	int i;
+	struct vsb *msg;
 
 	ASSERT_CLI();
 
 	vcl = vcl_find(name);
 	AZ(vcl);
 
-	vcl = VCL_Open(fn, ctx->msg);
+	msg = VSB_new_auto();
+	vcl = VCL_Open(fn, msg);
+	AZ(VSB_finish(msg));
 	if (vcl == NULL) {
-		AZ(VSB_finish(ctx->msg));
 		VCLI_SetResult(cli, CLIS_PARAM);
-		VCLI_Out(cli, "%s", VSB_data(ctx->msg));
+		VCLI_Out(cli, "%s", VSB_data(msg));
+		VSB_destroy(&msg);
 		return;
 	}
 
+	VSB_destroy(&msg);
+
 	vcl->loaded_name = strdup(name);
 	XXXAN(vcl->loaded_name);
 	VTAILQ_INIT(&vcl->director_list);
@@ -584,24 +666,20 @@ vcl_load(struct cli *cli, struct vrt_ctx *ctx,
 
 	vcl->temp = VCL_TEMP_INIT;
 
-	ctx->vcl = vcl;
+	if (vcl_send_event(vcl, VCL_EVENT_LOAD, &msg)) {
+		vcl_cancel_load(vcl, cli, msg, name, "initialization");
+		return;
+	}
+	VSB_destroy(&msg);
 
-	VSB_clear(ctx->msg);
-	ctx->method = VCL_MET_INIT;
-	i = vcl_send_event(ctx, VCL_EVENT_LOAD);
-	ctx->method = 0;
-	if (i || *ctx->handling != VCL_RET_OK) {
-		vcl_cancel_load(ctx, cli, name, "initialization");
-		return;
-	}
-	assert(*ctx->handling == VCL_RET_OK);
-	VSB_clear(ctx->msg);
-	i = vcl_set_state(ctx, state);
-	if (i) {
+	if (vcl_set_state(vcl, state, &msg)) {
 		assert(*state == '1');
-		vcl_cancel_load(ctx, cli, name, "warmup");
+		vcl_cancel_load(vcl, cli, msg, name, "warmup");
 		return;
 	}
+	if (msg)
+		VSB_destroy(&msg);
+
 	VCLI_Out(cli, "Loaded \"%s\" as \"%s\"", fn , name);
 	VTAILQ_INSERT_TAIL(&vcl_head, vcl, list);
 	Lck_Lock(&vcl_mtx);
@@ -617,29 +695,22 @@ vcl_load(struct cli *cli, struct vrt_ctx *ctx,
 void
 VCL_Poll(void)
 {
-	struct vrt_ctx *ctx;
+	struct vsb *nomsg = NULL;
 	struct vcl *vcl, *vcl2;
 
 	ASSERT_CLI();
-	ctx = VCL_Get_CliCtx(0);
 	VTAILQ_FOREACH_SAFE(vcl, &vcl_head, list, vcl2) {
 		if (vcl->temp == VCL_TEMP_BUSY ||
-		    vcl->temp == VCL_TEMP_COOLING) {
-			ctx->vcl = vcl;
-			ctx->syntax = ctx->vcl->conf->syntax;
-			ctx->method = 0;
-			(void)vcl_set_state(ctx, "0");
-		}
+		    vcl->temp == VCL_TEMP_COOLING)
+			AZ(vcl_set_state(vcl, "0", &nomsg));
+		AZ(nomsg);
 		if (vcl->discard && vcl->temp == VCL_TEMP_COLD) {
 			AZ(vcl->busy);
 			assert(vcl != vcl_active);
 			assert(VTAILQ_EMPTY(&vcl->ref_list));
 			VTAILQ_REMOVE(&vcl_head, vcl, list);
-			ctx->vcl = vcl;
-			ctx->syntax = ctx->vcl->conf->syntax;
-			ctx->method = VCL_MET_FINI;
-			AZ(vcl_send_event(ctx, VCL_EVENT_DISCARD));
-			ctx->method = 0;
+			AZ(vcl_send_event(vcl, VCL_EVENT_DISCARD, &nomsg));
+			AZ(nomsg);
 			vcl_KillBackends(vcl);
 			free(vcl->loaded_name);
 			VCL_Close(&vcl);
@@ -647,7 +718,6 @@ VCL_Poll(void)
 			VSC_C_main->n_vcl_discard--;
 		}
 	}
-	VCL_Rel_CliCtx(&ctx);
 }
 
 /*--------------------------------------------------------------------*/
@@ -673,8 +743,8 @@ vcl_cli_list(struct cli *cli, const char * const *av, void *priv)
 			flg = "discarded";
 		} else
 			flg = "available";
-		VSB_printf(vsb, "%s\t%s\t%s\t%6u\t%s",
-		    flg, vcl->state, vcl->temp, vcl->busy, vcl->loaded_name);
+		VSB_printf(vsb, "%s\t%s\t%s\t%6u\t%s", flg, vcl->state,
+		    vcl->temp->name, vcl->busy, vcl->loaded_name);
 		if (vcl->label != NULL) {
 			VSB_printf(vsb, "\t->\t%s", vcl->label->loaded_name);
 			if (vcl->nrefs)
@@ -709,7 +779,7 @@ vcl_cli_list_json(struct cli *cli, const char * const *av, void *priv)
 		} else
 			VCLI_Out(cli, "\"available\",\n");
 		VCLI_Out(cli, "\"state\": \"%s\",\n", vcl->state);
-		VCLI_Out(cli, "\"temperature\": \"%s\",\n", vcl->temp);
+		VCLI_Out(cli, "\"temperature\": \"%s\",\n", vcl->temp->name);
 		VCLI_Out(cli, "\"busy\": %u,\n", vcl->busy);
 		VCLI_Out(cli, "\"name\": \"%s\"", vcl->loaded_name);
 		if (vcl->label != NULL) {
@@ -738,36 +808,38 @@ vcl_cli_list_json(struct cli *cli, const char * const *av, void *priv)
 static void v_matchproto_(cli_func_t)
 vcl_cli_load(struct cli *cli, const char * const *av, void *priv)
 {
-	struct vrt_ctx *ctx;
 
 	AZ(priv);
 	ASSERT_CLI();
-	ctx = VCL_Get_CliCtx(1);
-	vcl_load(cli, ctx, av[2], av[3], av[4]);
-	VCL_Rel_CliCtx(&ctx);
+	// XXX move back code from vcl_load?
+	vcl_load(cli, av[2], av[3], av[4]);
 }
 
 static void v_matchproto_(cli_func_t)
 vcl_cli_state(struct cli *cli, const char * const *av, void *priv)
 {
-	struct vrt_ctx *ctx;
+	struct vcl *vcl;
+	struct vsb *msg = NULL;
 
 	AZ(priv);
 	ASSERT_CLI();
 	AN(av[2]);
 	AN(av[3]);
-	ctx = VCL_Get_CliCtx(1);
-	ctx->vcl = vcl_find(av[2]);
-	AN(ctx->vcl);			// MGT ensures this
-	if (vcl_set_state(ctx, av[3])) {
-		AZ(VSB_finish(ctx->msg));
+
+	vcl = vcl_find(av[2]);
+	AN(vcl);
+
+	if (vcl_set_state(vcl, av[3], &msg)) {
+		CHECK_OBJ_NOTNULL(msg, VSB_MAGIC);
+
 		VCLI_SetResult(cli, CLIS_CANT);
-		VCLI_Out(cli, "Failed <vcl.state %s %s>", ctx->vcl->loaded_name,
+		VCLI_Out(cli, "Failed <vcl.state %s %s>", vcl->loaded_name,
 		    av[3] + 1);
-		if (VSB_len(ctx->msg))
-			VCLI_Out(cli, "\nMessage:\n\t%s", VSB_data(ctx->msg));
+		if (VSB_len(msg))
+			VCLI_Out(cli, "\nMessage:\n\t%s", VSB_data(msg));
 	}
-	VCL_Rel_CliCtx(&ctx);
+	if (msg)
+		VSB_destroy(&msg);
 }
 
 static void v_matchproto_(cli_func_t)
@@ -787,16 +859,15 @@ vcl_cli_discard(struct cli *cli, const char * const *av, void *priv)
 	VSC_C_main->n_vcl_avail--;
 	vcl->discard = 1;
 	if (vcl->label != NULL) {
-		AZ(strcmp(vcl->state, VCL_TEMP_LABEL));
+		AZ(strcmp(vcl->state, VCL_TEMP_LABEL->name));
 		vcl->label->nlabels--;
 		vcl->label= NULL;
 	}
 	Lck_Unlock(&vcl_mtx);
 
-	if (!strcmp(vcl->state, VCL_TEMP_LABEL)) {
+	if (!strcmp(vcl->state, VCL_TEMP_LABEL->name)) {
 		VTAILQ_REMOVE(&vcl_head, vcl, list);
 		free(vcl->loaded_name);
-		AZ(errno=pthread_rwlock_destroy(&vcl->temp_rwl));
 		FREE_OBJ(vcl);
 	} else if (vcl->temp == VCL_TEMP_COLD) {
 		VCL_Poll();
@@ -818,10 +889,9 @@ vcl_cli_label(struct cli *cli, const char * const *av, void *priv)
 	if (lbl == NULL) {
 		ALLOC_OBJ(lbl, VCL_MAGIC);
 		AN(lbl);
-		bprintf(lbl->state, "%s", VCL_TEMP_LABEL);
+		bprintf(lbl->state, "%s", VCL_TEMP_LABEL->name);
 		lbl->temp = VCL_TEMP_WARM;
 		REPLACE(lbl->loaded_name, av[2]);
-		AZ(errno=pthread_rwlock_init(&lbl->temp_rwl, NULL));
 		VTAILQ_INSERT_TAIL(&vcl_head, lbl, list);
 	}
 	if (lbl->label != NULL)
@@ -876,6 +946,11 @@ vcl_cli_show(struct cli *cli, const char * const *av, void *priv)
 		return;
 	}
 	CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+	if (vcl->label) {
+		vcl = vcl->label;
+		CHECK_OBJ_NOTNULL(vcl, VCL_MAGIC);
+		AZ(vcl->label);
+	}
 	CHECK_OBJ_NOTNULL(vcl->conf, VCL_CONF_MAGIC);
 	if (verbose) {
 		for (i = 0; i < vcl->conf->nsrc; i++)
@@ -911,4 +986,5 @@ VCL_Init(void)
 	ws_snapshot_cli = WS_Snapshot(&ws_cli);
 	CLI_AddFuncs(vcl_cmds);
 	Lck_New(&vcl_mtx, lck_vcl);
+	VSL_Setup(&vsl_cli, NULL, 0);
 }
