@@ -6,6 +6,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -85,8 +87,8 @@ struct heritage {
 
 	struct vsmw			*proc_vsmw;
 
-	unsigned			min_vcl;
-	unsigned			max_vcl;
+	unsigned			min_vcl_version;
+	unsigned			max_vcl_version;
 };
 
 extern struct heritage heritage;
