@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -81,7 +83,7 @@ mcf_banner(struct cli *cli, const char *const *av, void *priv)
 	VCLI_Out(cli, "Varnish Cache CLI 1.0\n");
 	VCLI_Out(cli, "-----------------------------\n");
 	VCLI_Out(cli, "%s\n", VSB_data(vident) + 1);
-	VCLI_Out(cli, "%s\n", VCS_version);
+	VCLI_Out(cli, "%s\n", VCS_String("V"));
 	VCLI_Out(cli, "\n");
 	VCLI_Out(cli, "Type 'help' for command list.\n");
 	VCLI_Out(cli, "Type 'quit' to close CLI session.\n");
@@ -99,7 +101,7 @@ static struct cli_proto cli_proto[] = {
 
 /*--------------------------------------------------------------------*/
 
-static void v_matchproto_(cli_func_t)
+static void v_noreturn_ v_matchproto_(cli_func_t)
 mcf_panic(struct cli *cli, const char * const *av, void *priv)
 {
 
@@ -107,6 +109,8 @@ mcf_panic(struct cli *cli, const char * const *av, void *priv)
 	(void)av;
 	(void)priv;
 	AZ(strcmp("", "You asked for it"));
+	/* NOTREACHED */
+	abort();
 }
 
 static struct cli_proto cli_debug[] = {
@@ -122,7 +126,6 @@ mcf_askchild(struct cli *cli, const char * const *av, void *priv)
 	int i;
 	char *q;
 	unsigned u;
-	struct vsb *vsb;
 
 	(void)priv;
 	/*
@@ -137,22 +140,19 @@ mcf_askchild(struct cli *cli, const char * const *av, void *priv)
 		    "Type 'help' for more info.");
 		return;
 	}
-	vsb = VSB_new_auto();
+	VSB_clear(cli_buf);
 	for (i = 1; av[i] != NULL; i++) {
-		VSB_quote(vsb, av[i], strlen(av[i]), 0);
-		VSB_putc(vsb, ' ');
+		VSB_quote(cli_buf, av[i], strlen(av[i]), 0);
+		VSB_putc(cli_buf, ' ');
 	}
-	VSB_putc(vsb, '\n');
-	AZ(VSB_finish(vsb));
-	i = write(cli_o, VSB_data(vsb), VSB_len(vsb));
-	if (i != VSB_len(vsb)) {
-		VSB_destroy(&vsb);
+	VSB_putc(cli_buf, '\n');
+	AZ(VSB_finish(cli_buf));
+	if (VSB_tofile(cli_buf, cli_o)) {
 		VCLI_SetResult(cli, CLIS_COMMS);
 		VCLI_Out(cli, "CLI communication error");
 		MCH_Cli_Fail();
 		return;
 	}
-	VSB_destroy(&vsb);
 	if (VCLI_ReadResult(cli_i, &u, &q, mgt_param.cli_timeout))
 		MCH_Cli_Fail();
 	VCLI_SetResult(cli, u);
@@ -176,24 +176,18 @@ static struct cli_proto cli_askchild[] = {
 int
 mgt_cli_askchild(unsigned *status, char **resp, const char *fmt, ...)
 {
-	int i, j;
+	int i;
 	va_list ap;
 	unsigned u;
 
-	if (cli_buf == NULL) {
-		cli_buf = VSB_new_auto();
-		AN(cli_buf);
-	} else {
-		VSB_clear(cli_buf);
-	}
+	AN(status);
+	VSB_clear(cli_buf);
 
 	if (resp != NULL)
 		*resp = NULL;
-	if (status != NULL)
-		*status = 0;
+	*status = 0;
 	if (cli_i < 0 || cli_o < 0) {
-		if (status != NULL)
-			*status = CLIS_CANT;
+		*status = CLIS_CANT;
 		return (CLIS_CANT);
 	}
 	va_start(ap, fmt);
@@ -202,10 +196,8 @@ mgt_cli_askchild(unsigned *status, char **resp, const char *fmt, ...)
 	AZ(VSB_finish(cli_buf));
 	i = VSB_len(cli_buf);
 	assert(i > 0 && VSB_data(cli_buf)[i - 1] == '\n');
-	j = write(cli_o, VSB_data(cli_buf), i);
-	if (j != i) {
-		if (status != NULL)
-			*status = CLIS_COMMS;
+	if (VSB_tofile(cli_buf, cli_o)) {
+		*status = CLIS_COMMS;
 		if (resp != NULL)
 			*resp = strdup("CLI communication error");
 		MCH_Cli_Fail();
@@ -214,9 +206,8 @@ mgt_cli_askchild(unsigned *status, char **resp, const char *fmt, ...)
 
 	if (VCLI_ReadResult(cli_i, &u, resp, mgt_param.cli_timeout))
 		MCH_Cli_Fail();
-	if (status != NULL)
-		*status = u;
-	return (u == CLIS_OK ? 0 : u);
+	*status = u;
+	return (u == CLIS_OK || u == CLIS_TRUNCATED ? 0 : u);
 }
 
 /*--------------------------------------------------------------------*/
@@ -247,16 +238,16 @@ mgt_cli_stop_child(void)
 static void
 mgt_cli_challenge(struct cli *cli)
 {
-	int i;
+	size_t z;
 	uint8_t u;
 
 	AZ(VRND_RandomCrypto(cli->challenge, sizeof cli->challenge - 2));
-	for (i = 0; i < (sizeof cli->challenge) - 2; i++) {
+	for (z = 0; z < (sizeof cli->challenge) - 2; z++) {
 		AZ(VRND_RandomCrypto(&u, sizeof u));
-		cli->challenge[i] = (u % 26) + 'a';
+		cli->challenge[z] = (u % 26) + 'a';
 	}
-	cli->challenge[i++] = '\n';
-	cli->challenge[i] = '\0';
+	cli->challenge[z++] = '\n';
+	cli->challenge[z] = '\0';
 	VCLI_Out(cli, "%s", cli->challenge);
 	VCLI_Out(cli, "\nAuthentication required.\n");
 	VCLI_SetResult(cli, CLIS_AUTH);
@@ -365,12 +356,13 @@ mgt_cli_init_cls(void)
 
 	mgt_cls = VCLS_New(NULL);
 	AN(mgt_cls);
-	VCLS_SetLimit(mgt_cls, &mgt_param.cli_limit);
 	VCLS_SetHooks(mgt_cls, mgt_cli_cb_before, mgt_cli_cb_after);
 	VCLS_AddFunc(mgt_cls, MCF_NOAUTH, cli_auth);
 	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_proto);
 	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_debug);
 	VCLS_AddFunc(mgt_cls, MCF_AUTH, cli_askchild);
+	cli_buf = VSB_new_auto();
+	AN(cli_buf);
 }
 
 /*--------------------------------------------------------------------
@@ -692,14 +684,19 @@ void
 mgt_DumpRstCli(void)
 {
 	const struct cli_cmd_desc *cp;
-	int i, j;
+	const char *p;
+	int z;
+	size_t j;
 
 	qsort(cmds, ncmds, sizeof cmds[0], cli_cmp);
-	for (i = 0; i < ncmds; i++, cp++) {
-		cp = cmds[i];
+	for (z = 0; z < ncmds; z++, cp++) {
+		cp = cmds[z];
 		if (!strncmp(cp->request, "debug.", 6))
 			continue;
-		printf(".. _ref_cli_%s:\n\n", cp->syntax);
+		printf(".. _ref_cli_");
+		for (p = cp->request; *p; p++)
+			fputc(*p == '.' ? '_' : *p, stdout);
+		printf(":\n\n");
 		printf("%s\n", cp->syntax);
 		for (j = 0; j < strlen(cp->syntax); j++)
 			printf("~");
