@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -91,7 +93,7 @@ usage(void)
 
 	printf("\nBasic options:\n");
 
-	printf(FMT, "-a address[:port][,proto]",
+	printf(FMT, "-a [<name>=]address[:port][,proto]",
 	    "HTTP listen address and port");
 	printf(FMT, "   [,user=<u>][,group=<g>]",
 	    "Can be specified multiple times.");
@@ -174,7 +176,10 @@ usage(void)
 static void
 cli_check(const struct cli *cli)
 {
-	if (cli->result == CLIS_OK) {
+	if (cli->result == CLIS_OK || cli->result == CLIS_TRUNCATED) {
+		AZ(VSB_finish(cli->sb));
+		if (VSB_len(cli->sb) > 0)
+			fprintf(stderr, "Warnings:\n%s\n", VSB_data(cli->sb));
 		VSB_clear(cli->sb);
 		return;
 	}
@@ -281,7 +286,7 @@ mgt_initialize(struct cli *cli)
 
 	MCF_InitParams(cli);
 
-	VCC_VCL_Range(&heritage.min_vcl, &heritage.max_vcl);
+	VCC_VCL_Range(&heritage.min_vcl_version, &heritage.max_vcl_version);
 
 	cli_check(cli);
 }
@@ -590,15 +595,15 @@ main(int argc, char * const *argv)
 			REPLACE(fa->farg, "<-b argument>");
 			vsb = VSB_new_auto();
 			AN(vsb);
-			VSB_printf(vsb, "vcl 4.1;\n");
-			VSB_printf(vsb, "backend default {\n");
+			VSB_cat(vsb, "vcl 4.1;\n");
+			VSB_cat(vsb, "backend default {\n");
 			if (*optarg != '/')
 				VSB_printf(vsb, "    .host = \"%s\";\n",
 					   optarg);
 			else
 				VSB_printf(vsb, "    .path = \"%s\";\n",
 					   optarg);
-			VSB_printf(vsb, "}\n");
+			VSB_cat(vsb, "}\n");
 			AZ(VSB_finish(vsb));
 			fa->src = strdup(VSB_data(vsb));
 			AN(fa->src);
@@ -701,6 +706,8 @@ main(int argc, char * const *argv)
 		    VSB_data(cli[0].sb));
 	}
 
+	VCLS_SetLimit(mgt_cls, &mgt_param.cli_limit);
+
 	assert(d_flag == 0 || F_flag == 0);
 
 	if (C_flag) {
@@ -710,8 +717,8 @@ main(int argc, char * const *argv)
 			if (getenv("TMPDIR") != NULL)
 				VSB_printf(vsb, "%s", getenv("TMPDIR"));
 			else
-				VSB_printf(vsb, "/tmp");
-			VSB_printf(vsb, "/varnishd_C_XXXXXXX");
+				VSB_cat(vsb, "/tmp");
+			VSB_cat(vsb, "/varnishd_C_XXXXXXX");
 			AZ(VSB_finish(vsb));
 			Cn_arg = strdup(VSB_data(vsb));
 			AN(Cn_arg);
@@ -799,7 +806,9 @@ main(int argc, char * const *argv)
 			fprintf(stderr, "%s\n", VSB_data(cli->sb));
 			VSB_clear(cli->sb);
 		}
-		exit(cli->result == CLIS_OK ? 0 : 2);
+		if (cli->result == CLIS_OK || cli->result == CLIS_TRUNCATED)
+			exit(0);
+		exit(2);
 	} else {
 		while (!VTAILQ_EMPTY(&f_args)) {
 			fa = VTAILQ_FIRST(&f_args);
@@ -843,7 +852,7 @@ main(int argc, char * const *argv)
 	if (pfh2 != NULL)
 		VPF_Write(pfh2);
 
-	MGT_Complain(C_DEBUG, "Version: %s", VCS_version);
+	MGT_Complain(C_DEBUG, "Version: %s", VCS_String("V"));
 	MGT_Complain(C_DEBUG, "Platform: %s", VSB_data(vident) + 1);
 
 	if (d_flag)
