@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -31,96 +33,60 @@
 
 #include "config.h"
 
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <sys/stat.h>
 
 #include "mgt/mgt.h"
+#include "mgt/mgt_vcl.h"
 #include "common/heritage.h"
 
-#include "libvcc.h"
 #include "vcli_serve.h"
 #include "vct.h"
 #include "vev.h"
 #include "vtim.h"
 
-#define VCL_STATE(sym, str)					\
-	static const char * const VCL_STATE_ ## sym = str;
+struct vclstate {
+	const char		*name;
+};
+
+#define VCL_STATE(sym, str)						\
+	static const struct vclstate VCL_STATE_ ## sym[1] = {{ str }};
 #include "tbl/vcl_states.h"
 
-static const char * const VCL_STATE_LABEL = "label";
+static const struct vclstate VCL_STATE_LABEL[1] = {{ "label" }};
 
-static int vcl_count;
+static unsigned vcl_count;
 
-struct vclprog;
-struct vmodfile;
-
-struct vmoddep {
-	unsigned		magic;
-#define VMODDEP_MAGIC		0xc1490542
-	VTAILQ_ENTRY(vmoddep)	lfrom;
-	struct vmodfile		*to;
-	VTAILQ_ENTRY(vmoddep)	lto;
-};
-
-struct vcldep {
-	unsigned		magic;
-#define VCLDEP_MAGIC		0xa9a17dc2
-	struct vclprog		*from;
-	VTAILQ_ENTRY(vcldep)	lfrom;
-	struct vclprog		*to;
-	VTAILQ_ENTRY(vcldep)	lto;
-};
-
-struct vclprog {
-	unsigned		magic;
-#define VCLPROG_MAGIC		0x9ac09fea
-	VTAILQ_ENTRY(vclprog)	list;
-	char			*name;
-	char			*fname;
-	unsigned		warm;
-	const char *		state;
-	double			go_cold;
-	VTAILQ_HEAD(, vcldep)	dfrom;
-	VTAILQ_HEAD(, vcldep)	dto;
-	int			nto;
-	int			loaded;
-	VTAILQ_HEAD(, vmoddep)	vmods;
-};
-
-struct vmodfile {
-	unsigned		magic;
-#define VMODFILE_MAGIC		0xffa1a0d5
-	char			*fname;
-	VTAILQ_ENTRY(vmodfile)	list;
-	VTAILQ_HEAD(, vmoddep)	vcls;
-};
-
-static VTAILQ_HEAD(, vclprog)	vclhead = VTAILQ_HEAD_INITIALIZER(vclhead);
-static VTAILQ_HEAD(, vmodfile)	vmodhead = VTAILQ_HEAD_INITIALIZER(vmodhead);
+struct vclproghead vclhead = VTAILQ_HEAD_INITIALIZER(vclhead);
+struct vmodfilehead vmodhead = VTAILQ_HEAD_INITIALIZER(vmodhead);
 static struct vclprog		*active_vcl;
 static struct vev *e_poker;
 
-static int mgt_vcl_setstate(struct cli *, struct vclprog *, const char *);
+static int mgt_vcl_setstate(struct cli *, struct vclprog *,
+    const struct vclstate *);
+static int mgt_vcl_settemp(struct cli *, struct vclprog *, unsigned);
+static int mgt_vcl_askchild(struct cli *, struct vclprog *, unsigned);
+static void mgt_vcl_set_cooldown(struct vclprog *, vtim_mono);
 
 /*--------------------------------------------------------------------*/
 
-static const char *
-mcf_vcl_parse_state(const char *s)
+static const struct vclstate *
+mcf_vcl_parse_state(struct cli *cli, const char *s)
 {
-	if (s == NULL || *s == '\0')
-		return (NULL);
-#define VCL_STATE(sym, str)					\
-	if (!strcmp(s, VCL_STATE_ ## sym))			\
-		return (VCL_STATE_ ## sym);
+	if (s != NULL) {
+#define VCL_STATE(sym, str)				\
+		if (!strcmp(s, str))			\
+			return (VCL_STATE_ ## sym);
 #include "tbl/vcl_states.h"
+	}
+	VCLI_Out(cli, "State must be one of auto, cold or warm.");
+	VCLI_SetResult(cli, CLIS_PARAM);
 	return (NULL);
 }
 
-static struct vclprog *
+struct vclprog *
 mcf_vcl_byname(const char *name)
 {
 	struct vclprog *vp;
@@ -182,7 +148,7 @@ mcf_find_no_vcl(struct cli *cli, const char *name)
 	return (1);
 }
 
-static int
+int
 mcf_is_label(const struct vclprog *vp)
 {
 	return (vp->state == VCL_STATE_LABEL);
@@ -190,22 +156,27 @@ mcf_is_label(const struct vclprog *vp)
 
 /*--------------------------------------------------------------------*/
 
-static void
+struct vcldep *
 mgt_vcl_dep_add(struct vclprog *vp_from, struct vclprog *vp_to)
 {
 	struct vcldep *vd;
 
 	CHECK_OBJ_NOTNULL(vp_from, VCLPROG_MAGIC);
 	CHECK_OBJ_NOTNULL(vp_to, VCLPROG_MAGIC);
+	assert(vp_to->state != VCL_STATE_COLD);
+
 	ALLOC_OBJ(vd, VCLDEP_MAGIC);
-	XXXAN(vd);
+	AN(vd);
+
+	mgt_vcl_set_cooldown(vp_from, -1);
+	mgt_vcl_set_cooldown(vp_to, -1);
+
 	vd->from = vp_from;
 	VTAILQ_INSERT_TAIL(&vp_from->dfrom, vd, lfrom);
 	vd->to = vp_to;
 	VTAILQ_INSERT_TAIL(&vp_to->dto, vd, lto);
 	vp_to->nto++;
-	assert(vp_to->state == VCL_STATE_WARM ||	/* vcl.label ... */
-	    vp_to->state == VCL_STATE_LABEL);		/* return(vcl(...)) */
+	return (vd);
 }
 
 static void
@@ -216,19 +187,15 @@ mgt_vcl_dep_del(struct vcldep *vd)
 	VTAILQ_REMOVE(&vd->from->dfrom, vd, lfrom);
 	VTAILQ_REMOVE(&vd->to->dto, vd, lto);
 	vd->to->nto--;
-	if (vd->to->nto == 0 && vd->to->state == VCL_STATE_WARM) {
-		vd->to->state = VCL_STATE_AUTO;
-		AZ(vd->to->go_cold);
-		(void)mgt_vcl_setstate(NULL, vd->to, VCL_STATE_AUTO);
-		AN(vd->to->go_cold);
-	}
+	if (vd->to->nto == 0)
+		mgt_vcl_set_cooldown(vd->to, VTIM_mono());
 	FREE_OBJ(vd);
 }
 
 /*--------------------------------------------------------------------*/
 
 static struct vclprog *
-mgt_vcl_add(const char *name, const char *state)
+mgt_vcl_add(const char *name, const struct vclstate *state)
 {
 	struct vclprog *vp;
 
@@ -261,8 +228,10 @@ mgt_vcl_del(struct vclprog *vp)
 	struct vmodfile *vf;
 
 	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
-	while (!VTAILQ_EMPTY(&vp->dto))
-		mgt_vcl_dep_del(VTAILQ_FIRST(&vp->dto));
+	assert(VTAILQ_EMPTY(&vp->dto));
+
+	mgt_vcl_symtab_clean(vp);
+
 	while (!VTAILQ_EMPTY(&vp->dfrom))
 		mgt_vcl_dep_del(VTAILQ_FIRST(&vp->dfrom));
 
@@ -307,90 +276,6 @@ mgt_vcl_del(struct vclprog *vp)
 	FREE_OBJ(vp);
 }
 
-void
-mgt_vcl_depends(struct vclprog *vp1, const char *name)
-{
-	struct vclprog *vp2;
-
-	CHECK_OBJ_NOTNULL(vp1, VCLPROG_MAGIC);
-
-	vp2 = mcf_vcl_byname(name);
-	CHECK_OBJ_NOTNULL(vp2, VCLPROG_MAGIC);
-	mgt_vcl_dep_add(vp1, vp2);
-}
-
-static int
-mgt_vcl_cache_vmod(const char *nm, const char *fm, const char *to)
-{
-	int fi, fo;
-	int ret = 0;
-	ssize_t sz;
-	char buf[BUFSIZ];
-
-	fo = open(to, O_WRONLY | O_CREAT | O_EXCL, 0744);
-	if (fo < 0 && errno == EEXIST)
-		return (0);
-	if (fo < 0) {
-		fprintf(stderr, "Creating copy of vmod %s: %s\n",
-		    nm, vstrerror(errno));
-		return (1);
-	}
-	fi = open(fm, O_RDONLY);
-	if (fi < 0) {
-		fprintf(stderr, "Opening vmod %s from %s: %s\n",
-		    nm, fm, vstrerror(errno));
-		AZ(unlink(to));
-		closefd(&fo);
-		return (1);
-	}
-	while (1) {
-		sz = read(fi, buf, sizeof buf);
-		if (sz == 0)
-			break;
-		if (sz < 0 || sz != write(fo, buf, sz)) {
-			fprintf(stderr, "Copying vmod %s: %s\n",
-			    nm, vstrerror(errno));
-			AZ(unlink(to));
-			ret = 1;
-			break;
-		}
-	}
-	closefd(&fi);
-	AZ(fchmod(fo, 0444));
-	closefd(&fo);
-	return (ret);
-}
-
-void
-mgt_vcl_vmod(struct vclprog *vp, const char *src, const char *dst)
-{
-	struct vmodfile *vf;
-	struct vmoddep *vd;
-
-	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
-	AN(src);
-	AN(dst);
-	assert(!strncmp(dst, "./vmod_cache/", 13));
-
-	VTAILQ_FOREACH(vf, &vmodhead, list)
-		if (!strcmp(vf->fname, dst))
-			break;
-	if (vf == NULL) {
-		ALLOC_OBJ(vf, VMODFILE_MAGIC);
-		AN(vf);
-		REPLACE(vf->fname, dst);
-		AN(vf->fname);
-		VTAILQ_INIT(&vf->vcls);
-		AZ(mgt_vcl_cache_vmod(vp->name, src, dst));
-		VTAILQ_INSERT_TAIL(&vmodhead, vf, list);
-	}
-	ALLOC_OBJ(vd, VMODDEP_MAGIC);
-	AN(vd);
-	vd->to = vf;
-	VTAILQ_INSERT_TAIL(&vp->vmods, vd, lfrom);
-	VTAILQ_INSERT_TAIL(&vf->vcls, vd, lto);
-}
-
 int
 mgt_has_vcl(void)
 {
@@ -398,50 +283,70 @@ mgt_has_vcl(void)
 	return (!VTAILQ_EMPTY(&vclhead));
 }
 
-static unsigned
-mgt_vcl_cooldown(struct vclprog *vp)
+/*
+ * go_cold
+ *
+ * -1: leave alone
+ *  0: timer not started - not currently used
+ * >0: when timer started
+ */
+static void
+mgt_vcl_set_cooldown(struct vclprog *vp, vtim_mono now)
 {
-	double now;
+	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
 
-	if (vp->state != VCL_STATE_AUTO)
-		return (0);
-
-	now = VTIM_mono();
-	if (vp->go_cold > 0 && vp->go_cold + mgt_param.vcl_cooldown < now)
-		return (1);
-
-	if (vp->go_cold == 0 && vp != active_vcl)
+	if (vp == active_vcl ||
+	    vp->state != VCL_STATE_AUTO ||
+	    vp->warm == 0 ||
+	    !VTAILQ_EMPTY(&vp->dto) ||
+	    !VTAILQ_EMPTY(&vp->dfrom))
+		vp->go_cold = -1;
+	else
 		vp->go_cold = now;
-
-	return (0);
 }
 
 static int
-mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const char *vs)
+mgt_vcl_settemp(struct cli *cli, struct vclprog *vp, unsigned warm)
 {
-	unsigned status, warm;
+	int i;
+
+	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
+
+	if (warm == vp->warm)
+		return (0);
+
+	if (vp->state == VCL_STATE_AUTO || vp->state == VCL_STATE_LABEL) {
+		mgt_vcl_set_cooldown(vp, -1);
+		i = mgt_vcl_askchild(cli, vp, warm);
+		mgt_vcl_set_cooldown(vp, VTIM_mono());
+	} else {
+		i = mgt_vcl_setstate(cli, vp,
+		    warm ? VCL_STATE_WARM : VCL_STATE_COLD);
+	}
+
+	return (i);
+}
+
+static int
+mgt_vcl_requirewarm(struct cli *cli, struct vclprog *vp)
+{
+	if (vp->state == VCL_STATE_COLD) {
+		VCLI_SetResult(cli, CLIS_CANT);
+		VCLI_Out(cli, "VCL '%s' is cold - set to auto or warm first",
+		    vp->name);
+		return (1);
+	}
+	return (mgt_vcl_settemp(cli, vp, 1));
+}
+
+static int
+mgt_vcl_askchild(struct cli *cli, struct vclprog *vp, unsigned warm)
+{
+	unsigned status;
 	char *p;
 	int i;
 
-	assert(vs != VCL_STATE_LABEL);
-
-	if (vp == active_vcl || mcf_is_label(vp)) {
-		AN(vp->warm);
-		/* Only the poker sends COLD indiscriminately, ignore it */
-		if (vs == VCL_STATE_COLD)
-			AZ(cli);
-		return (0);
-	}
-
-	if (vs == VCL_STATE_AUTO)
-		vs = (mgt_vcl_cooldown(vp) ? VCL_STATE_COLD : VCL_STATE_WARM);
-	else
-		vp->go_cold = 0;
-
-	warm = (vs == VCL_STATE_WARM ? 1 : 0);
-
-	if (vp->warm == warm)
-		return (0);
+	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
 
 	if (!MCH_Running()) {
 		vp->warm = warm;
@@ -449,7 +354,7 @@ mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const char *vs)
 	}
 
 	i = mgt_cli_askchild(&status, &p, "vcl.state %s %d%s\n",
-	    vp->name, warm, vs);
+	    vp->name, warm, vp->state->name);
 	if (i && cli != NULL) {
 		VCLI_SetResult(cli, status);
 		VCLI_Out(cli, "%s", p);
@@ -457,7 +362,7 @@ mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const char *vs)
 		MGT_Complain(C_ERR,
 		    "Please file ticket: VCL poker problem: "
 		    "'vcl.state %s %d%s' -> %03d '%s'",
-		    vp->name, warm, vp->state, i, p);
+		    vp->name, warm, vp->state->name, i, p);
 	} else {
 		/* Success, update mgt's VCL state to reflect child's
 		   state */
@@ -468,6 +373,47 @@ mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const char *vs)
 	return (i);
 }
 
+static int
+mgt_vcl_setstate(struct cli *cli, struct vclprog *vp, const struct vclstate *vs)
+{
+	unsigned warm;
+	int i;
+	const struct vclstate *os;
+
+	CHECK_OBJ_NOTNULL(vp, VCLPROG_MAGIC);
+
+	assert(vs != VCL_STATE_LABEL);
+
+	if (mcf_is_label(vp)) {
+		AN(vp->warm);
+		/* do not touch labels */
+		return (0);
+	}
+
+	if (vp->state == vs)
+		return (0);
+
+	os = vp->state;
+	vp->state = vs;
+
+	if (vp == active_vcl) {
+		assert (vs == VCL_STATE_WARM || vs == VCL_STATE_AUTO);
+		AN(vp->warm);
+		warm = 1;
+	} else if (vs == VCL_STATE_AUTO) {
+		warm = vp->warm;
+	} else {
+		warm = (vs == VCL_STATE_WARM ? 1 : 0);
+	}
+
+	i = mgt_vcl_askchild(cli, vp, warm);
+	if (i == 0)
+		mgt_vcl_set_cooldown(vp, VTIM_mono());
+	else
+		vp->state = os;
+	return (i);
+}
+
 /*--------------------------------------------------------------------*/
 
 static void
@@ -477,6 +423,7 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 	unsigned status;
 	char *lib, *p;
 	struct vclprog *vp;
+	const struct vclstate *vs;
 	char buf[32];
 
 	AN(cli);
@@ -493,17 +440,14 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 	}
 
 	if (state == NULL)
-		state = VCL_STATE_AUTO;
+		vs = VCL_STATE_AUTO;
 	else
-		state = mcf_vcl_parse_state(state);
+		vs = mcf_vcl_parse_state(cli, state);
 
-	if (state == NULL) {
-		VCLI_Out(cli, "State must be one of auto, cold or warm.");
-		VCLI_SetResult(cli, CLIS_PARAM);
+	if (vs == NULL)
 		return;
-	}
 
-	vp = mgt_vcl_add(vclname, state);
+	vp = mgt_vcl_add(vclname, vs);
 	lib = mgt_VccCompile(cli, vp, vclname, vclsrc, vclsrcfile, C_flag);
 	if (lib == NULL) {
 		mgt_vcl_del(vp);
@@ -516,7 +460,7 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 	if (active_vcl == NULL)
 		active_vcl = vp;
 
-	if (cli->result == CLIS_OK &&
+	if ((cli->result == CLIS_OK || cli->result == CLIS_TRUNCATED) &&
 	    vcl_count > mgt_param.max_vcl &&
 	    mgt_param.max_vcl_handling == 1) {
 		VCLI_Out(cli, "%d VCLs loaded\n", vcl_count);
@@ -528,7 +472,7 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 		return;
 
 	if (mgt_cli_askchild(&status, &p, "vcl.load %s %s %d%s\n",
-	    vp->name, vp->fname, vp->warm, vp->state)) {
+	    vp->name, vp->fname, vp->warm, vp->state->name)) {
 		mgt_vcl_del(vp);
 		VCLI_Out(cli, "%s", p);
 		VCLI_SetResult(cli, status);
@@ -537,8 +481,7 @@ mgt_new_vcl(struct cli *cli, const char *vclname, const char *vclsrc,
 	}
 	free(p);
 
-	if (vp->warm && vp->state == VCL_STATE_AUTO)
-		vp->go_cold = VTIM_mono();
+	mgt_vcl_set_cooldown(vp, VTIM_mono());
 }
 
 /*--------------------------------------------------------------------*/
@@ -556,20 +499,8 @@ mgt_vcl_startup(struct cli *cli, const char *vclsrc, const char *vclname,
 		bprintf(buf, "boot%d", n++);
 		vclname = buf;
 	}
+	active_vcl = NULL;
 	mgt_new_vcl(cli, vclname, vclsrc, origin, NULL, C_flag);
-	active_vcl = mcf_vcl_byname(vclname);
-}
-
-/*--------------------------------------------------------------------*/
-
-void
-mgt_vcl_export_labels(struct vcc *vcc)
-{
-	struct vclprog *vp;
-	VTAILQ_FOREACH(vp, &vclhead, list) {
-		if (mcf_is_label(vp))
-			VCC_Predef(vcc, "VCL_VCL", vp->name);
-	}
 }
 
 /*--------------------------------------------------------------------*/
@@ -584,7 +515,7 @@ mgt_push_vcls(struct cli *cli, unsigned *status, char **p)
 	AN(active_vcl);
 
 	/* The VCL has not been loaded yet, it cannot fail */
-	AZ(mgt_vcl_setstate(cli, active_vcl, VCL_STATE_WARM));
+	(void)cli;
 
 	VTAILQ_FOREACH(vp, &vclhead, list)
 		vp->loaded = 0;
@@ -611,7 +542,8 @@ mgt_push_vcls(struct cli *cli, unsigned *status, char **p)
 			} else {
 				if (mgt_cli_askchild(status, p,
 				    "vcl.load \"%s\" %s %d%s\n",
-				    vp->name, vp->fname, vp->warm, vp->state))
+				    vp->name, vp->fname, vp->warm,
+				    vp->state->name))
 					return (1);
 			}
 			vp->loaded = 1;
@@ -656,7 +588,7 @@ mcf_vcl_load(struct cli *cli, const char * const *av, void *priv)
 static void v_matchproto_(cli_func_t)
 mcf_vcl_state(struct cli *cli, const char * const *av, void *priv)
 {
-	const char *state;
+	const struct vclstate *state;
 	struct vclprog *vp;
 
 	(void)priv;
@@ -670,43 +602,25 @@ mcf_vcl_state(struct cli *cli, const char * const *av, void *priv)
 		return;
 	}
 
-	state = mcf_vcl_parse_state(av[3]);
-	if (state == NULL) {
-		VCLI_Out(cli, "State must be one of auto, cold or warm.");
-		VCLI_SetResult(cli, CLIS_PARAM);
+	state = mcf_vcl_parse_state(cli, av[3]);
+	if (state == NULL)
 		return;
-	}
 
-	if (!VTAILQ_EMPTY(&vp->dto)) {
-		assert(vp->state != VCL_STATE_COLD);
-		if (state == VCL_STATE_COLD) {
+	if (state == VCL_STATE_COLD) {
+		if (!VTAILQ_EMPTY(&vp->dto)) {
+			assert(vp->state != VCL_STATE_COLD);
 			VCLI_Out(cli, "A labeled VCL cannot be set cold");
 			VCLI_SetResult(cli, CLIS_CANT);
 			return;
 		}
-	}
-
-	if (vp->state == state)
-		return;
-
-	if (state == VCL_STATE_AUTO) {
-		vp->state = VCL_STATE_AUTO;
-		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_AUTO);
-	} else if (state == VCL_STATE_COLD) {
 		if (vp == active_vcl) {
 			VCLI_Out(cli, "Cannot set the active VCL cold.");
 			VCLI_SetResult(cli, CLIS_CANT);
 			return;
 		}
-		vp->state = VCL_STATE_AUTO;
-		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_COLD);
-	} else if (state == VCL_STATE_WARM) {
-		if (mgt_vcl_setstate(cli, vp, VCL_STATE_WARM) == 0)
-			vp->state = VCL_STATE_WARM;
-	} else {
-		VCLI_Out(cli, "State must be one of auto, cold or warm.");
-		VCLI_SetResult(cli, CLIS_PARAM);
 	}
+
+	(void)mgt_vcl_setstate(cli, vp, state);
 }
 
 static void v_matchproto_(cli_func_t)
@@ -715,6 +629,7 @@ mcf_vcl_use(struct cli *cli, const char * const *av, void *priv)
 	unsigned status;
 	char *p = NULL;
 	struct vclprog *vp, *vp2;
+	vtim_mono now;
 
 	(void)priv;
 	vp = mcf_find_vcl(cli, av[2]);
@@ -722,22 +637,22 @@ mcf_vcl_use(struct cli *cli, const char * const *av, void *priv)
 		return;
 	if (vp == active_vcl)
 		return;
-	if (mgt_vcl_setstate(cli, vp, VCL_STATE_WARM))
+
+	if (mgt_vcl_requirewarm(cli, vp))
 		return;
+
 	if (MCH_Running() &&
 	    mgt_cli_askchild(&status, &p, "vcl.use %s\n", av[2])) {
 		VCLI_SetResult(cli, status);
 		VCLI_Out(cli, "%s", p);
-		AZ(vp->go_cold);
-		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_AUTO);
 	} else {
 		VCLI_Out(cli, "VCL '%s' now active", av[2]);
 		vp2 = active_vcl;
 		active_vcl = vp;
-		if (vp2 != NULL) {
-			AZ(vp2->go_cold);
-			(void)mgt_vcl_setstate(cli, vp2, VCL_STATE_AUTO);
-		}
+		now = VTIM_mono();
+		mgt_vcl_set_cooldown(vp, now);
+		if (vp2 != NULL)
+			mgt_vcl_set_cooldown(vp2, now);
 	}
 	free(p);
 }
@@ -779,12 +694,14 @@ mcf_vcl_discard(struct cli *cli, const char * const *av, void *priv)
 		}
 		return;
 	}
-	if (mcf_is_label(vp))
+	if (mcf_is_label(vp)) {
 		AN(vp->warm);
-	else
+		vp->warm = 0;
+	} else {
 		(void)mgt_vcl_setstate(cli, vp, VCL_STATE_COLD);
+	}
 	if (MCH_Running()) {
-		assert(vp->state != VCL_STATE_WARM);
+		AZ(vp->warm);
 		if (mgt_cli_askchild(&status, &p, "vcl.discard %s\n", av[2]))
 			assert(status == CLIS_OK || status == CLIS_COMMS);
 		free(p);
@@ -799,6 +716,7 @@ mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 	char *p;
 	struct vclprog *vp;
 	struct vcldep *vd;
+	const struct vclstate *vs;
 	struct vsb *vsb;
 
 	/* NB: Shall generate same output as vcl_cli_list() */
@@ -819,8 +737,8 @@ mcf_vcl_list(struct cli *cli, const char * const *av, void *priv)
 		VTAILQ_FOREACH(vp, &vclhead, list) {
 			VSB_printf(vsb, "%s",
 			    vp == active_vcl ? "active" : "available");
-			VSB_printf(vsb, "\t%s\t%s", vp->state, vp->warm ?
-			    VCL_STATE_WARM : VCL_STATE_COLD);
+			vs = vp->warm ?  VCL_STATE_WARM : VCL_STATE_COLD;
+			VSB_printf(vsb, "\t%s\t%s", vp->state->name, vs->name);
 			VSB_printf(vsb, "\t%6s\t%s", "-", vp->name);
 			if (mcf_is_label(vp)) {
 				vd = VTAILQ_FIRST(&vp->dfrom);
@@ -846,6 +764,7 @@ mcf_vcl_list_json(struct cli *cli, const char * const *av, void *priv)
 	char *p;
 	struct vclprog *vp;
 	struct vcldep *vd;
+	const struct vclstate *vs;
 
 	/* NB: Shall generate same output as vcl_cli_list() */
 
@@ -864,9 +783,9 @@ mcf_vcl_list_json(struct cli *cli, const char * const *av, void *priv)
 			VSB_indent(cli->sb, 2);
 			VCLI_Out(cli, "\"status\": \"%s\",\n",
 			    vp == active_vcl ? "active" : "available");
-			VCLI_Out(cli, "\"state\": \"%s\",\n", vp->state);
-			VCLI_Out(cli, "\"temperature\": \"%s\",\n",
-			    vp->warm ? VCL_STATE_WARM : VCL_STATE_COLD);
+			VCLI_Out(cli, "\"state\": \"%s\",\n", vp->state->name);
+			vs = vp->warm ?  VCL_STATE_WARM : VCL_STATE_COLD;
+			VCLI_Out(cli, "\"temperature\": \"%s\",\n", vs->name);
 			VCLI_Out(cli, "\"name\": \"%s\"", vp->name);
 			if (mcf_is_label(vp)) {
 				vd = VTAILQ_FIRST(&vp->dfrom);
@@ -894,6 +813,27 @@ mcf_vcl_list_json(struct cli *cli, const char * const *av, void *priv)
 	}
 }
 
+static int
+mcf_vcl_check_dag(struct cli *cli, struct vclprog *tree, struct vclprog *target)
+{
+	struct vcldep *vd;
+
+	if (target == tree)
+		return (1);
+	VTAILQ_FOREACH(vd, &tree->dfrom, lfrom) {
+		if (!mcf_vcl_check_dag(cli, vd->to, target))
+			continue;
+		if (mcf_is_label(tree))
+			VCLI_Out(cli, "Label %s points to vcl %s\n",
+			    tree->name, vd->to->name);
+		else
+			VCLI_Out(cli, "Vcl %s uses Label %s\n",
+			    tree->name, vd->to->name);
+		return (1);
+	}
+	return(0);
+}
+
 static void v_matchproto_(cli_func_t)
 mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 {
@@ -906,8 +846,12 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 	(void)priv;
 	if (mcf_invalid_vclname(cli, av[2]))
 		return;
-	if (mcf_invalid_vclname(cli, av[3]))
+	vpl = mcf_vcl_byname(av[2]);
+	if (vpl != NULL && !mcf_is_label(vpl)) {
+		VCLI_SetResult(cli, CLIS_PARAM);
+		VCLI_Out(cli, "%s is not a label", vpl->name);
 		return;
+	}
 	vpt = mcf_find_vcl(cli, av[3]);
 	if (vpt == NULL)
 		return;
@@ -916,39 +860,24 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 		VCLI_Out(cli, "VCL labels cannot point to labels");
 		return;
 	}
-	vpl = mcf_vcl_byname(av[2]);
 	if (vpl != NULL) {
-		if (!mcf_is_label(vpl)) {
-			VCLI_SetResult(cli, CLIS_PARAM);
-			VCLI_Out(cli, "%s is not a label", vpl->name);
-			return;
-		}
-		if (!VTAILQ_EMPTY(&vpl->dfrom) &&
-		    VTAILQ_FIRST(&vpl->dfrom)->to == vpt) {
-			VCLI_SetResult(cli, CLIS_PARAM);
-			VCLI_Out(cli, "VCL '%s' already has label '%s'",
-			    vpt->name, vpl->name);
-			return;
-		}
-		if (!VTAILQ_EMPTY(&vpt->dfrom) &&
-		    !VTAILQ_EMPTY(&vpl->dto)) {
-			VCLI_SetResult(cli, CLIS_PARAM);
-			VCLI_Out(cli, "return(vcl) can only be used from"
-			    " the active VCL.\n\n");
+		if (VTAILQ_FIRST(&vpl->dfrom)->to != vpt &&
+		    mcf_vcl_check_dag(cli, vpt, vpl)) {
 			VCLI_Out(cli,
-			    "Label %s is used in return(vcl) from VCL %s\n",
-			    vpl->name, VTAILQ_FIRST(&vpl->dto)->from->name);
-			VCLI_Out(cli, "and VCL %s also has return(vcl)",
-			    vpt->name);
+			    "Pointing label %s at %s would create a loop",
+			    vpl->name, vpt->name);
+			VCLI_SetResult(cli, CLIS_PARAM);
 			return;
 		}
 	}
 
-	if (mgt_vcl_setstate(cli, vpt, VCL_STATE_WARM))
+	if (mgt_vcl_requirewarm(cli, vpt))
 		return;
-	vpt->state = VCL_STATE_WARM; /* XXX: race with the poker? */
 
 	if (vpl != NULL) {
+		/* potentially fail before we delete the dependency */
+		if (mgt_vcl_requirewarm(cli, vpl))
+			return;
 		mgt_vcl_dep_del(VTAILQ_FIRST(&vpl->dfrom));
 		AN(VTAILQ_EMPTY(&vpl->dfrom));
 	} else {
@@ -956,8 +885,9 @@ mcf_vcl_label(struct cli *cli, const char * const *av, void *priv)
 	}
 
 	AN(vpl);
-	vpl->warm = 1;
-	mgt_vcl_dep_add(vpl, vpt);
+	if (mgt_vcl_requirewarm(cli, vpl))
+		return;
+	(void)mgt_vcl_dep_add(vpl, vpt);
 
 	if (!MCH_Running())
 		return;
@@ -976,13 +906,19 @@ static int v_matchproto_(vev_cb_f)
 mgt_vcl_poker(const struct vev *e, int what)
 {
 	struct vclprog *vp;
+	vtim_mono now;
 
 	(void)e;
 	(void)what;
 	e_poker->timeout = mgt_param.vcl_cooldown * .45;
-	VTAILQ_FOREACH(vp, &vclhead, list)
-		if (mgt_vcl_cooldown(vp))
-			(void)mgt_vcl_setstate(NULL, vp, VCL_STATE_COLD);
+	now = VTIM_mono();
+	VTAILQ_FOREACH(vp, &vclhead, list) {
+		if (vp->go_cold == 0)
+			mgt_vcl_set_cooldown(vp, now);
+		else if (vp->go_cold > 0 &&
+		    vp->go_cold + mgt_param.vcl_cooldown < now)
+			(void)mgt_vcl_settemp(NULL, vp, 0);
+	}
 	return (0);
 }
 
@@ -996,6 +932,7 @@ static struct cli_proto cli_vcl[] = {
 	{ CLICMD_VCL_DISCARD,		"", mcf_vcl_discard },
 	{ CLICMD_VCL_LIST,		"", mcf_vcl_list, mcf_vcl_list_json },
 	{ CLICMD_VCL_LABEL,		"", mcf_vcl_label },
+	{ CLICMD_DEBUG_VCL_SYMTAB,	"d", mcf_vcl_symtab },
 	{ NULL }
 };
 
@@ -1004,16 +941,15 @@ static struct cli_proto cli_vcl[] = {
 static void
 mgt_vcl_atexit(void)
 {
-	struct vclprog *vp;
+	struct vclprog *vp, *vp2;
 
 	if (getpid() != heritage.mgt_pid)
 		return;
 	active_vcl = NULL;
-	do {
-		vp = VTAILQ_FIRST(&vclhead);
-		if (vp != NULL)
-			mgt_vcl_del(vp);
-	} while (vp != NULL);
+	while (!VTAILQ_EMPTY(&vclhead))
+		VTAILQ_FOREACH_SAFE(vp, &vclhead, list, vp2)
+			if (VTAILQ_EMPTY(&vp->dto))
+				mgt_vcl_del(vp);
 }
 
 void
