@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -45,7 +47,7 @@
 /*-------------------------------------------------------------------*/
 
 static struct storage *
-sml_stv_alloc(const struct stevedore *stv, size_t size, int flags)
+sml_stv_alloc(const struct stevedore *stv, ssize_t size, int flags)
 {
 	struct storage *st;
 
@@ -74,7 +76,7 @@ sml_stv_alloc(const struct stevedore *stv, size_t size, int flags)
 		if (size <= cache_param->fetch_chunksize)
 			break;
 
-		size >>= 1;
+		size /= 2;
 	}
 	CHECK_OBJ_ORNULL(st, STORAGE_MAGIC);
 	return (st);
@@ -343,7 +345,7 @@ sml_iterator(struct worker *wrk, struct objcore *oc,
  */
 
 static struct storage *
-objallocwithnuke(struct worker *wrk, const struct stevedore *stv, size_t size,
+objallocwithnuke(struct worker *wrk, const struct stevedore *stv, ssize_t size,
     int flags)
 {
 	struct storage *st = NULL;
@@ -497,8 +499,7 @@ sml_bocdone(struct worker *wrk, struct objcore *oc, struct boc *boc)
 	if (boc->stevedore_priv != NULL) {
 		/* Free any leftovers from Trim */
 		CAST_OBJ_NOTNULL(st, boc->stevedore_priv, STORAGE_MAGIC);
-		boc->stevedore_priv = 0;
-		CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
+		boc->stevedore_priv = NULL;
 		sml_stv_free(stv, st);
 	}
 
