@@ -7,6 +7,8 @@
  * Author: Dag-Erling Smørgrav <des@des.no>
  * Author: Guillaume Quintard <guillaume.quintard@gmail.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -53,6 +55,7 @@
 #include "vas.h"
 #include "vut.h"
 #include "vtim.h"
+#include "vapi/vsig.h"
 
 #define HIST_N 2000		/* how far back we remember */
 #define HIST_RES 100		/* bucket resolution */
@@ -131,8 +134,6 @@ static const struct profile profiles[] = {
 
 static const struct profile *active_profile;
 
-static volatile sig_atomic_t quit = 0;
-
 static void
 update(void)
 {
@@ -141,17 +142,18 @@ update(void)
 	const unsigned n = w * hist_range;
 	unsigned bm[n], bh[n];
 	unsigned max;
-	unsigned i, j, scale;
+	unsigned scale;
+	int i, j;
 	unsigned k, l;
 
 	erase();
 
 	/* Draw horizontal axis */
-	for (i = 0; i < n; ++i)
-		(void)mvaddch(LINES - 2, i, '-');
-	for (k = 0, l = hist_low; k < hist_range; ++k, ++l) {
-		(void)mvaddch(LINES - 2, w * k, '+');
-		mvprintw(LINES - 1, w * k, "|1e%d", l);
+	for (k = 0; k < n; ++k)
+		(void)mvaddch(LINES - 2, k, '-');
+	for (i = 0, j = hist_low; i < hist_range; ++i, ++j) {
+		(void)mvaddch(LINES - 2, w * i, '+');
+		mvprintw(LINES - 1, w * i, "|1e%d", j);
 	}
 
 	if (end_of_file)
@@ -190,11 +192,11 @@ update(void)
 		mvprintw((LINES - 2) - j, 0, "%u_", j * scale);
 
 	/* show them */
-	for (i = 0; i < n; ++i) {
-		for (j = 0; j < bm[i] / scale; ++j)
-			(void)mvaddch((LINES - 3) - j, i, '#');
-		for (; j < (bm[i] + bh[i]) / scale; ++j)
-			(void)mvaddch((LINES - 3) - j, i, '|');
+	for (k = 0; k < n; ++k) {
+		for (l = 0; l < bm[k] / scale; ++l)
+			(void)mvaddch((LINES - 3) - l, k, '#');
+		for (; l < (bm[k] + bh[k]) / scale; ++l)
+			(void)mvaddch((LINES - 3) - l, k, '|');
 	}
 
 	refresh();
@@ -305,7 +307,7 @@ accumulate(struct VSL_data *vsl, struct VSL_transaction * const pt[],
 			i = hist_high * HIST_RES - 1;
 		i -= hist_low * HIST_RES;
 		assert(i >= 0);
-		assert(i < hist_buckets);
+		assert((unsigned)i < hist_buckets);
 
 		AZ(pthread_mutex_lock(&mtx));
 
@@ -384,7 +386,7 @@ do_curses(void *arg)
 	intrflush(stdscr, FALSE);
 	curs_set(0);
 	erase();
-	while (!quit && !vut->last_sighup) {
+	while (!VSIG_int && !VSIG_term && !VSIG_hup) {
 		AZ(pthread_mutex_lock(&mtx));
 		update();
 		AZ(pthread_mutex_unlock(&mtx));
@@ -413,7 +415,6 @@ do_curses(void *arg)
 		case 'Q':
 		case 'q':
 			AZ(raise(SIGINT));
-			quit = 1;
 			break;
 		case '0':
 		case '1':
