@@ -6,6 +6,8 @@
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  * Author: Dag-Erling Smørgrav <des@des.no>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -257,6 +259,18 @@ usage(int status)
 	exit(status);
 }
 
+static int
+key_bindings(void)
+{
+
+#define BINDING_KEY(chr, name, next)	\
+	printf("<%s>" next, name);
+#define BINDING(name, desc)		\
+	printf("\n%s\n\n", desc);
+#include "varnishstat_bindings.h"
+	return (0);
+}
+
 int
 main(int argc, char * const *argv)
 {
@@ -267,6 +281,9 @@ main(int argc, char * const *argv)
 	int has_f = 0;
 	struct vsc *vsc;
 
+	if (argc == 2 && !strcmp(argv[1], "--bindings"))
+		exit(key_bindings());
+
 	vut = VUT_InitProg(argc, argv, &vopt_spec);
 	AN(vut);
 	vd = VSM_New();
