@@ -7,6 +7,8 @@
  * Author: Dag-Erling Smørgrav <des@des.no>
  * Author: Martin Blix Grydeland <martin@varnish-software.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -45,6 +47,7 @@
 #include "miniobj.h"
 #include "vqueue.h"
 #include "vtim.h"
+#include "vapi/vsig.h"
 
 #include "varnishstat.h"
 #include "vcurses.h"
@@ -61,6 +64,12 @@
 
 #define VALUE_MAX		999999999999
 
+enum kb_e {
+#define BINDING(name, desc) KB_ ## name,
+#define BINDING_SIG
+#include "varnishstat_bindings.h"
+};
+
 struct ma {
 	unsigned n, nmax;
 	double acc;
@@ -100,7 +109,7 @@ static const volatile uint64_t *main_cache_hit;
 static const volatile uint64_t *main_cache_miss;
 
 static int l_status, l_bar_t, l_points, l_bar_b, l_info;
-static int colw_name = COLW_NAME_MIN;
+static unsigned colw_name = COLW_NAME_MIN;
 static WINDOW *w_status = NULL;
 static WINDOW *w_bar_t = NULL;
 static WINDOW *w_points = NULL;
@@ -120,6 +129,10 @@ static double t_sample = 0.;
 static double interval = 1.;
 static unsigned vsm_status = 0;
 
+#define NOTIF_MAXLEN 256
+static char notification_message[NOTIF_MAXLEN] = "";
+static vtim_mono notification_eol = 0.0;
+
 static void
 init_hitrate(void)
 {
@@ -435,6 +448,9 @@ draw_status(void)
 	mvwprintw(w_status, 1, 0, "Uptime child: ");
 	running(w_status, up_chld, VSM_WRK_RUNNING);
 
+	if (VTIM_mono() < notification_eol)
+		mvwaddstr(w_status, 2, 0, notification_message);
+
 	if (COLS > 70) {
 		mvwprintw(w_status, 0, getmaxx(w_status) - 37,
 		    "Hitrate n: %8u %8u %8u", hitrate.hr_10.n, hitrate.hr_100.n,
@@ -859,68 +875,92 @@ draw_screen(void)
 static void
 handle_keypress(int ch)
 {
+	enum kb_e kb;
+
 	switch (ch) {
-	case KEY_UP:
-	case 107:  /* k */
+#define BINDING_KEY(chr, name, or)	\
+	case chr:
+#define BINDING(name, desc)		\
+		kb = KB_ ## name;	\
+		break;
+#define BINDING_SIG
+#include "varnishstat_bindings.h"
+	default:
+		return;
+	}
+
+	switch (kb) {
+	case KB_UP:
 		if (current == 0)
 			return;
 		current--;
 		break;
-	case KEY_DOWN:
-	case 106:  /* j */
+	case KB_DOWN:
 		if (current == n_ptarray - 1)
 			return;
 		current++;
 		break;
-	case KEY_PPAGE:
-	case 'b':
+	case KB_PAGEUP:
 		current -= l_points;
 		page_start -= l_points;
 		break;
-	case KEY_NPAGE:
-	case ' ':
+	case KB_PAGEDOWN:
 		current += l_points;
 		if (page_start + l_points < n_ptarray - 1)
 			page_start += l_points;
 		break;
-	case 'd':
+	case KB_TOP:
+		current = 0;
+		break;
+	case KB_BOTTOM:
+		current = n_ptarray - 1;
+		break;
+	case KB_UNSEEN:
 		hide_unseen = 1 - hide_unseen;
 		rebuild = 1;
 		break;
-	case 'e':
+	case KB_SCALE:
 		scale = 1 - scale;
 		rebuild = 1;
 		break;
-	case 'g':
-		current = 0;
-		page_start = 0;
+	case KB_ACCEL:
+		interval += 0.1;
+		(void)snprintf(notification_message, NOTIF_MAXLEN,
+		    "Refresh interval set to %.1f seconds.", interval);
+
+		notification_eol = VTIM_mono() + 1.25;
 		break;
-	case 'G':
-		current = n_ptarray - 1;
-		page_start = (current - l_points) + 1;
+	case KB_DECEL:
+		interval -= 0.1;
+		if (interval < 0.1)
+			interval = 0.1;
+		(void)snprintf(notification_message, NOTIF_MAXLEN,
+		    "Refresh interval set to %.1f seconds.", interval);
+
+		notification_eol = VTIM_mono() + 1.25;
 		break;
-	case 'v':
+	case KB_VERBOSE:
 		verbosity = VSC_ChangeLevel(verbosity, 1);
 		rebuild = 1;
 		break;
-	case 'V':
+	case KB_QUIET:
 		verbosity = VSC_ChangeLevel(verbosity, -1);
 		rebuild = 1;
 		break;
-	case 'q':
+	case KB_QUIT:
 		keep_running = 0;
 		return;
-	case '\003':		/* Ctrl-C */
+	case KB_SIG_INT:
 		AZ(raise(SIGINT));
 		return;
-	case '\024':		/* Ctrl-T */
+	case KB_SAMPLE:
 		sample = 1;
 		return;
-	case '\032':		/* Ctrl-Z */
+	case KB_SIG_TSTP:
 		AZ(raise(SIGTSTP));
 		return;
 	default:
-		return;
+		WRONG("unhandled key binding");
 	}
 
 	update_position();
@@ -980,14 +1020,12 @@ delpt(void *priv, const struct VSC_point *const vpt)
 }
 
 void
-do_curses(struct vsm *vsm, struct vsc *vsc, double delay)
+do_curses(struct vsm *vsm, struct vsc *vsc)
 {
 	long t;
 	int ch;
 	double now;
 
-	interval = delay;
-
 	verbosity = VSC_ChangeLevel(NULL, 0);
 
 	initscr();
@@ -1005,7 +1043,7 @@ do_curses(struct vsm *vsm, struct vsc *vsc, double delay)
 	build_pt_array();
 	init_hitrate();
 
-	while (keep_running) {
+	while (keep_running && !VSIG_int && !VSIG_term && !VSIG_hup) {
 		(void)VSC_Iter(vsc, vsm, NULL, NULL);
 		vsm_status = VSM_Status(vsm);
 		if (vsm_status & (VSM_MGT_RESTARTED|VSM_WRK_RESTARTED))
