@@ -1,9 +1,11 @@
 /*-
- * Copyright (c) 2008-2015 Varnish Software AS
+ * Copyright (c) 2008-2019 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -42,7 +44,6 @@
 
 #include "vct.h"
 #include "vfil.h"
-#include "vgz.h"
 #include "vnum.h"
 #include "vrnd.h"
 #include "vtcp.h"
@@ -70,14 +71,15 @@ extern const struct cmds http_cmds[];
  * SECTION: client-server.macros Macros and automatic behaviour
  *
  * To make things easier in the general case, clients will connect by default
- * to the first Varnish server declared and the -vcl+backend switch of the
- * ``varnish`` command will add all the declared servers as backends.
+ * to a Varnish server called v1. To connect to a different Varnish server, use
+ * '-connect ${vNAME_sock}'.
  *
- * Be careful though, servers will by default listen to the 127.0.0.1 IP and
- * will pick a random port, and publish 3 macros: sNAME_addr, sNAME_port and
- * sNAME_sock, but only once they are started.
- * For 'varnish -vcl+backend' to create the vcl with the correct values, the
- * server must be started first.
+ * The -vcl+backend switch of the ``varnish`` command will add all the declared
+ * servers as backends. Be careful though, servers will by default listen to
+ * the 127.0.0.1 IP and will pick a random port, and publish 3 macros:
+ * sNAME_addr, sNAME_port and sNAME_sock, but only once they are started. For
+ * 'varnish -vcl+backend' to create the vcl with the correct values, the server
+ * must be started first.
  *
  * SECTION: client-server.args Arguments
  *
@@ -212,14 +214,12 @@ synth_body(const char *len, int rnd)
 static void
 http_write(const struct http *hp, int lvl, const char *pfx)
 {
-	ssize_t l;
 
 	AZ(VSB_finish(hp->vsb));
 	vtc_dump(hp->vl, lvl, pfx, VSB_data(hp->vsb), VSB_len(hp->vsb));
-	l = write(hp->fd, VSB_data(hp->vsb), VSB_len(hp->vsb));
-	if (l != VSB_len(hp->vsb))
-		vtc_log(hp->vl, hp->fatal, "Write failed: (%zd vs %zd) %s",
-		    l, VSB_len(hp->vsb), strerror(errno));
+	if (VSB_tofile(hp->vsb, hp->fd))
+		vtc_log(hp->vl, hp->fatal, "Write failed: %s",
+		    strerror(errno));
 }
 
 /**********************************************************************
@@ -467,6 +467,9 @@ http_splitheader(struct http *hp, int req)
 		hh[n++] = p++;
 		while (*p != '\0' && !vct_iscrlf(p, hp->rx_e))
 			p++;
+		if (*p == '\0') {
+			break;
+		}
 		q = p;
 		p = vct_skipcrlf(p, hp->rx_e);
 		*q = '\0';
@@ -670,6 +673,9 @@ http_rxhdr(struct http *hp)
 	l = hp->rx_p - hp->rx_b;
 	vtc_dump(hp->vl, 4, "rxhdr", hp->rx_b, l);
 	vtc_log(hp->vl, 4, "rxhdrlen = %zd", l);
+	if (i < 1)
+		vtc_log(hp->vl, hp->fatal, "HTTP header is incomplete");
+	*hp->rx_p = '\0';
 	hp->body = hp->rx_p;
 }
 
@@ -744,116 +750,22 @@ cmd_http_rxresphdrs(CMD_ARGS)
 		    "Multiple Content-Length headers.\n");
 }
 
-/**********************************************************************
- * Ungzip rx'ed body
+/* SECTION: client-server.spec.gunzip
+ *
+ * gunzip
+ *         Gunzip the body in place.
  */
-
-#define OVERHEAD 64L
-
 static void
 cmd_http_gunzip(CMD_ARGS)
 {
-	int i;
-	z_stream vz;
 	struct http *hp;
-	char *p;
-	unsigned l;
 
 	(void)av;
 	(void)cmd;
 	(void)vl;
+
 	CAST_OBJ_NOTNULL(hp, priv, HTTP_MAGIC);
-
-	memset(&vz, 0, sizeof vz);
-
-	AN(hp->body);
-	if (hp->body[0] != (char)0x1f || hp->body[1] != (char)0x8b)
-		vtc_log(hp->vl, hp->fatal,
-		    "Gunzip error: Body lacks gzip magics");
-	vz.next_in = TRUST_ME(hp->body);
-	vz.avail_in = hp->bodyl;
-
-	l = hp->bodyl * 10;
-	p = calloc(1, l);
-	AN(p);
-
-	vz.next_out = TRUST_ME(p);
-	vz.avail_out = l;
-
-	assert(Z_OK == inflateInit2(&vz, 31));
-	i = inflate(&vz, Z_FINISH);
-	assert(vz.total_out < l);
-	hp->bodyl = vz.total_out;
-	memcpy(hp->body, p, hp->bodyl);
-	free(p);
-	vtc_log(hp->vl, 3, "new bodylen %u", hp->bodyl);
-	vtc_dump(hp->vl, 4, "body", hp->body, hp->bodyl);
-	bprintf(hp->bodylen, "%u", hp->bodyl);
-#ifdef VGZ_EXTENSIONS
-	vtc_log(hp->vl, 4, "startbit = %ju %ju/%ju",
-	    (uintmax_t)vz.start_bit,
-	    (uintmax_t)vz.start_bit >> 3, (uintmax_t)vz.start_bit & 7);
-	vtc_log(hp->vl, 4, "lastbit = %ju %ju/%ju",
-	    (uintmax_t)vz.last_bit,
-	    (uintmax_t)vz.last_bit >> 3, (uintmax_t)vz.last_bit & 7);
-	vtc_log(hp->vl, 4, "stopbit = %ju %ju/%ju",
-	    (uintmax_t)vz.stop_bit,
-	    (uintmax_t)vz.stop_bit >> 3, (uintmax_t)vz.stop_bit & 7);
-#endif
-	if (i != Z_STREAM_END)
-		vtc_log(hp->vl, hp->fatal,
-		    "Gunzip error = %d (%s) in:%jd out:%jd",
-		    i, vz.msg, (intmax_t)vz.total_in, (intmax_t)vz.total_out);
-	assert(Z_OK == inflateEnd(&vz));
-	hp->body[hp->bodyl] = '\0';
-}
-
-/**********************************************************************
- * Create a gzip'ed body
- */
-
-static void
-gzip_body(const struct http *hp, const char *txt, char **body, int *bodylen)
-{
-	int l;
-	z_stream vz;
-#ifdef VGZ_EXTENSIONS
-	int i;
-#endif
-
-	memset(&vz, 0, sizeof vz);
-
-	l = strlen(txt);
-	*body = calloc(1, l + OVERHEAD);
-	AN(*body);
-
-	vz.next_in = TRUST_ME(txt);
-	vz.avail_in = l;
-
-	vz.next_out = TRUST_ME(*body);
-	vz.avail_out = l + OVERHEAD;
-
-	assert(Z_OK == deflateInit2(&vz,
-	    hp->gziplevel, Z_DEFLATED, 31, 9, Z_DEFAULT_STRATEGY));
-	assert(Z_STREAM_END == deflate(&vz, Z_FINISH));
-	*bodylen = vz.total_out;
-#ifdef VGZ_EXTENSIONS
-	i = vz.stop_bit & 7;
-	if (hp->gzipresidual >= 0 && hp->gzipresidual != i)
-		vtc_log(hp->vl, hp->fatal,
-		    "Wrong gzip residual got %d wanted %d",
-		    i, hp->gzipresidual);
-	vtc_log(hp->vl, 4, "startbit = %ju %ju/%ju",
-	    (uintmax_t)vz.start_bit,
-	    (uintmax_t)vz.start_bit >> 3, (uintmax_t)vz.start_bit & 7);
-	vtc_log(hp->vl, 4, "lastbit = %ju %ju/%ju",
-	    (uintmax_t)vz.last_bit,
-	    (uintmax_t)vz.last_bit >> 3, (uintmax_t)vz.last_bit & 7);
-	vtc_log(hp->vl, 4, "stopbit = %ju %ju/%ju",
-	    (uintmax_t)vz.stop_bit,
-	    (uintmax_t)vz.stop_bit >> 3, (uintmax_t)vz.stop_bit & 7);
-	assert(Z_OK == deflateEnd(&vz));
-#endif
+	vtc_gunzip(hp, hp->body, &hp->bodyl);
 }
 
 /**********************************************************************
@@ -864,10 +776,11 @@ static char* const *
 http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
     char *body, unsigned nohost)
 {
-	int bodylen = 0;
+	long bodylen = 0;
 	char *b, *c;
 	char *nullbody;
 	char *m;
+	ssize_t len;
 	int nolen = 0;
 	int l;
 
@@ -912,6 +825,14 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 					bodylen--;
 				}
 			}
+		} else if (!strcmp(*av, "-bodyfrom")) {
+			assert(body == nullbody);
+			free(body);
+			body = VFIL_readfile(NULL, av[1], &len);
+			AN(body);
+			assert(len < INT_MAX);
+			bodylen = len;
+			av++;
 		} else if (!strcmp(*av, "-bodylen")) {
 			assert(body == nullbody);
 			free(body);
@@ -928,7 +849,7 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 			assert(body == nullbody);
 			free(body);
 			b = synth_body(av[1], 1);
-			gzip_body(hp, b, &body, &bodylen);
+			vtc_gzip(hp, b, &body, &bodylen);
 			free(b);
 			VSB_printf(hp->vsb, "Content-Encoding: gzip%s", nl);
 			// vtc_hexdump(hp->vl, 4, "gzip", (void*)body, bodylen);
@@ -936,7 +857,7 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 		} else if (!strcmp(*av, "-gzipbody")) {
 			assert(body == nullbody);
 			free(body);
-			gzip_body(hp, av[1], &body, &bodylen);
+			vtc_gzip(hp, av[1], &body, &bodylen);
 			VSB_printf(hp->vsb, "Content-Encoding: gzip%s", nl);
 			// vtc_hexdump(hp->vl, 4, "gzip", (void*)body, bodylen);
 			av++;
@@ -950,7 +871,7 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
 		free(m);
 	}
 	if (body != NULL && !nolen)
-		VSB_printf(hp->vsb, "Content-Length: %d%s", bodylen, nl);
+		VSB_printf(hp->vsb, "Content-Length: %ld%s", bodylen, nl);
 	VSB_cat(hp->vsb, nl);
 	if (body != NULL) {
 		VSB_bcat(hp->vsb, body, bodylen);
@@ -970,9 +891,12 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
  *         from who can send them is that the first line (request line vs
  *         status line), so all the options are prety much the same.
  *
- *         \-req STRING (txreq only)
+ *         \-method STRING (txreq only)
  *                 What method to use (default: "GET").
  *
+ *         \-req STRING (txreq only)
+ *                 Alias for -method.
+ *
  *         \-url STRING (txreq only)
  *                 What location to use (default "/").
  *
@@ -1007,22 +931,25 @@ http_tx_parse_args(char * const *av, struct vtclog *vl, struct http *hp,
  *         \-body STRING
  *                 Input STRING as body.
  *
+ *         \-bodyfrom FILE
+ *                 Same as -body but content is read from FILE.
+ *
  *         \-bodylen NUMBER
  *                 Generate and input a body that is NUMBER bytes-long.
  *
  *         \-gziplevel NUMBER
- *		   Set the gzip level (call it before any of the other gzip
- *		   switches).
+ *                 Set the gzip level (call it before any of the other gzip
+ *                 switches).
  *
  *         \-gzipresidual NUMBER
  *                 Add extra gzip bits. You should never need it.
  *
  *         \-gzipbody STRING
- *                 Zip STRING and send it as body.
+ *                 Gzip STRING and send it as body.
  *
  *         \-gziplen NUMBER
- *                 Combine -body and -gzipbody: create a body of length NUMBER,
- *                 zip it and send as body.
+ *                 Combine -bodylen and -gzipbody: generate a string of length
+ *                 NUMBER, gzip it and send as body.
  */
 
 /**********************************************************************
@@ -1146,7 +1073,7 @@ cmd_http_rxreq(CMD_ARGS)
 
 /* SECTION: client-server.spec.rxreqhdrs
  *
- * rxreqhdrs
+ * rxreqhdrs (server only)
  *         Receive and parse a request's headers (but not the body).
  */
 
@@ -1275,6 +1202,7 @@ cmd_http_txreq(CMD_ARGS)
 
 	VSB_clear(hp->vsb);
 
+	hp->head_method = 0;
 	for (; *av != NULL; av++) {
 		if (!strcmp(*av, "-url")) {
 			url = av[1];
@@ -1282,7 +1210,8 @@ cmd_http_txreq(CMD_ARGS)
 		} else if (!strcmp(*av, "-proto")) {
 			proto = av[1];
 			av++;
-		} else if (!strcmp(*av, "-req")) {
+		} else if (!strcmp(*av, "-method") ||
+		    !strcmp(*av, "-req")) {
 			req = av[1];
 			hp->head_method = !strcasecmp(av[1], "HEAD") ;
 			av++;
@@ -1443,7 +1372,6 @@ cmd_http_sendhex(CMD_ARGS)
 {
 	struct vsb *vsb;
 	struct http *hp;
-	int j;
 
 	(void)cmd;
 	(void)vl;
@@ -1453,8 +1381,9 @@ cmd_http_sendhex(CMD_ARGS)
 	vsb = vtc_hex_to_bin(hp->vl, av[1]);
 	assert(VSB_len(vsb) >= 0);
 	vtc_hexdump(hp->vl, 4, "sendhex", VSB_data(vsb), VSB_len(vsb));
-	j = write(hp->fd, VSB_data(vsb), VSB_len(vsb));
-	assert(j == VSB_len(vsb));
+	if (VSB_tofile(vsb, hp->fd))
+		vtc_log(hp->vl, hp->fatal, "Write failed: %s",
+		    strerror(errno));
 	VSB_destroy(&vsb);
 }
 
@@ -1904,10 +1833,11 @@ http_process_cleanup(void *arg)
 
 int
 http_process(struct vtclog *vl, const char *spec, int sock, int *sfd,
-	     const char *addr)
+    const char *addr, int rcvbuf)
 {
 	struct http *hp;
-	int retval;
+	int retval, oldbuf;
+	socklen_t intlen = sizeof(int);
 
 	(void)sfd;
 	ALLOC_OBJ(hp, HTTP_MAGIC);
@@ -1915,6 +1845,20 @@ http_process(struct vtclog *vl, const char *spec, int sock, int *sfd,
 	hp->fd = sock;
 	hp->timeout = vtc_maxdur * 1000 / 2;
 
+	if (rcvbuf) {
+		// XXX setsockopt() too late on SunOS
+		// https://github.com/varnishcache/varnish-cache/pull/2980#issuecomment-486214661
+		hp->rcvbuf = rcvbuf;
+
+		oldbuf = 0;
+		AZ(getsockopt(hp->fd, SOL_SOCKET, SO_RCVBUF, &oldbuf, &intlen));
+		AZ(setsockopt(hp->fd, SOL_SOCKET, SO_RCVBUF, &rcvbuf, intlen));
+		AZ(getsockopt(hp->fd, SOL_SOCKET, SO_RCVBUF, &rcvbuf, &intlen));
+
+		vtc_log(vl, 3, "-rcvbuf fd=%d old=%d new=%d actual=%d",
+		    hp->fd, oldbuf, hp->rcvbuf, rcvbuf);
+	}
+
 	hp->nrxbuf = 2048*1024;
 	hp->rx_b = malloc(hp->nrxbuf);
 	AN(hp->rx_b);
