@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -45,9 +47,10 @@
 #define VCT_TCHAR		(1<<9)
 #define VCT_ID			(1<<10)
 #define VCT_IDENT		(VCT_ALPHA | VCT_DIGIT | VCT_ID)
-#define VCT_VAR			(1<<11)
 #define VCT_VT			(1<<12)
 #define VCT_SPACE		(VCT_LWS | VCT_VT)
+#define VCT_UPPER		(1<<13)
+#define VCT_LOWER		(1<<14)
 
 extern const uint16_t vct_typtab[256];
 
@@ -68,12 +71,13 @@ vct_is(int x, uint16_t y)
 #define vct_isspace(x) vct_is(x, VCT_SPACE)
 #define vct_isdigit(x) vct_is(x, VCT_DIGIT)
 #define vct_isalpha(x) vct_is(x, VCT_ALPHA)
+#define vct_islower(x) vct_is(x, VCT_LOWER)
+#define vct_isupper(x) vct_is(x, VCT_UPPER)
 #define vct_isalnum(x) vct_is(x, VCT_ALPHA | VCT_DIGIT)
 #define vct_issep(x) vct_is(x, VCT_SEPARATOR)
 #define vct_issepctl(x) vct_is(x, VCT_SEPARATOR | VCT_CTL)
 #define vct_isident1(x) vct_isalpha(x)
 #define vct_isident(x) vct_is(x, VCT_IDENT)
-#define vct_isvar(x) vct_is(x, VCT_IDENT | VCT_VAR)
 #define vct_isxmlnamestart(x) vct_is(x, VCT_XMLNAMESTART)
 #define vct_isxmlname(x) vct_is(x, VCT_XMLNAMESTART | VCT_XMLNAME)
 #define vct_istchar(x) vct_is(x, VCT_ALPHA | VCT_DIGIT | VCT_TCHAR)
