@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -95,10 +97,7 @@ vjsn_delete(struct vjsn **jp)
 {
 	struct vjsn *js;
 
-	AN(jp);
-	js = *jp;
-	*jp = NULL;
-	CHECK_OBJ_NOTNULL(js, VJSN_MAGIC);
+	TAKE_OBJ_NOTNULL(js, jp, VJSN_MAGIC);
 	if (js->value != NULL)
 		vjsn_val_delete(js->value);
 	free(js->raw);
@@ -398,20 +397,28 @@ vjsn_value(struct vjsn *js)
 }
 
 struct vjsn *
-vjsn_parse(const char *src, const char **err)
+vjsn_parse_end(const char *from, const char *to, const char **err)
 {
 	struct vjsn *js;
 	char *p, *e;
+	size_t sz;
 
-	AN(src);
+	AN(from);
 
 	AN(err);
 	*err = NULL;
 
-	p = strdup(src);
+	if (to == NULL)
+		to = strchr(from, '\0');
+	AN(to);
+
+	sz = to - from;
+
+	p = malloc(sz + 1L);
 	AN(p);
-	e = strchr(p, '\0');
-	AN(e);
+	memcpy(p, from, sz);
+	p[sz] = '\0';
+	e = p + sz;
 
 	ALLOC_OBJ(js, VJSN_MAGIC);
 	AN(js);
@@ -434,6 +441,13 @@ vjsn_parse(const char *src, const char **err)
 	return (js);
 }
 
+struct vjsn *
+vjsn_parse(const char *src, const char **err)
+{
+
+	return (vjsn_parse_end(src, NULL, err));
+}
+
 struct vjsn_val *
 vjsn_child(const struct vjsn_val *vv, const char *key)
 {
@@ -453,6 +467,7 @@ vjsn_dump_i(const struct vjsn_val *jsv, FILE *fo, int indent)
 {
 	struct vjsn_val *jsve;
 
+	CHECK_OBJ_NOTNULL(jsv, VJSN_VAL_MAGIC);
 	printf("%*s", indent, "");
 	if (jsv->name != NULL)
 		printf("[\"%s\"]: ", jsv->name);
@@ -470,6 +485,13 @@ vjsn_dump_i(const struct vjsn_val *jsv, FILE *fo, int indent)
 		vjsn_dump_i(jsve, fo, indent + 2);
 }
 
+void
+vjsn_dump_val(const struct vjsn_val *jsv, FILE *fo)
+{
+	CHECK_OBJ_NOTNULL(jsv, VJSN_VAL_MAGIC);
+	vjsn_dump_i(jsv, fo, 0);
+}
+
 void
 vjsn_dump(const struct vjsn *js, FILE *fo)
 {
