@@ -6,6 +6,8 @@
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  * Author: Martin Blix Grydeland <martin@varnish-software.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -45,7 +47,7 @@
 #include "vqueue.h"
 #include "vjsn.h"
 #include "vsb.h"
-#include "vend.h"
+#include "vsc_priv.h"
 #include "vmb.h"
 
 #include "vapi/vsc.h"
@@ -69,9 +71,14 @@ struct vsc_seg {
 #define VSC_SEG_MAGIC		0x801177d4
 	VTAILQ_ENTRY(vsc_seg)	list;
 	struct vsm_fantom	fantom[1];
+	struct vsc_head		*head;
+	char			*body;
+
 	struct vjsn		*vj;
+
 	unsigned		npoints;
 	struct vsc_pt		*points;
+	int			exposed;
 };
 
 struct vsc {
@@ -91,17 +98,17 @@ struct vsc {
  * Build the static level, type and point descriptions
  */
 
-#define VSC_LEVEL_F(v,l,e,d) \
-    static const struct VSC_level_desc level_##v = {#v, l, e, d};
+enum vsc_levels {
+#define VSC_LEVEL_F(v,l,e,d) v,
 #include "tbl/vsc_levels.h"
-
-static const struct VSC_level_desc * const levels[] = {
-#define VSC_LEVEL_F(v,l,e,d) &level_##v,
-#include "tbl/vsc_levels.h"
-#undef VSC_LEVEL_F
 };
 
-static const size_t nlevels = sizeof(levels)/sizeof(*levels);
+static const struct VSC_level_desc levels[] = {
+#define VSC_LEVEL_F(v,l,e,d) [v] = {#v, l, e, d},
+#include "tbl/vsc_levels.h"
+};
+
+static const ssize_t nlevels = sizeof(levels)/sizeof(*levels);
 
 /*--------------------------------------------------------------------*/
 
@@ -162,17 +169,6 @@ VSC_Arg(struct vsc *vsc, char arg, const char *opt)
 	}
 }
 
-/*--------------------------------------------------------------------
- */
-void
-VSC_State(struct vsc *vsc, VSC_new_f *fn, VSC_destroy_f *fd, void *priv)
-{
-
-	CHECK_OBJ_NOTNULL(vsc, VSC_MAGIC);
-	vsc->fnew = fn;
-	vsc->fdestroy = fd;
-	vsc->priv = priv;
-}
 
 /*--------------------------------------------------------------------
  */
@@ -203,8 +199,8 @@ vsc_clean_point(struct vsc_pt *point)
 	REPLACE(point->name, NULL);
 }
 
-static int
-vsc_fill_point(const struct vsc *vsc, const struct vsm_fantom *fantom,
+static void
+vsc_fill_point(const struct vsc *vsc, const struct vsc_seg *seg,
     const struct vjsn_val *vv, struct vsb *vsb, struct vsc_pt *point)
 {
 	struct vjsn_val *vt;
@@ -217,11 +213,11 @@ vsc_fill_point(const struct vsc *vsc, const struct vsm_fantom *fantom,
 	assert(vt->type == VJSN_STRING);
 
 	VSB_clear(vsb);
-	VSB_printf(vsb, "%s.%s", fantom->ident, vt->value);
+	VSB_printf(vsb, "%s.%s", seg->fantom->ident, vt->value);
 	AZ(VSB_finish(vsb));
 
 	if (vsc_filter(vsc, VSB_data(vsb)))
-		return (0);
+		return;
 
 	point->name = strdup(VSB_data(vsb));
 	AN(point->name);
@@ -272,11 +268,11 @@ vsc_fill_point(const struct vsc *vsc, const struct vsm_fantom *fantom,
 	assert(vt->type == VJSN_STRING);
 
 	if (!strcmp(vt->value, "info"))  {
-		point->point.level = &level_info;
+		point->point.level = &levels[info];
 	} else if (!strcmp(vt->value, "diag")) {
-		point->point.level = &level_diag;
+		point->point.level = &levels[diag];
 	} else if (!strcmp(vt->value, "debug")) {
-		point->point.level = &level_debug;
+		point->point.level = &levels[debug];
 	} else {
 		WRONG("Illegal level");
 	}
@@ -284,38 +280,35 @@ vsc_fill_point(const struct vsc *vsc, const struct vsm_fantom *fantom,
 	vt = vjsn_child(vv, "index");
 	AN(vt);
 
-	point->point.ptr = (volatile void*)
-	    ((volatile char*)fantom->b + atoi(vt->value));
-	return (1);
+	point->point.ptr = (volatile void*)(seg->body + atoi(vt->value));
 }
 
 static void
-vsc_del_seg(const struct vsc *vsc, struct vsm *vsm, struct vsc_seg *sp)
+vsc_del_seg(const struct vsc *vsc, struct vsm *vsm, struct vsc_seg **spp)
 {
 	unsigned u;
 	struct vsc_pt *pp;
+	struct vsc_seg *sp;
 
 	CHECK_OBJ_NOTNULL(vsc, VSC_MAGIC);
 	AN(vsm);
-	CHECK_OBJ_NOTNULL(sp, VSC_SEG_MAGIC);
+	TAKE_OBJ_NOTNULL(sp, spp, VSC_SEG_MAGIC);
 	AZ(VSM_Unmap(vsm, sp->fantom));
-	vjsn_delete(&sp->vj);
-	pp = sp->points;
-	for(u = 0; u < sp->npoints; u++, pp++) {
-		if (vsc->fdestroy != NULL)
-			vsc->fdestroy(vsc->priv, &pp->point);
-		vsc_clean_point(pp);
+	if (sp->vj != NULL) {
+		vjsn_delete(&sp->vj);
+	} else {
+		pp = sp->points;
+		for (u = 0; u < sp->npoints; u++, pp++)
+			vsc_clean_point(pp);
+		free(sp->points);
 	}
-	free(sp->points);
 	FREE_OBJ(sp);
 }
 
 static struct vsc_seg *
 vsc_add_seg(const struct vsc *vsc, struct vsm *vsm, const struct vsm_fantom *fp)
 {
-	struct vsc_seg *sp;
-	uint64_t u;
-	const char *p;
+	struct vsc_seg *sp, *spd;
 	const char *e;
 	struct vjsn_val *vv, *vve;
 	struct vsb *vsb;
@@ -335,40 +328,78 @@ vsc_add_seg(const struct vsc *vsc, struct vsm *vsm, const struct vsm_fantom *fp)
 		FREE_OBJ(sp);
 		return (NULL);
 	}
-	// AZ(VSM_Map(vsm, sp->fantom));
-
-	u = vbe64dec(sp->fantom->b);
-	if (u == 0) {
+	sp->head = sp->fantom->b;
+	if (sp->head->ready == 0) {
 		VRMB();
 		usleep(100000);
-		u = vbe64dec(sp->fantom->b);
 	}
-	assert(u > 0);
-	p = (char*)sp->fantom->b + 8 + u;
-	assert (p < (char*)sp->fantom->e);
-	sp->vj = vjsn_parse(p, &e);
+	assert(sp->head->ready > 0);
+	sp->body = (char*)sp->fantom->b + sp->head->body_offset;
+
+	if (!strcmp(fp->class, VSC_CLASS)) {
+		VTAILQ_FOREACH(spd, &vsc->segs, list)
+			if (spd->head->doc_id == sp->head->doc_id)
+				break;
+		AN(spd);
+		// XXX: Refcount ?
+		vve = vjsn_child(spd->vj->value, "elements");
+		AN(vve);
+		sp->npoints = strtoul(vve->value, NULL, 0);
+		sp->points = calloc(sp->npoints, sizeof *sp->points);
+		AN(sp->points);
+		vsb = VSB_new_auto();
+		AN(vsb);
+		vve = vjsn_child(spd->vj->value, "elem");
+		AN(vve);
+		pp = sp->points;
+		VTAILQ_FOREACH(vv, &vve->children, list) {
+			vsc_fill_point(vsc, sp, vv, vsb, pp);
+			pp++;
+		}
+		VSB_destroy(&vsb);
+		return (sp);
+	}
+	assert(!strcmp(fp->class, VSC_DOC_CLASS));
+	sp->vj = vjsn_parse(sp->body, &e);
 	XXXAZ(e);
-	vve = vjsn_child(sp->vj->value, "elements");
-	AN(vve);
-	sp->npoints = strtoul(vve->value, NULL, 0);
-	sp->points = calloc(sp->npoints, sizeof *sp->points);
-	AN(sp->points);
-	vsb = VSB_new_auto();
-	AN(vsb);
-	vve = vjsn_child(sp->vj->value, "elem");
-	AN(vve);
-	pp = sp->points;
-	VTAILQ_FOREACH(vv, &vve->children, list) {
-		if (vsc_fill_point(vsc, sp->fantom, vv, vsb, pp) &&
-			vsc->fnew != NULL)
-			pp->point.priv = vsc->fnew(vsc->priv, &pp->point);
-		pp++;
-	}
-	VSB_destroy(&vsb);
 	AN(sp->vj);
 	return (sp);
 }
 
+/*--------------------------------------------------------------------
+ */
+
+static void
+vsc_expose(const struct vsc *vsc, struct vsc_seg *sp, int del)
+{
+	struct vsc_pt *pp;
+	unsigned u;
+	int expose;
+
+	if (vsc->fnew != NULL && !sp->exposed &&
+	    !del && sp->head->ready == 1)
+		expose = 1;
+	else if (vsc->fdestroy != NULL && sp->exposed &&
+	    (del || sp->head->ready == 2))
+		expose = 0;
+	else
+		return;
+
+	pp = sp->points;
+	for (u = 0; u < sp->npoints; u++, pp++) {
+		if (pp->name == NULL)
+			continue;
+		if (expose)
+			pp->point.priv = vsc->fnew(vsc->priv, &pp->point);
+		else
+			vsc->fdestroy(vsc->priv, &pp->point);
+	}
+	sp->exposed = expose;
+}
+
+/*--------------------------------------------------------------------
+ */
+
 static int
 vsc_iter_seg(const struct vsc *vsc, const struct vsc_seg *sp,
     VSC_iter_f *fiter, void *priv)
@@ -381,7 +412,7 @@ vsc_iter_seg(const struct vsc *vsc, const struct vsc_seg *sp,
 	CHECK_OBJ_NOTNULL(sp, VSC_SEG_MAGIC);
 	AN(fiter);
 	pp = sp->points;
-	for(u = 0; u < sp->npoints && i == 0; u++, pp++) {
+	for (u = 0; u < sp->npoints && i == 0; u++, pp++) {
 		if (pp->name != NULL)
 			i = fiter(priv, &pp->point);
 	}
@@ -399,7 +430,9 @@ VSC_Iter(struct vsc *vsc, struct vsm *vsm, VSC_iter_f *fiter, void *priv)
 	AN(vsm);
 	sp = VTAILQ_FIRST(&vsc->segs);
 	VSM_FOREACH(&ifantom, vsm) {
-		if (strcmp(ifantom.class, VSC_CLASS))
+		AN(ifantom.class);
+		if (strcmp(ifantom.class, VSC_CLASS) &&
+		    strcmp(ifantom.class, VSC_DOC_CLASS))
 			continue;
 		while (sp != NULL &&
 		    (strcmp(ifantom.ident, sp->fantom->ident) ||
@@ -407,21 +440,24 @@ VSC_Iter(struct vsc *vsc, struct vsm *vsm, VSC_iter_f *fiter, void *priv)
 			sp2 = sp;
 			sp = VTAILQ_NEXT(sp, list);
 			VTAILQ_REMOVE(&vsc->segs, sp2, list);
-			vsc_del_seg(vsc, vsm, sp2);
+			vsc_expose(vsc, sp2, 1);
+			vsc_del_seg(vsc, vsm, &sp2);
 		}
-		if (sp != NULL) {
-			if (fiter != NULL)
-				i = vsc_iter_seg(vsc, sp, fiter, priv);
-			sp = VTAILQ_NEXT(sp, list);
-		} else {
+		if (sp == NULL) {
 			sp = vsc_add_seg(vsc, vsm, &ifantom);
 			if (sp != NULL) {
 				VTAILQ_INSERT_TAIL(&vsc->segs, sp, list);
-				if (fiter != NULL)
-					i = vsc_iter_seg(vsc, sp, fiter, priv);
-				sp = NULL;
+				vsc_expose(vsc, sp, 0);
 			}
+		} else {
+			vsc_expose(vsc, sp, 0);
 		}
+		if (sp != NULL) {
+			if (fiter != NULL && sp->head->ready < 2)
+				i = vsc_iter_seg(vsc, sp, fiter, priv);
+			sp = VTAILQ_NEXT(sp, list);
+		}
+
 		if (i)
 			break;
 	}
@@ -429,11 +465,30 @@ VSC_Iter(struct vsc *vsc, struct vsm *vsm, VSC_iter_f *fiter, void *priv)
 		sp2 = sp;
 		sp = VTAILQ_NEXT(sp, list);
 		VTAILQ_REMOVE(&vsc->segs, sp2, list);
-		vsc_del_seg(vsc, vsm, sp2);
+		vsc_expose(vsc, sp2, 1);
+		vsc_del_seg(vsc, vsm, &sp2);
 	}
 	return (i);
 }
 
+/*--------------------------------------------------------------------
+ */
+
+void
+VSC_State(struct vsc *vsc, VSC_new_f *fn, VSC_destroy_f *fd, void *priv)
+{
+	struct vsc_seg *sp;
+
+	CHECK_OBJ_NOTNULL(vsc, VSC_MAGIC);
+	assert((fn == NULL && fd == NULL) || (fn != NULL && fd != NULL));
+	if (fd == NULL) {
+		VTAILQ_FOREACH(sp, &vsc->segs, list)
+			vsc_expose(vsc, sp, 1);
+	}
+	vsc->fnew = fn;
+	vsc->fdestroy = fd;
+	vsc->priv = priv;
+}
 
 /*--------------------------------------------------------------------
  */
@@ -444,19 +499,19 @@ VSC_ChangeLevel(const struct VSC_level_desc *old, int chg)
 	int i;
 
 	if (old == NULL)
-		old = levels[0];
+		old = &levels[0];
 	for (i = 0; i < nlevels; i++)
-		if (old == levels[i])
+		if (old == &levels[i])
 			break;
 	if (i == nlevels)
 		i = 0;
-	else
-		i += chg;
+
+	i += chg;
 	if (i >= nlevels)
 		i = nlevels - 1;
 	if (i < 0)
 		i = 0;
-	return (levels[i]);
+	return (&levels[i]);
 }
 
 /*--------------------------------------------------------------------*/
@@ -486,7 +541,8 @@ VSC_Destroy(struct vsc **vscp, struct vsm *vsm)
 	vsc_delete_sf_list(&vsc->sf_list_exclude);
 	VTAILQ_FOREACH_SAFE(sp, &vsc->segs, list, sp2) {
 		VTAILQ_REMOVE(&vsc->segs, sp, list);
-		vsc_del_seg(vsc, vsm, sp);
+		vsc_expose(vsc, sp, 1);
+		vsc_del_seg(vsc, vsm, &sp);
 	}
 	FREE_OBJ(vsc);
 }
