@@ -5,6 +5,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -186,7 +188,7 @@ vcc_expr_edit(struct vcc *tl, vcc_type_t fmt, const char *p, struct expr *e1,
 			    "  const char * strs_%u_s[%d];\n",
 			    tl->unique, tl->unique, e3->nstr);
 			VSB_printf(e->vsb,
-			    "VRT_BundleStrands(%d, &strs_%u_a, strs_%u_s,"
+			    "VPI_BundleStrands(%d, &strs_%u_a, strs_%u_s,"
 			    "\v+\n%s,\nvrt_magic_string_end\v-\n)",
 			    e3->nstr, tl->unique, tl->unique,
 			VSB_data(e3->vsb));
@@ -261,7 +263,7 @@ vcc_expr_tobool(struct vcc *tl, struct expr **e)
 	else if ((*e)->fmt == DURATION)
 		*e = vcc_expr_edit(tl, BOOL, "(\v1 > 0)", *e, NULL);
 	else if ((*e)->fmt == STRINGS)
-		*e = vcc_expr_edit(tl, BOOL, "(\vS != 0)", *e, NULL);
+		*e = vcc_expr_edit(tl, BOOL, "VRT_Strands2Bool(\vT)", *e, NULL);
 	/*
 	 * We do not provide automatic folding from REAL to BOOL
 	 * because comparing to zero is seldom an exact science
@@ -290,7 +292,7 @@ vcc_expr_tostring(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 		(*e)->nstr = 1;
 	} else {
 		if ((*e)->fmt == BLOB)
-			VSB_printf(tl->sb,
+			VSB_cat(tl->sb,
 			    "Wrong use of BLOB value.\n"
 			    "BLOBs can only be used as arguments to VMOD"
 			    " functions.\n");
@@ -302,55 +304,6 @@ vcc_expr_tostring(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	}
 }
 
-/*--------------------------------------------------------------------
- */
-
-static void v_matchproto_(sym_expr_t)
-vcc_Eval_Regsub(struct vcc *tl, struct expr **e, struct token *t,
-    struct symbol *sym, vcc_type_t fmt)
-{
-	struct expr *e2;
-	int all = sym->eval_priv == NULL ? 0 : 1;
-	char buf[128];
-	struct vsb vsb;
-
-	(void)t;
-	(void)fmt;
-	SkipToken(tl, '(');
-	vcc_expr0(tl, &e2, STRING);
-	ERRCHK(tl);
-	SkipToken(tl, ',');
-	ExpectErr(tl, CSTR);
-
-	AN(VSB_new(&vsb, buf, sizeof buf, VSB_FIXEDLEN));
-	VSB_printf(&vsb, "VRT_regsub(ctx, %d,\v+\n\v1,\n", all);
-	vcc_regexp(tl, &vsb);
-	ERRCHK(tl);
-	AZ(VSB_finish(&vsb));
-	*e = vcc_expr_edit(tl, STRING, VSB_data(&vsb), e2, NULL);
-	SkipToken(tl, ',');
-	vcc_expr0(tl, &e2, STRING);
-	ERRCHK(tl);
-	*e = vcc_expr_edit(tl, STRINGS, "\v1,\n\v2)\v-", *e, e2);
-	(*e)->nstr = 1;
-	SkipToken(tl, ')');
-}
-
-/*--------------------------------------------------------------------
- */
-
-static void v_matchproto_(sym_expr_t)
-vcc_Eval_BoolConst(struct vcc *tl, struct expr **e, struct token *t,
-    struct symbol *sym, vcc_type_t fmt)
-{
-
-	(void)t;
-	(void)tl;
-	(void)fmt;
-	*e = vcc_mk_expr(BOOL, "(0==%d)", sym->eval_priv == NULL ? 1 : 0);
-	(*e)->constant = EXPR_CONST;
-}
-
 /*--------------------------------------------------------------------
  */
 
@@ -364,7 +317,8 @@ vcc_Eval_Handle(struct vcc *tl, struct expr **e, struct token *t,
 	AN(sym->rname);
 	AZ(type->stringform);
 
-	if (sym->type != STRING && type == STRINGS) {
+	if (sym->type->tostring == NULL &&
+	    sym->type != STRING && type == STRINGS) {
 		*e = vcc_mk_expr(STRINGS, "\"%s\"", sym->name);
 		(*e)->nstr = 1;
 		(*e)->constant |= EXPR_CONST | EXPR_STR_CONST;
@@ -404,22 +358,24 @@ vcc_priv_arg(struct vcc *tl, const char *p, const struct symbol *sym)
 {
 	char buf[64];
 	struct inifin *ifp;
-	const char *vmod, *f = NULL;
+	const char *f = NULL;
 	struct procprivhead *marklist = NULL;
 
 	AN(sym);
-	AN(sym->vmod);
-	vmod = sym->vmod;
+	AN(sym->vmod_name);
 
-	if (!strcmp(p, "PRIV_VCL")) {
-		return (vcc_mk_expr(VOID, "&vmod_priv_%s", vmod));
-	} else if (!strcmp(p, "PRIV_CALL")) {
+	if (!strcmp(p, "PRIV_VCL"))
+		return (vcc_mk_expr(VOID, "&vmod_priv_%s", sym->vmod_name));
+
+	if (!strcmp(p, "PRIV_CALL")) {
 		bprintf(buf, "vmod_priv_%u", tl->unique++);
 		ifp = New_IniFin(tl);
 		Fh(tl, 0, "static struct vmod_priv %s;\n", buf);
 		VSB_printf(ifp->fin, "\tVRT_priv_fini(&%s);", buf);
 		return (vcc_mk_expr(VOID, "&%s", buf));
-	} else if (!strcmp(p, "PRIV_TASK")) {
+	}
+
+	if (!strcmp(p, "PRIV_TASK")) {
 		f = "task";
 		marklist = &tl->curproc->priv_tasks;
 	} else if (!strcmp(p, "PRIV_TOP")) {
@@ -430,9 +386,9 @@ vcc_priv_arg(struct vcc *tl, const char *p, const struct symbol *sym)
 	}
 	AN(f);
 	AN(marklist);
-	bprintf(buf, "ARG_priv_%s_%s", f, vmod);
+	bprintf(buf, "ARG_priv_%s_%s", f, sym->vmod_name);
 
-	if (vcc_MarkPriv(tl, marklist, vmod) == NULL)
+	if (vcc_MarkPriv(tl, marklist, sym->vmod_name) == NULL)
 		VSB_printf(tl->curproc->prologue,
 			   "  struct vmod_priv *%s = "
 			   "VRT_priv_%s(ctx, &VGC_vmod_%s);\n"
@@ -441,7 +397,7 @@ vcc_priv_arg(struct vcc *tl, const char *p, const struct symbol *sym)
 			   "for vmod %s\");\n"
 			   "    return;\n"
 			   "  }\n",
-			   buf, f, vmod, buf, f, vmod);
+			   buf, f, sym->vmod_name, buf, f, sym->vmod_name);
 	return (vcc_mk_expr(VOID, "%s", buf));
 }
 
@@ -482,8 +438,8 @@ vcc_do_arg(struct vcc *tl, struct func_arg *fa)
 			if (vcc_IdIs(tl->t, vv->value))
 				break;
 		if (vv == NULL) {
-			VSB_printf(tl->sb, "Wrong enum value.");
-			VSB_printf(tl->sb, "  Expected one of:\n");
+			VSB_cat(tl->sb, "Wrong enum value.");
+			VSB_cat(tl->sb, "  Expected one of:\n");
 			VTAILQ_FOREACH(vv, &fa->enums->children, list)
 				VSB_printf(tl->sb, "\t%s\n", vv->value);
 			vcc_ErrWhere(tl, tl->t);
@@ -511,7 +467,7 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 	VTAILQ_HEAD(,func_arg) head;
 	struct token *t1;
 	const struct vjsn_val *vv, *vvp;
-	const char *sa;
+	const char *sa, *extra_sep;
 	char ssa[64];
 	int n;
 
@@ -529,8 +485,13 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 	}
 	vv = VTAILQ_NEXT(vv, list);
 	SkipToken(tl, '(');
-	if (extra == NULL)
+	if (extra == NULL) {
 		extra = "";
+		extra_sep = "";
+	} else {
+		AN(*extra);
+		extra_sep = ", ";
+	}
 	VTAILQ_INIT(&head);
 	for (;vv != NULL; vv = VTAILQ_NEXT(vv, list)) {
 		assert(vv->type == VJSN_ARRAY);
@@ -616,10 +577,11 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 	}
 
 	if (sa != NULL)
-		e1 = vcc_mk_expr(rfmt, "%s(ctx%s,\v+ &(%s){\n",
-		    cfunc, extra, sa);
+		e1 = vcc_mk_expr(rfmt, "%s(ctx%s%s,\v+\n&(%s)\v+ {\n",
+		    cfunc, extra_sep, extra, sa);
 	else
-		e1 = vcc_mk_expr(rfmt, "%s(ctx%s\v+", cfunc, extra);
+		e1 = vcc_mk_expr(rfmt, "%s(ctx%s%s\v+",
+		    cfunc, extra_sep, extra);
 	n = 0;
 	VTAILQ_FOREACH_SAFE(fa, &head, list, fa2) {
 		n++;
@@ -653,7 +615,7 @@ vcc_func(struct vcc *tl, struct expr **e, const void *priv,
 		free(fa);
 	}
 	if (sa != NULL) {
-		*e = vcc_expr_edit(tl, e1->fmt, "\v1\n})\v-", e1, NULL);
+		*e = vcc_expr_edit(tl, e1->fmt, "\v1\v-\n}\v-\n)", e1, NULL);
 	} else {
 		*e = vcc_expr_edit(tl, e1->fmt, "\v1\v-\n)", e1, NULL);
 	}
@@ -672,7 +634,7 @@ vcc_Eval_Func(struct vcc *tl, const struct vjsn_val *spec,
 
 	vcc_func(tl, &e, spec, extra, sym);
 	if (tl->err)
-		VSB_printf(tl->sb, "While compiling function call:\n");
+		VSB_cat(tl->sb, "While compiling function call:\n");
 	ERRCHK(tl);
 	vcc_expr_fmt(tl->fb, tl->indent, e);
 	VSB_cat(tl->fb, ";\n");
@@ -702,7 +664,7 @@ vcc_Eval_SymFunc(struct vcc *tl, struct expr **e, struct token *t,
 
 /*--------------------------------------------------------------------
  * SYNTAX:
- *    Expr4:
+ *    Expr5:
  *	'(' ExprCor ')'
  *	symbol
  *	CNUM
@@ -711,7 +673,7 @@ vcc_Eval_SymFunc(struct vcc *tl, struct expr **e, struct token *t,
  */
 
 static void
-vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
+vcc_expr5(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 {
 	struct expr *e1, *e2;
 	const char *ip, *sign;
@@ -734,24 +696,12 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	}
 	switch (tl->t->tok) {
 	case ID:
-		if (vcc_IdIs(tl->t, "default") && fmt == PROBE) {
-			vcc_NextToken(tl);
-			*e = vcc_mk_expr(PROBE, "%s", vcc_default_probe(tl));
-			return;
-		}
-		if (vcc_IdIs(tl->t, "default") && fmt == BACKEND) {
-			vcc_NextToken(tl);
-			*e = vcc_mk_expr(BACKEND,
-			    "*(VCL_conf.default_director)");
-			return;
-		}
 		t = tl->t;
-		sym = VCC_SymbolGet(tl, SYM_NONE, "Symbol not found",
-		    XREF_REF);
+		sym = VCC_SymbolGet(tl, SYM_NONE, SYMTAB_PARTIAL, XREF_REF);
 		ERRCHK(tl);
 		AN(sym);
 		if (sym->kind == SYM_FUNC && sym->type == VOID) {
-			VSB_printf(tl->sb, "Function returns VOID:\n");
+			VSB_cat(tl->sb, "Function returns VOID:\n");
 			vcc_ErrWhere(tl, tl->t);
 			return;
 		}
@@ -760,7 +710,7 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 			AZ(*e);
 			sym->eval(tl, e, t, sym, fmt);
 			if (tl->err) {
-				VSB_printf(tl->sb,
+				VSB_cat(tl->sb,
 				    "While compiling function call:\n\n");
 				vcc_ErrWhere2(tl, t, tl->t);
 			}
@@ -773,11 +723,11 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 			return;
 		}
 		VSB_printf(tl->sb,
-		    "Symbol type (%s) can not be used in expression.\n",
-		    sym->kind->name);
-		vcc_ErrWhere(tl, tl->t);
+		    "Symbol '%.*s' type (%s) can not be used in expression.\n",
+		    PF(t), sym->kind->name);
+		vcc_ErrWhere(tl, t);
 		if (sym->def_b != NULL) {
-			VSB_printf(tl->sb, "That symbol was defined here:\n");
+			VSB_cat(tl->sb, "That symbol was defined here:\n");
 			vcc_ErrWhere(tl, sym->def_b);
 		}
 		return;
@@ -791,8 +741,8 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 				 * sockaddr_un if it happens to exist and
 				 * is a socket. So don't let that happen.
 				 */
-				VSB_printf(tl->sb,
-					"Cannot convert to an IP address: ");
+				VSB_cat(tl->sb,
+				    "Cannot convert to an IP address: ");
 				vcc_ErrToken(tl, tl->t);
 				vcc_ErrWhere(tl, tl->t);
 				return;
@@ -822,8 +772,12 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 		    fmt != STRINGS)
 			break;
 		vcc_NextToken(tl);
-		if (tl->t->tok != FNUM && tl->t->tok != CNUM)
-			break;
+		if (tl->t->tok != FNUM && tl->t->tok != CNUM) {
+			vcc_expr_cor(tl, &e1, fmt);
+			ERRCHK(tl);
+			*e = vcc_expr_edit(tl, e1->fmt, "-(\v1)", e1, NULL);
+			return;
+		}
 		sign = "-";
 		/* FALLTHROUGH */
 	case FNUM:
@@ -852,12 +806,87 @@ vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	default:
 		break;
 	}
-	VSB_printf(tl->sb, "Unknown token ");
+	VSB_cat(tl->sb, "Unknown token ");
 	vcc_ErrToken(tl, tl->t);
 	VSB_printf(tl->sb, " when looking for %s\n\n", vcc_utype(fmt));
 	vcc_ErrWhere(tl, tl->t);
 }
 
+/*--------------------------------------------------------------------
+ * SYNTAX:
+ *    Expr4:
+ *      Expr5 [ '.' (type_attribute | type_method()) ]*
+ *
+ * type_attributes is information already existing, requiring no
+ * processing or resource usage.
+ *
+ * type_methods are calls and may do (significant processing, change things,
+ * eat workspace etc.
+ */
+
+static const struct vcc_methods {
+	vcc_type_t		type_from;
+	vcc_type_t		type_to;
+	const char		*method;
+	const char		*impl;
+	int			func;
+} vcc_methods[] = {
+	//{ BACKEND, BOOL,	"healthy",	"VRT_Healthy(ctx, \v1, 0)" },
+
+#define VRTSTVVAR(nm, vtype, ctype, dval) \
+	{ STEVEDORE, vtype, #nm, "VRT_stevedore_" #nm "(\v1)", 0},
+#include "tbl/vrt_stv_var.h"
+
+	{ STRINGS, STRING, "upper", "VRT_UpperLowerStrands(ctx, \vT, 1)", 1 },
+	{ STRINGS, STRING, "lower", "VRT_UpperLowerStrands(ctx, \vT, 0)", 1 },
+
+	{ NULL, NULL,		NULL,		NULL},
+};
+
+static void
+vcc_expr4(struct vcc *tl, struct expr **e, vcc_type_t fmt)
+{
+	const struct vcc_methods *vm;
+
+	*e = NULL;
+	vcc_expr5(tl, e, fmt);
+	ERRCHK(tl);
+	AN(*e);
+	while (tl->t->tok == '.') {
+		vcc_NextToken(tl);
+		ExpectErr(tl, ID);
+
+		for(vm = vcc_methods; vm->type_from != NULL; vm++) {
+
+			if (vm->type_from == (*e)->fmt &&
+			    vcc_IdIs(tl->t, vm->method))
+				break;
+		}
+
+		if (vm->type_from == NULL) {
+			VSB_cat(tl->sb, "Unknown property ");
+			vcc_ErrToken(tl, tl->t);
+			VSB_printf(tl->sb,
+			 " for type %s\n", (*e)->fmt->name);
+			vcc_ErrWhere(tl, tl->t);
+			return;
+		}
+		vcc_NextToken(tl);
+		*e = vcc_expr_edit(tl, vm->type_to, vm->impl, *e, NULL);
+		ERRCHK(tl);
+		if ((*e)->fmt == STRING) {
+			(*e)->fmt = STRINGS;
+			(*e)->nstr = 1;
+		}
+		if (vm->func) {
+			ExpectErr(tl, '(');
+			vcc_NextToken(tl);
+			ExpectErr(tl, ')');
+			vcc_NextToken(tl);
+		}
+	}
+}
+
 /*--------------------------------------------------------------------
  * SYNTAX:
  *    ExprMul:
@@ -1125,8 +1154,8 @@ static const struct cmps vcc_cmps[] = {
 
 	{BOOL,		T_EQ,		cmp_simple, "((!(\v1)) == (!(\v2)))" },
 	{BOOL,		T_NEQ,		cmp_simple, "((!(\v1)) != (!(\v2)))" },
-	{IP,		T_EQ,		cmp_simple, "!VRT_ipcmp(\v1, \v2)" },
-	{IP,		T_NEQ,		cmp_simple, "VRT_ipcmp(\v1, \v2)" },
+	{IP,		T_EQ,		cmp_simple, "!VRT_ipcmp(ctx, \v1, \v2)" },
+	{IP,		T_NEQ,		cmp_simple, "VRT_ipcmp(ctx, \v1, \v2)" },
 
 	{IP,		'~',		cmp_acl, "" },
 	{IP,		T_NOMATCH,	cmp_acl, "!" },
@@ -1206,7 +1235,7 @@ vcc_expr_not(struct vcc *tl, struct expr **e, vcc_type_t fmt)
 	vcc_expr_tobool(tl, e);
 	ERRCHK(tl);
 	if ((*e)->fmt != BOOL) {
-		VSB_printf(tl->sb, "'!' must be followed by BOOL, found ");
+		VSB_cat(tl->sb, "'!' must be followed by BOOL, found ");
 		VSB_printf(tl->sb, "%s.\n", vcc_utype((*e)->fmt));
 		vcc_ErrWhere2(tl, tk, tl->t);
 	} else {
@@ -1374,7 +1403,7 @@ vcc_Expr(struct vcc *tl, vcc_type_t fmt)
 	assert(e->fmt == fmt);
 
 	vcc_expr_fmt(tl->fb, tl->indent, e);
-	VSB_printf(tl->fb, "\n");
+	VSB_cat(tl->fb, "\n");
 	vcc_delete_expr(e);
 }
 
@@ -1399,6 +1428,77 @@ vcc_Act_Call(struct vcc *tl, struct token *t, struct symbol *sym)
 	}
 	vcc_delete_expr(e);
 }
+/*--------------------------------------------------------------------
+ */
+
+static void v_matchproto_(sym_expr_t)
+vcc_Eval_Regsub(struct vcc *tl, struct expr **e, struct token *t,
+    struct symbol *sym, vcc_type_t fmt)
+{
+	struct expr *e2;
+	int all = sym->eval_priv == NULL ? 0 : 1;
+	char buf[128];
+	struct vsb vsb;
+
+	(void)t;
+	(void)fmt;
+	SkipToken(tl, '(');
+	vcc_expr0(tl, &e2, STRING);
+	ERRCHK(tl);
+	SkipToken(tl, ',');
+	ExpectErr(tl, CSTR);
+
+	AN(VSB_new(&vsb, buf, sizeof buf, VSB_FIXEDLEN));
+	VSB_printf(&vsb, "VRT_regsub(ctx, %d,\v+\n\v1,\n", all);
+	vcc_regexp(tl, &vsb);
+	ERRCHK(tl);
+	AZ(VSB_finish(&vsb));
+	*e = vcc_expr_edit(tl, STRING, VSB_data(&vsb), e2, NULL);
+	SkipToken(tl, ',');
+	vcc_expr0(tl, &e2, STRING);
+	ERRCHK(tl);
+	*e = vcc_expr_edit(tl, STRINGS, "\v1,\n\v2)\v-", *e, e2);
+	(*e)->nstr = 1;
+	SkipToken(tl, ')');
+}
+
+/*--------------------------------------------------------------------
+ */
+
+static void v_matchproto_(sym_expr_t)
+vcc_Eval_BoolConst(struct vcc *tl, struct expr **e, struct token *t,
+    struct symbol *sym, vcc_type_t fmt)
+{
+
+	(void)t;
+	(void)tl;
+	(void)fmt;
+	*e = vcc_mk_expr(BOOL, "(0==%d)", sym->eval_priv == NULL ? 1 : 0);
+	(*e)->constant = EXPR_CONST;
+}
+
+/*--------------------------------------------------------------------
+ */
+
+static void v_matchproto_(sym_expr_t)
+vcc_Eval_Default(struct vcc *tl, struct expr **e, struct token *t,
+    struct symbol *sym, vcc_type_t fmt)
+{
+	(void)e;
+	(void)fmt;
+	(void)sym;
+	(void)t;
+
+	if (fmt == PROBE)
+		*e = vcc_mk_expr(PROBE, "%s", vcc_default_probe(tl));
+	else if (fmt == BACKEND)
+		*e = vcc_mk_expr(BACKEND, "*(VCL_conf.default_director)");
+	else {
+		VSB_printf(tl->sb,
+		    "Symbol 'default' is a reserved word.\n");
+		vcc_ErrWhere(tl, t);
+	}
+}
 
 /*--------------------------------------------------------------------
  */
@@ -1431,4 +1531,9 @@ vcc_Expr_Init(struct vcc *tl)
 	sym->type = BOOL;
 	sym->eval = vcc_Eval_BoolConst;
 	sym->eval_priv = NULL;
+
+	sym = VCC_MkSym(tl, "default", SYM_FUNC, VCL_LOW, VCL_HIGH);
+	AN(sym);
+	sym->type = BACKEND;	// ... can also (sometimes) deliver PROBE
+	sym->eval = vcc_Eval_Default;
 }
