@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -40,29 +42,36 @@
 #include "vmod_abi.h"
 #include "vsb.h"
 
+struct vmod_open {
+	unsigned		magic;
+#define VMOD_OPEN_MAGIC		0x9995b1f3
+	void			*hdl;
+	const char		*err;
+};
+
 static int
 vcc_path_dlopen(void *priv, const char *fn)
 {
-	void *hdl, **pp;
+	struct vmod_open *vop;
 
-	AN(priv);
+	CAST_OBJ_NOTNULL(vop, priv, VMOD_OPEN_MAGIC);
 	AN(fn);
 
-	hdl = dlopen(fn, RTLD_NOW | RTLD_LOCAL);
-	if (hdl == NULL)
+	vop->hdl = dlopen(fn, RTLD_NOW | RTLD_LOCAL);
+	if (vop->hdl == NULL) {
+		vop->err = dlerror();
 		return (-1);
-	pp = priv;
-	*pp = hdl;
+	}
 	return (0);
 }
 
 static void
-func_sym(struct symbol *sym, const char *vmod, const struct vjsn_val *v)
+func_sym(struct symbol *sym, const char *vmod_name, const struct vjsn_val *v)
 {
 
 	assert(v->type == VJSN_ARRAY);
 	sym->action = vcc_Act_Call;
-	sym->vmod = vmod;
+	sym->vmod_name = vmod_name;
 	sym->eval = vcc_Eval_SymFunc;
 	sym->eval_priv = v;
 	v = VTAILQ_FIRST(&v->children);
@@ -74,17 +83,16 @@ func_sym(struct symbol *sym, const char *vmod, const struct vjsn_val *v)
 }
 
 static void
-vcc_json_always(struct vcc *tl, const struct symbol *msym)
+vcc_json_always(struct vcc *tl, const struct vjsn *vj, const char *vmod_name)
 {
 	struct inifin *ifp;
-	const struct vjsn *vj;
 	const struct vjsn_val *vv, *vv2;
 	double vmod_syntax = 0.0;
 
+	AN(vj);
+	AN(vmod_name);
 	ifp = NULL;
 
-	CAST_OBJ_NOTNULL(vj, msym->eval_priv, VJSN_MAGIC);
-
 	VTAILQ_FOREACH(vv, &vj->value->children, list) {
 		assert(vv->type == VJSN_ARRAY);
 		vv2 = VTAILQ_FIRST(&vv->children);
@@ -103,13 +111,13 @@ vcc_json_always(struct vcc *tl, const struct symbol *msym)
 			VSB_printf(ifp->ini,
 			    "\tif (%s(ctx, &vmod_priv_%s, VCL_EVENT_LOAD))\n"
 			    "\t\treturn(1);",
-			    vv2->value, msym->name);
+			    vv2->value, vmod_name);
 			VSB_printf(ifp->fin,
 			    "\t\t(void)%s(ctx, &vmod_priv_%s,\n"
 			    "\t\t\t    VCL_EVENT_DISCARD);",
-			    vv2->value, msym->name);
+			    vv2->value, vmod_name);
 			VSB_printf(ifp->event, "%s(ctx, &vmod_priv_%s, ev)",
-			    vv2->value, msym->name);
+			    vv2->value, vmod_name);
 		} else if (!strcmp(vv2->value, "$FUNC")) {
 		} else if (!strcmp(vv2->value, "$OBJ")) {
 		} else {
@@ -139,70 +147,114 @@ vcc_json_wildcard(struct vcc *tl, struct symbol *msym, struct symbol *tsym)
 		    !strcmp(vv2->value, tsym->name)) {
 			tsym->kind = SYM_FUNC;
 			tsym->noref = 1;
-			func_sym(tsym, msym->name, VTAILQ_NEXT(vv2, list));
+			func_sym(tsym, msym->vmod_name, VTAILQ_NEXT(vv2, list));
 			return;
 		} else if (!strcmp(vv1->value, "$OBJ") &&
 			   !strcmp(vv2->value, tsym->name)) {
 			tsym->kind = SYM_OBJECT;
 			tsym->eval_priv = vv2;
-			tsym->vmod = msym->name;
+			tsym->vmod_name = msym->vmod_name;
 			return;
 		}
 	}
 	tl->err = 1;
 }
 
+static const struct vmod_data *
+vcc_VmodSanity(struct vcc *tl, void *hdl, struct token *mod, char *fnp)
+{
+	char buf[256];
+	const struct vmod_data *vmd;
+
+	bprintf(buf, "Vmod_%.*s_Data", PF(mod));
+	vmd = dlsym(hdl, buf);
+	if (vmd == NULL) {
+		VSB_printf(tl->sb, "Malformed VMOD %.*s\n", PF(mod));
+		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
+		VSB_cat(tl->sb, "\t(no Vmod_Data symbol)\n");
+		vcc_ErrWhere(tl, mod);
+		return (NULL);
+	}
+	if (vmd->vrt_major == 0 && vmd->vrt_minor == 0 &&
+	    (vmd->abi == NULL || strcmp(vmd->abi, VMOD_ABI_Version) != 0)) {
+		VSB_printf(tl->sb, "Incompatible VMOD %.*s\n", PF(mod));
+		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
+		VSB_printf(tl->sb, "\tABI mismatch, expected <%s>, got <%s>\n",
+			   VMOD_ABI_Version, vmd->abi);
+		vcc_ErrWhere(tl, mod);
+		return (NULL);
+	}
+	if (vmd->vrt_major != 0 && (vmd->vrt_major != VRT_MAJOR_VERSION ||
+	    vmd->vrt_minor > VRT_MINOR_VERSION)) {
+		VSB_printf(tl->sb, "Incompatible VMOD %.*s\n", PF(mod));
+		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
+		VSB_printf(tl->sb, "\tVMOD wants ABI version %u.%u\n",
+		    vmd->vrt_major, vmd->vrt_minor);
+		VSB_printf(tl->sb, "\tvarnishd provides ABI version %u.%u\n",
+		    VRT_MAJOR_VERSION, VRT_MINOR_VERSION);
+		vcc_ErrWhere(tl, mod);
+		return (NULL);
+	}
+	if (vmd->name == NULL ||
+	    vmd->func == NULL ||
+	    vmd->func_len <= 0 ||
+	    vmd->proto == NULL ||
+	    vmd->abi == NULL) {
+		VSB_printf(tl->sb, "Mangled VMOD %.*s\n", PF(mod));
+		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
+		VSB_cat(tl->sb, "\tInconsistent metadata\n");
+		vcc_ErrWhere(tl, mod);
+		return (NULL);
+	}
+	if (!vcc_IdIs(mod, vmd->name)) {
+		VSB_printf(tl->sb, "Wrong file for VMOD %.*s\n", PF(mod));
+		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
+		VSB_printf(tl->sb, "\tContains vmod \"%s\"\n", vmd->name);
+		vcc_ErrWhere(tl, mod);
+		return (NULL);
+	}
+	return (vmd);
+}
+
 void
 vcc_ParseImport(struct vcc *tl)
 {
-	void *hdl;
-	char fn[1024], *fnp, *fnpx;
-	char buf[256];
+	char fn[1024], *fnpx;
 	const char *p;
-	struct token *mod, *t1;
+	struct token *mod, *tmod, *t1;
 	struct inifin *ifp;
-	struct symbol *msym;
+	struct symbol *msym, *vsym;
 	const struct vmod_data *vmd;
 	struct vjsn *vj;
-	int again = 0;
+	struct vmod_open vop[1];
 
+	INIT_OBJ(vop, VMOD_OPEN_MAGIC);
 	t1 = tl->t;
 	SkipToken(tl, ID);		/* "import" */
 
-
-	ExpectErr(tl, ID);
+	ExpectErr(tl, ID);		/* "vmod_name" */
 	mod = tl->t;
-	msym = VCC_SymbolGet(tl, SYM_NONE, SYMTAB_NOERR, XREF_NONE);
-
-	if (msym != NULL && msym->kind != SYM_VMOD) {
-		/*
-		 * We need to make sure the entire std.* namespace is empty
-		 */
-		VSB_printf(tl->sb, "Module %.*s conflicts with other symbol.\n",
-		    PF(mod));
-		vcc_ErrWhere2(tl, t1, tl->t);
-		return;
+	tmod = VTAILQ_NEXT(mod, list);
+	if (tmod->tok == ID && vcc_IdIs(tmod, "as")) {
+		vcc_NextToken(tl);		/* "vmod_name" */
+		vcc_NextToken(tl);		/* "as" */
+		ExpectErr(tl, ID);		/* "vcl_name" */
 	}
-	if (msym != NULL) {
-		again = 1;
-	} else {
+	tmod = tl->t;
 
-		msym = VCC_SymbolGet(tl, SYM_VMOD, SYMTAB_CREATE, XREF_NONE);
-		ERRCHK(tl);
-		AN(msym);
-		msym->def_b = t1;
-		msym->def_e = tl->t;
-	}
+	msym = VCC_SymbolGet(tl, SYM_VMOD, SYMTAB_CREATE, XREF_NONE);
+	ERRCHK(tl);
+	AN(msym);
 
 	if (tl->t->tok == ID) {
 		if (!vcc_IdIs(tl->t, "from")) {
-			VSB_printf(tl->sb, "Expected 'from path ...'\n");
+			VSB_cat(tl->sb, "Expected 'from path ...'\n");
 			vcc_ErrWhere(tl, tl->t);
 			return;
 		}
 		vcc_NextToken(tl);
 		if (!tl->unsafe_path && strchr(tl->t->dec, '/')) {
-			VSB_printf(tl->sb,
+			VSB_cat(tl->sb,
 			    "'import ... from path ...' is unsafe.\nAt:");
 			vcc_ErrToken(tl, tl->t);
 			vcc_ErrWhere(tl, tl->t);
@@ -221,112 +273,92 @@ vcc_ParseImport(struct vcc *tl)
 
 	SkipToken(tl, ';');
 
-	if (!again)
-		msym->def_e = tl->t;
-
-
-	if (VFIL_searchpath(tl->vmod_path, vcc_path_dlopen, &hdl, fn, &fnpx)) {
-		VSB_printf(tl->sb, "Could not load VMOD %.*s\n", PF(mod));
-		VSB_printf(tl->sb, "\tFile name: %s\n",
-		    fnpx != NULL ? fnpx : fn);
-		VSB_printf(tl->sb, "\tdlerror: %s\n", dlerror());
+	if (VFIL_searchpath(tl->vmod_path, vcc_path_dlopen, vop, fn, &fnpx)) {
+		if (vop->err == NULL) {
+			VSB_printf(tl->sb,
+			    "Could not find VMOD %.*s\n", PF(mod));
+		} else {
+			VSB_printf(tl->sb,
+			    "Could not open VMOD %.*s\n", PF(mod));
+			VSB_printf(tl->sb, "\tFile name: %s\n",
+			    fnpx != NULL ? fnpx : fn);
+			VSB_printf(tl->sb, "\tdlerror: %s\n", vop->err);
+		}
 		vcc_ErrWhere(tl, mod);
 		free(fnpx);
 		return;
 	}
 
-	AN(fnpx);
-	fnp = TlDup(tl, fnpx);
-	free(fnpx);
-
-	bprintf(buf, "Vmod_%.*s_Data", PF(mod));
-	vmd = dlsym(hdl, buf);
-	if (vmd == NULL) {
-		VSB_printf(tl->sb, "Malformed VMOD %.*s\n", PF(mod));
-		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
-		VSB_printf(tl->sb, "\t(no Vmod_Data symbol)\n");
-		vcc_ErrWhere(tl, mod);
-		return;
-	}
-	if (vmd->vrt_major == 0 && vmd->vrt_minor == 0 &&
-	    strcmp(vmd->abi, VMOD_ABI_Version) != 0) {
-		VSB_printf(tl->sb, "Incompatible VMOD %.*s\n", PF(mod));
-		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
-		VSB_printf(tl->sb, "\tABI mismatch, expected <%s>, got <%s>\n",
-			   VMOD_ABI_Version, vmd->abi);
-		vcc_ErrWhere(tl, mod);
-		return;
-	}
-	if (vmd->vrt_major != 0 && (vmd->vrt_major != VRT_MAJOR_VERSION ||
-	    vmd->vrt_minor > VRT_MINOR_VERSION)) {
-		VSB_printf(tl->sb, "Incompatible VMOD %.*s\n", PF(mod));
-		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
-		VSB_printf(tl->sb, "\tVMOD wants ABI version %u.%u\n",
-		    vmd->vrt_major, vmd->vrt_minor);
-		VSB_printf(tl->sb, "\tvarnishd provides ABI version %u.%u\n",
-		    VRT_MAJOR_VERSION, VRT_MINOR_VERSION);
-		vcc_ErrWhere(tl, mod);
-		return;
-	}
-	if (vmd->name == NULL ||
-	    vmd->func == NULL ||
-	    vmd->func_len <= 0 ||
-	    vmd->proto == NULL ||
-	    vmd->abi == NULL) {
-		VSB_printf(tl->sb, "Mangled VMOD %.*s\n", PF(mod));
-		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
-		VSB_printf(tl->sb, "\tInconsistent metadata\n");
-		vcc_ErrWhere(tl, mod);
+	vmd = vcc_VmodSanity(tl, vop->hdl, mod, fnpx);
+	if (vmd == NULL || tl->err) {
+		AZ(dlclose(vop->hdl));
+		free(fnpx);
 		return;
 	}
 
-	if (!vcc_IdIs(mod, vmd->name)) {
-		VSB_printf(tl->sb, "Wrong VMOD file %.*s\n", PF(mod));
-		VSB_printf(tl->sb, "\tFile name: %s\n", fnp);
-		VSB_printf(tl->sb, "\tContains vmod \"%s\"\n", vmd->name);
-		vcc_ErrWhere(tl, mod);
+	if (msym->extra != NULL) {
+		if (!strcmp(msym->extra, vmd->file_id)) {
+			/* Identical import is OK */
+		} else {
+			VSB_printf(tl->sb,
+			    "Another module already imported as %.*s.\n",
+			    PF(tmod));
+			vcc_ErrWhere2(tl, t1, tl->t);
+		}
+		AZ(dlclose(vop->hdl));
+		free(fnpx);
 		return;
 	}
+	msym->def_b = t1;
+	msym->def_e = tl->t;
+
+	VTAILQ_FOREACH(vsym, &tl->sym_vmods, sideways) {
+		assert(vsym->kind == SYM_VMOD);
+		if (!strcmp(vsym->extra, vmd->file_id)) {
+			/* Already loaded under different name */
+			msym->eval_priv = vsym->eval_priv;
+			msym->wildcard = vsym->wildcard;
+			msym->extra = vsym->extra;
+			msym->vmod_name = vsym->vmod_name;
+			AZ(dlclose(vop->hdl));
+			free(fnpx);
+			return;
+		}
+	}
 
-	if (again && strcmp(vmd->file_id, msym->extra)) {
-		VSB_printf(tl->sb,
-		    "Different version of module %.*s already imported.\n",
-		    PF(mod));
-		vcc_ErrWhere2(tl, t1, tl->t);
-		VSB_printf(tl->sb, "Previous import was here:\n");
-		vcc_ErrWhere2(tl, msym->def_b, msym->def_e);
-	}
-	if (again) {
-		AZ(dlclose(hdl));
-		return;
-	}
+	VTAILQ_INSERT_TAIL(&tl->sym_vmods, msym, sideways);
 
 	ifp = New_IniFin(tl);
 
-	VSB_printf(ifp->ini, "\tif (VRT_Vmod_Init(ctx,\n");
+	VSB_cat(ifp->ini, "\tif (VPI_Vmod_Init(ctx,\n");
 	VSB_printf(ifp->ini, "\t    &VGC_vmod_%.*s,\n", PF(mod));
 	VSB_printf(ifp->ini, "\t    %u,\n", tl->vmod_count++);
 	VSB_printf(ifp->ini, "\t    &%s,\n", vmd->func_name);
 	VSB_printf(ifp->ini, "\t    sizeof(%s),\n", vmd->func_name);
 	VSB_printf(ifp->ini, "\t    \"%.*s\",\n", PF(mod));
-	VSB_printf(ifp->ini, "\t    ");
-	VSB_quote(ifp->ini, fnp, -1, VSB_QUOTE_CSTR);
-	VSB_printf(ifp->ini, ",\n");
+	VSB_cat(ifp->ini, "\t    ");
+	VSB_quote(ifp->ini, fnpx, -1, VSB_QUOTE_CSTR);
+	VSB_cat(ifp->ini, ",\n");
 	AN(vmd);
 	AN(vmd->file_id);
 	VSB_printf(ifp->ini, "\t    \"%s\",\n", vmd->file_id);
 	VSB_printf(ifp->ini, "\t    \"./vmod_cache/_vmod_%.*s.%s\"\n",
 	    PF(mod), vmd->file_id);
-	VSB_printf(ifp->ini, "\t    ))\n");
-	VSB_printf(ifp->ini, "\t\treturn(1);");
+	VSB_cat(ifp->ini, "\t    ))\n");
+	VSB_cat(ifp->ini, "\t\treturn(1);");
 
-	VSB_printf(tl->fi, "%s VMOD %s ./vmod_cache/_vmod_%.*s.%s */\n",
-	    VCC_INFO_PREFIX, fnp, PF(mod), vmd->file_id);
+	VSB_cat(tl->symtab, ",\n    {\n");
+	VSB_cat(tl->symtab, "\t\"dir\": \"import\",\n");
+	VSB_cat(tl->symtab, "\t\"type\": \"$VMOD\",\n");
+	VSB_printf(tl->symtab, "\t\"name\": \"%.*s\",\n", PF(mod));
+	VSB_printf(tl->symtab, "\t\"file\": \"%s\",\n", fnpx);
+	VSB_printf(tl->symtab, "\t\"dst\": \"./vmod_cache/_vmod_%.*s.%s\"\n",
+	    PF(mod), vmd->file_id);
+	VSB_cat(tl->symtab, "    }");
 
 	/* XXX: zero the function pointer structure ?*/
 	VSB_printf(ifp->fin, "\t\tVRT_priv_fini(&vmod_priv_%.*s);", PF(mod));
-	VSB_printf(ifp->final,
-	    "\t\tVRT_Vmod_Unload(ctx, &VGC_vmod_%.*s);", PF(mod));
+	VSB_printf(ifp->final, "\t\tVPI_Vmod_Unload(&VGC_vmod_%.*s);", PF(mod));
 
 	vj = vjsn_parse(vmd->json, &p);
 	XXXAZ(p);
@@ -334,52 +366,44 @@ vcc_ParseImport(struct vcc *tl)
 	msym->eval_priv = vj;
 	msym->wildcard = vcc_json_wildcard;
 	msym->extra = TlDup(tl, vmd->file_id);
+	msym->vmod_name = TlDup(tl, vmd->name);
 
-	vcc_json_always(tl, msym);
+	vcc_json_always(tl, vj, msym->vmod_name);
 
 	Fh(tl, 0, "\n/* --- BEGIN VMOD %.*s --- */\n\n", PF(mod));
 	Fh(tl, 0, "static struct vmod *VGC_vmod_%.*s;\n", PF(mod));
 	Fh(tl, 0, "static struct vmod_priv vmod_priv_%.*s;\n", PF(mod));
 	Fh(tl, 0, "\n%s\n", vmd->proto);
 	Fh(tl, 0, "\n/* --- END VMOD %.*s --- */\n\n", PF(mod));
+	free(fnpx);
 }
 
 void v_matchproto_(sym_act_f)
 vcc_Act_New(struct vcc *tl, struct token *t, struct symbol *sym)
 {
-	struct symbol *sy1, *sy2, *sy3;
+	struct symbol *isym, *osym, *msym;
 	struct inifin *ifp;
 	struct vsb *buf;
 	const struct vjsn_val *vv, *vf;
-	const char *p;
 	int null_ok = 0;
 
 	(void)sym;
+	(void)t;
+
 	ExpectErr(tl, ID);
 	vcc_ExpectVid(tl, "VCL object");
 	ERRCHK(tl);
-	sy1 = VCC_HandleSymbol(tl, INSTANCE, "vo");
+	isym = VCC_HandleSymbol(tl, INSTANCE, "vo");
 	ERRCHK(tl);
-	AN(sy1);
-	sy1->noref = 1;
-
-	ExpectErr(tl, '=');
-	vcc_NextToken(tl);
+	AN(isym);
+	isym->noref = 1;
 
+	SkipToken(tl, '=');
 	ExpectErr(tl, ID);
-	t = tl->t;
-	sy2 = VCC_SymbolGet(tl, SYM_OBJECT, "Symbol not found", XREF_NONE);
+	osym = VCC_SymbolGet(tl, SYM_OBJECT, SYMTAB_EXISTING, XREF_NONE);
 	ERRCHK(tl);
-	AN(sy2);
-	if (sy2->eval_priv == NULL) {
-		VSB_printf(tl->sb, "Constructor not found: ");
-		vcc_ErrToken(tl, t);
-		VSB_printf(tl->sb, " at ");
-		vcc_ErrWhere(tl, t);
-		return;
-	}
-
-	CAST_OBJ_NOTNULL(vv, sy2->eval_priv, VJSN_VAL_MAGIC);
+	AN(osym);
+	CAST_OBJ_NOTNULL(vv, osym->eval_priv, VJSN_VAL_MAGIC);
 	// vv = object name
 
 	vv = VTAILQ_NEXT(vv, list);
@@ -389,12 +413,12 @@ vcc_Act_New(struct vcc *tl, struct token *t, struct symbol *sym)
 		if (!strcmp(vf->name, "NULL_OK") && vf->type == VJSN_TRUE)
 			null_ok = 1;
 	if (!null_ok)
-		VTAILQ_INSERT_TAIL(&tl->sym_objects, sy1, sideways);
+		VTAILQ_INSERT_TAIL(&tl->sym_objects, isym, sideways);
 
 	vv = VTAILQ_NEXT(vv, list);
 	// vv = struct name
 
-	Fh(tl, 0, "static %s *%s;\n\n", vv->value, sy1->rname);
+	Fh(tl, 0, "static %s *%s;\n\n", vv->value, isym->rname);
 	vv = VTAILQ_NEXT(vv, list);
 
 	vf = VTAILQ_FIRST(&vv->children);
@@ -406,13 +430,13 @@ vcc_Act_New(struct vcc *tl, struct token *t, struct symbol *sym)
 
 	buf = VSB_new_auto();
 	AN(buf);
-	VSB_printf(buf, ", &%s, \"%s\"", sy1->rname, sy1->name);
+	VSB_printf(buf, "&%s, \"%s\"", isym->rname, isym->name);
 	AZ(VSB_finish(buf));
-	vcc_Eval_Func(tl, vf, VSB_data(buf), sy2);
+	vcc_Eval_Func(tl, vf, VSB_data(buf), osym);
 	VSB_destroy(&buf);
 	ERRCHK(tl);
 	SkipToken(tl, ';');
-	sy1->def_e = tl->t;
+	isym->def_e = tl->t;
 
 	vf = VTAILQ_FIRST(&vv->children);
 	vv = VTAILQ_NEXT(vv, list);
@@ -423,15 +447,13 @@ vcc_Act_New(struct vcc *tl, struct token *t, struct symbol *sym)
 	vf = VTAILQ_FIRST(&vf->children);
 	vf = VTAILQ_NEXT(vf, list);
 	ifp = New_IniFin(tl);
-	VSB_printf(ifp->fin, "\t\tif (%s)\n", sy1->rname);
-	VSB_printf(ifp->fin, "\t\t\t\t%s(&%s);", vf->value, sy1->rname);
+	VSB_printf(ifp->fin, "\t\tif (%s)\n", isym->rname);
+	VSB_printf(ifp->fin, "\t\t\t\t%s(&%s);", vf->value, isym->rname);
 
 	/* Instantiate symbols for the methods */
 	buf = VSB_new_auto();
 	AN(buf);
-	VSB_printf(buf, ", %s", sy1->rname);
-	AZ(VSB_finish(buf));
-	p = TlDup(tl, VSB_data(buf));
+
 	while (vv != NULL) {
 		vf = VTAILQ_FIRST(&vv->children);
 		assert(vf->type == VJSN_STRING);
@@ -440,12 +462,12 @@ vcc_Act_New(struct vcc *tl, struct token *t, struct symbol *sym)
 		assert(vf->type == VJSN_STRING);
 
 		VSB_clear(buf);
-		VSB_printf(buf, "%s.%s", sy1->name, vf->value);
+		VSB_printf(buf, "%s.%s", isym->name, vf->value);
 		AZ(VSB_finish(buf));
-		sy3 = VCC_MkSym(tl, VSB_data(buf), SYM_FUNC, VCL_LOW, VCL_HIGH);
-		AN(sy3);
-		func_sym(sy3, sy2->vmod, VTAILQ_NEXT(vf, list));
-		sy3->extra = p;
+		msym = VCC_MkSym(tl, VSB_data(buf), SYM_FUNC, VCL_LOW, VCL_HIGH);
+		AN(msym);
+		func_sym(msym, osym->vmod_name, VTAILQ_NEXT(vf, list));
+		msym->extra = isym->rname;
 		vv = VTAILQ_NEXT(vv, list);
 	}
 	VSB_destroy(&buf);
