@@ -1,9 +1,11 @@
 /*-
- * Copyright (c) 2012-2015 Varnish Software AS
+ * Copyright (c) 2012-2019 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@FreeBSD.org>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -47,16 +49,21 @@ struct priv_vcl {
 #define PRIV_VCL_MAGIC		0x8E62FA9D
 	char			*foo;
 	uintptr_t		obj_cb;
-	struct vcl		*vcl;
-	struct vclref		*vclref;
+	struct vclref		*vclref_discard;
+	struct vclref		*vclref_cold;
+	VCL_DURATION		vcl_discard_delay;
+	VCL_BACKEND		be;
+	unsigned		cold_be;
+	unsigned		cooling_be;
 };
 
-static VCL_DURATION vcl_release_delay = 0.0;
 
 static pthread_mutex_t vsc_mtx = PTHREAD_MUTEX_INITIALIZER;
 static struct vsc_seg *vsc_seg = NULL;
 static struct VSC_debug *vsc = NULL;
 static int loads;
+static const int store_ip_token;
+static const int fail_rollback_token;
 
 /**********************************************************************/
 
@@ -89,6 +96,74 @@ static const struct vfp xyzzy_rot13 = {
 
 /**********************************************************************/
 
+#define ROT13_BUFSZ (1 << 13)
+
+static int v_matchproto_(vdp_init_f)
+xyzzy_rot13_init(struct req *req, void **priv)
+{
+	(void)req;
+	AN(priv);
+	*priv = malloc(ROT13_BUFSZ);
+	if (*priv == NULL)
+		return (-1);
+	return (0);
+}
+
+static int v_matchproto_(vdp_bytes_f)
+xyzzy_rot13_bytes(struct req *req, enum vdp_action act, void **priv,
+    const void *ptr, ssize_t len)
+{
+	char *q;
+	const char *pp;
+	int i, j, retval = 0;
+
+	(void)act;
+	AN(priv);
+	AN(*priv);
+	AN(ptr);
+	if (len <= 0)
+		return (0);
+	q = *priv;
+	pp = ptr;
+
+	for (i = 0, j = 0; j < len; i++, j++) {
+		if (pp[j] >= 'A' && pp[j] <= 'Z')
+			q[i] = (((pp[j] - 'A') + 13) % 26) + 'A';
+		else if (pp[j] >= 'a' && pp[j] <= 'z')
+			q[i] = (((pp[j] - 'a') + 13) % 26) + 'a';
+		else
+			q[i] = pp[j];
+		if (i == ROT13_BUFSZ - 1) {
+			retval = VDP_bytes(req, VDP_FLUSH, q, ROT13_BUFSZ);
+			if (retval != 0)
+				return (retval);
+			i = -1;
+		}
+	}
+	if (i >= 0)
+		retval = VDP_bytes(req, VDP_FLUSH, q, i + 1L);
+	return (retval);
+}
+
+static int v_matchproto_(vdp_fini_f)
+xyzzy_rot13_fini(struct req *req, void **priv)
+{
+	(void)req;
+	AN(priv);
+	free(*priv);
+	*priv = NULL;
+	return (0);
+}
+
+static const struct vdp xyzzy_vdp_rot13 = {
+	.name  = "rot13",
+	.init  = xyzzy_rot13_init,
+	.bytes = xyzzy_rot13_bytes,
+	.fini  = xyzzy_rot13_fini,
+};
+
+/**********************************************************************/
+
 VCL_STRING v_matchproto_(td_debug_author)
 xyzzy_author(VRT_CTX, VCL_ENUM person, VCL_ENUM someone)
 {
@@ -273,8 +348,8 @@ priv_vcl_free(void *priv)
 		ObjUnsubscribeEvents(&priv_vcl->obj_cb);
 		VSL(SLT_Debug, 0, "Unsubscribed from Object Events");
 	}
-	AZ(priv_vcl->vcl);
-	AZ(priv_vcl->vclref);
+	AZ(priv_vcl->vclref_discard);
+	AZ(priv_vcl->vclref_cold);
 	FREE_OBJ(priv_vcl);
 	AZ(priv_vcl);
 }
@@ -289,7 +364,7 @@ event_load(VRT_CTX, struct vmod_priv *priv)
 	loads++;
 
 	if (cache_param->nuke_limit == 42) {
-		VSB_printf(ctx->msg, "nuke_limit is not the answer.");
+		VSB_cat(ctx->msg, "nuke_limit is not the answer.");
 		return (-1);
 	}
 
@@ -305,9 +380,65 @@ event_load(VRT_CTX, struct vmod_priv *priv)
 	 * API should look like, do NOT do this anywhere else.
 	 */
 	VRT_AddVFP(ctx, &xyzzy_rot13);
+
+	VRT_AddVDP(ctx, &xyzzy_vdp_rot13);
 	return (0);
 }
 
+VCL_VOID
+xyzzy_vcl_prevent_cold(VRT_CTX, struct vmod_priv *priv)
+{
+	struct priv_vcl *priv_vcl;
+	char buf[32];
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(priv);
+	CAST_OBJ_NOTNULL(priv_vcl, priv->priv, PRIV_VCL_MAGIC);
+	AZ(priv_vcl->vclref_cold);
+
+	bprintf(buf, "vmod-debug ref on %s", VCL_Name(ctx->vcl));
+	priv_vcl->vclref_cold = VRT_VCL_Prevent_Cold(ctx, buf);
+}
+
+VCL_VOID
+xyzzy_vcl_allow_cold(VRT_CTX, struct vmod_priv *priv)
+{
+	struct priv_vcl *priv_vcl;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(priv);
+	CAST_OBJ_NOTNULL(priv_vcl, priv->priv, PRIV_VCL_MAGIC);
+	AN(priv_vcl->vclref_cold);
+	VRT_VCL_Allow_Cold(&priv_vcl->vclref_cold);
+}
+
+VCL_VOID
+xyzzy_cold_backend(VRT_CTX, struct vmod_priv *priv)
+{
+	struct priv_vcl *priv_vcl;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(priv);
+	CAST_OBJ_NOTNULL(priv_vcl, priv->priv, PRIV_VCL_MAGIC);
+	priv_vcl->cold_be = 1;
+}
+
+VCL_VOID
+xyzzy_cooling_backend(VRT_CTX, struct vmod_priv *priv)
+{
+	struct priv_vcl *priv_vcl;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(priv);
+	CAST_OBJ_NOTNULL(priv_vcl, priv->priv, PRIV_VCL_MAGIC);
+	priv_vcl->cooling_be = 1;
+}
+
+static const struct vdi_methods empty_methods[1] = {{
+	.magic =	VDI_METHODS_MAGIC,
+	.type =	"debug.dummy"
+}};
+
 static int
 event_warm(VRT_CTX, const struct vmod_priv *priv)
 {
@@ -318,54 +449,79 @@ event_warm(VRT_CTX, const struct vmod_priv *priv)
 
 	AN(ctx->msg);
 	if (cache_param->max_esi_depth == 42) {
-		VSB_printf(ctx->msg, "max_esi_depth is not the answer.");
+		VSB_cat(ctx->msg, "max_esi_depth is not the answer.");
 		return (-1);
 	}
 
 	CAST_OBJ_NOTNULL(priv_vcl, priv->priv, PRIV_VCL_MAGIC);
-	AZ(priv_vcl->vcl);
-	AZ(priv_vcl->vclref);
+	AZ(priv_vcl->vclref_discard);
+
+	if (!priv_vcl->cold_be) {
+		/* NB: set up a COOLING step unless we want a COLD backend. */
+		bprintf(buf, "vmod-debug ref on %s", VCL_Name(ctx->vcl));
+		priv_vcl->vclref_discard = VRT_VCL_Prevent_Discard(ctx, buf);
+	}
+
+	AZ(priv_vcl->be);
+	priv_vcl->be = VRT_AddDirector(ctx, empty_methods,
+	    NULL, "%s", "dir_warmcold");
 
-	bprintf(buf, "vmod-debug ref on %s", VCL_Name(ctx->vcl));
-	priv_vcl->vcl = ctx->vcl;
-	priv_vcl->vclref = VRT_ref_vcl(ctx, buf);
 	return (0);
 }
 
 static void*
 cooldown_thread(void *priv)
 {
-	struct vrt_ctx ctx;
 	struct priv_vcl *priv_vcl;
 
 	CAST_OBJ_NOTNULL(priv_vcl, priv, PRIV_VCL_MAGIC);
-	AN(priv_vcl->vcl);
-	AN(priv_vcl->vclref);
+	AN(priv_vcl->vclref_discard);
 
-	INIT_OBJ(&ctx, VRT_CTX_MAGIC);
-	ctx.vcl = priv_vcl->vcl;
-
-	VTIM_sleep(vcl_release_delay);
-	VRT_rel_vcl(&ctx, &priv_vcl->vclref);
-	priv_vcl->vcl = NULL;
+	VTIM_sleep(priv_vcl->vcl_discard_delay);
+	VRT_VCL_Allow_Discard(&priv_vcl->vclref_discard);
 	return (NULL);
 }
 
+static VCL_BACKEND
+create_cold_backend(VRT_CTX)
+{
+	struct vrt_backend be[1];
+
+	INIT_OBJ(be, VRT_BACKEND_MAGIC);
+	be->path = "/";
+	be->vcl_name = "doomed";
+	return (VRT_new_backend(ctx, be));
+}
+
 static int
 event_cold(VRT_CTX, const struct vmod_priv *priv)
 {
 	pthread_t thread;
 	struct priv_vcl *priv_vcl;
 
+	AZ(ctx->msg);
+
 	CAST_OBJ_NOTNULL(priv_vcl, priv->priv, PRIV_VCL_MAGIC);
-	AN(priv_vcl->vcl);
-	AN(priv_vcl->vclref);
 
 	VSL(SLT_Debug, 0, "%s: VCL_EVENT_COLD", VCL_Name(ctx->vcl));
 
-	if (vcl_release_delay == 0.0) {
-		VRT_rel_vcl(ctx, &priv_vcl->vclref);
-		priv_vcl->vcl = NULL;
+	VRT_DelDirector(&priv_vcl->be);
+
+	if (priv_vcl->cold_be) {
+		AZ(priv_vcl->vclref_discard);
+		priv_vcl->be = create_cold_backend(ctx);
+		WRONG("unreachable");
+	}
+
+	if (priv_vcl->cooling_be) {
+		AN(priv_vcl->vclref_discard);
+		priv_vcl->be = create_cold_backend(ctx);
+		AZ(priv_vcl->be);
+	}
+
+	if (priv_vcl->vcl_discard_delay == 0.0) {
+		AN(priv_vcl->vclref_discard);
+		VRT_VCL_Allow_Discard(&priv_vcl->vclref_discard);
 		return (0);
 	}
 
@@ -380,7 +536,10 @@ event_discard(VRT_CTX, void *priv)
 
 	(void)priv;
 
+	AZ(ctx->msg);
+
 	VRT_RemoveVFP(ctx, &xyzzy_rot13);
+	VRT_RemoveVDP(ctx, &xyzzy_vdp_rot13);
 
 	if (--loads)
 		return (0);
@@ -406,17 +565,19 @@ xyzzy_event_function(VRT_CTX, struct vmod_priv *priv, enum vcl_event_e e)
 	case VCL_EVENT_WARM:	return (event_warm(ctx, priv));
 	case VCL_EVENT_COLD:	return (event_cold(ctx, priv));
 	case VCL_EVENT_DISCARD:	return (event_discard(ctx, priv));
-	default: return (0);
+	default: WRONG("we should test all possible events");
 	}
 }
 
-VCL_VOID v_matchproto_(td_debug_vcl_release_delay)
-xyzzy_vcl_release_delay(VRT_CTX, VCL_DURATION delay)
+VCL_VOID v_matchproto_(td_debug_vcl_discard_delay)
+xyzzy_vcl_discard_delay(VRT_CTX, struct vmod_priv *priv, VCL_DURATION delay)
 {
+	struct priv_vcl *priv_vcl;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	CAST_OBJ_NOTNULL(priv_vcl, priv->priv, PRIV_VCL_MAGIC);
 	assert(delay > 0.0);
-	vcl_release_delay = delay;
+	priv_vcl->vcl_discard_delay = delay;
 }
 
 VCL_BOOL v_matchproto_(td_debug_match_acl)
@@ -424,7 +585,7 @@ xyzzy_match_acl(VRT_CTX, VCL_ACL acl, VCL_IP ip)
 {
 
 	CHECK_OBJ_ORNULL(ctx, VRT_CTX_MAGIC);
-	CHECK_OBJ_ORNULL(acl, VRT_ACL_MAGIC);
+	AN(acl);
 	assert(VSA_Sane(ip));
 
 	return (VRT_acl_match(ctx, acl, ip));
@@ -514,11 +675,7 @@ xyzzy_concat__fini(struct xyzzy_debug_concat **concatp)
 	struct xyzzy_debug_concat *concat;
 	void *p;
 
-	if (concatp == NULL || *concatp == NULL)
-		return;
-	CHECK_OBJ(*concatp, CONCAT_MAGIC);
-	concat = *concatp;
-	*concatp = NULL;
+	TAKE_OBJ_NOTNULL(concat, concatp, CONCAT_MAGIC);
 	p = TRUST_ME(concat->s);
 	free(p);
 	FREE_OBJ(concat);
@@ -689,7 +846,10 @@ xyzzy_catflap(VRT_CTX, VCL_ENUM type)
 	CHECK_OBJ_NOTNULL(req, REQ_MAGIC);
 	XXXAZ(req->vcf);
 	req->vcf = WS_Alloc(req->ws, sizeof *req->vcf);
-	AN(req->vcf);
+	if (req->vcf == NULL) {
+		VRT_fail(ctx, "WS_Alloc failed in debug.catflap()");
+		return;
+	}
 	INIT_OBJ(req->vcf, VCF_MAGIC);
 	if (type == VENUM(first) || type == VENUM(miss)) {
 		req->vcf->func = xyzzy_catflap_simple;
@@ -719,3 +879,268 @@ xyzzy_stk(VRT_CTX)
 
 	return (0);
 }
+
+VCL_VOID
+xyzzy_sndbuf(VRT_CTX, VCL_BYTES arg)
+{
+	int fd = -1, oldbuf, newbuf, buflen;
+	socklen_t intlen = sizeof(int);
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	if (ctx->bo) {
+		CHECK_OBJ(ctx->bo, BUSYOBJ_MAGIC);
+		INCOMPL();
+	}
+	else if (ctx->req) {
+		CHECK_OBJ(ctx->req, REQ_MAGIC);
+		CHECK_OBJ(ctx->req->sp, SESS_MAGIC);
+		fd = ctx->req->sp->fd;
+	}
+	else {
+		VRT_fail(ctx, "debug.sndbuf() called outside a transaction.");
+		return;
+	}
+
+	xxxassert(fd >= 0);
+
+	if (arg > INT_MAX)
+		buflen = INT_MAX;
+	else if (arg <= 0)
+		buflen = 0;
+	else
+		buflen = (int)arg;
+
+	oldbuf = 0;
+	AZ(getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &oldbuf, &intlen));
+
+	newbuf = buflen;
+	AZ(setsockopt(fd, SOL_SOCKET, SO_SNDBUF, &newbuf, intlen));
+	AZ(getsockopt(fd, SOL_SOCKET, SO_SNDBUF, &newbuf, &intlen));
+
+	AN(ctx->vsl);
+	VSLb(ctx->vsl, SLT_Debug, "SO_SNDBUF fd=%d old=%d new=%d actual=%d",
+	    fd, oldbuf, buflen, newbuf);
+}
+
+VCL_VOID
+xyzzy_store_ip(VRT_CTX, VCL_IP ip)
+{
+	struct vmod_priv *priv;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	priv = VRT_priv_task(ctx, &store_ip_token);
+	if (priv == NULL) {
+		VRT_fail(ctx, "no priv task - out of ws?");
+		return;
+	}
+
+	AZ(priv->free);
+	assert(VSA_Sane(ip));
+	priv->priv = TRUST_ME(ip);
+}
+
+VCL_IP
+xyzzy_get_ip(VRT_CTX)
+{
+	struct vmod_priv *priv;
+	VCL_IP ip;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	priv = VRT_priv_task(ctx, &store_ip_token);
+	AN(priv);
+	AZ(priv->free);
+
+	ip = priv->priv;
+	assert(VSA_Sane(ip));
+	return (ip);
+}
+
+/**********************************************************************
+ * For testing import code on bad vmod files (m00003.vtc)
+ */
+
+//lint -save -e9075 external symbol '...' defined without a prior declaration
+
+const struct vmod_data Vmod_wrong0_Data = {
+	.vrt_major =    0,
+	.vrt_minor =    0,
+};
+
+//lint -save -e835 A zero has been given as left argument to operatorp'+'
+const struct vmod_data Vmod_wrong1_Data = {
+	.vrt_major =    VRT_MAJOR_VERSION,
+	.vrt_minor =    VRT_MINOR_VERSION + 1,
+};
+//lint -restore
+
+static const struct foo {
+	int bar;
+} foo_struct[1];
+
+const struct vmod_data Vmod_wrong2_Data = {
+	.vrt_major =    VRT_MAJOR_VERSION,
+	.vrt_minor =    VRT_MINOR_VERSION,
+	.name =		"wrongN",
+	.func =		foo_struct,
+	.func_len =	sizeof foo_struct,
+	.func_name =	"foo_struct",
+	.proto =	"blablabla",
+};
+
+const struct vmod_data Vmod_wrong3_Data = {
+	.vrt_major =    VRT_MAJOR_VERSION,
+	.vrt_minor =    VRT_MINOR_VERSION,
+	.name =		"wrongN",
+	.func =		foo_struct,
+	.func_len =	sizeof foo_struct,
+	.func_name =	"foo_struct",
+	.proto =	"blablabla",
+	.abi =		"abiblabla",
+};
+
+//lint -restore
+
+/*---------------------------------------------------------------------*/
+
+struct VPFX(debug_director) {
+	unsigned			magic;
+#define VMOD_DEBUG_DIRECTOR_MAGIC	0x66b9ff3d
+	VCL_BACKEND			dir;
+};
+
+/* XXX more callbacks ? */
+static vdi_healthy_f vmod_debug_director_healthy;
+static vdi_resolve_f vmod_debug_director_resolve;
+
+static const struct vdi_methods vmod_debug_director_methods[1] = {{
+	.magic =	VDI_METHODS_MAGIC,
+	.type =		"debug.director",
+	.resolve =	vmod_debug_director_resolve,
+	.healthy =	vmod_debug_director_healthy
+}};
+
+VCL_VOID v_matchproto_(td_xyzzy_debug_director__init)
+xyzzy_director__init(VRT_CTX, struct VPFX(debug_director) **dp,
+    const char *vcl_name)
+{
+	struct VPFX(debug_director) *d;
+
+	AN(dp);
+	AZ(*dp);
+	ALLOC_OBJ(d, VMOD_DEBUG_DIRECTOR_MAGIC);
+	AN(d);
+
+	*dp = d;
+	d->dir = VRT_AddDirector(ctx, vmod_debug_director_methods, d,
+	    "%s", vcl_name);
+}
+
+VCL_VOID v_matchproto_(td_xyzzy_debug_director__fini)
+xyzzy_director__fini(struct VPFX(debug_director) **dp)
+{
+	struct VPFX(debug_director) *d;
+
+	TAKE_OBJ_NOTNULL(d, dp, VMOD_DEBUG_DIRECTOR_MAGIC);
+	VRT_DelDirector(&d->dir);
+	FREE_OBJ(d);
+}
+
+VCL_BACKEND v_matchproto_(td_xyzzy_debug_director_fail)
+xyzzy_director_fail(VRT_CTX, struct VPFX(debug_director) *d)
+{
+	CHECK_OBJ_NOTNULL(d, VMOD_DEBUG_DIRECTOR_MAGIC);
+	(void) ctx;
+
+	return (d->dir);
+}
+
+static VCL_BOOL v_matchproto_(vdi_healthy_f)
+vmod_debug_director_healthy(VRT_CTX, VCL_BACKEND dir, VCL_TIME *changed)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	(void) dir;
+	(void) changed;
+
+	VRT_fail(ctx, "fail");
+	return (1);
+}
+
+static VCL_BACKEND v_matchproto_(vdi_resolve_f)
+vmod_debug_director_resolve(VRT_CTX, VCL_BACKEND dir)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	(void) dir;
+
+	VRT_fail(ctx, "fail");
+	return (NULL);
+}
+
+VCL_STRING v_matchproto_(td_xyzzy_debug_client_ip)
+xyzzy_client_ip(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	return (SES_Get_String_Attr(ctx->sp, SA_CLIENT_IP));
+}
+
+VCL_STRING v_matchproto_(td_xyzzy_debug_client_port)
+xyzzy_client_port(VRT_CTX)
+{
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	return (SES_Get_String_Attr(ctx->sp, SA_CLIENT_PORT));
+}
+
+static void
+fail_f(void *priv)
+{
+	VRT_CTX;
+
+	CAST_OBJ_NOTNULL(ctx, priv, VRT_CTX_MAGIC);
+	VRT_fail(ctx, "thou shalt not rollet back");
+}
+
+VCL_VOID v_matchproto_(td_xyzzy_debug_fail_rollback)
+xyzzy_fail_rollback(VRT_CTX)
+{
+	struct vmod_priv *p;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	p = VRT_priv_task(ctx, &fail_rollback_token);
+	if (p == NULL) {
+		VRT_fail(ctx, "no priv task - out of ws?");
+		return;
+	}
+
+	if (p->priv != NULL) {
+		assert(p->priv == ctx);
+		assert(p->free == fail_f);
+		return;
+	}
+
+	p->priv = TRUST_ME(ctx);
+	p->free = fail_f;
+}
+
+VCL_VOID v_matchproto_(td_xyzzy_debug_ok_rollback)
+xyzzy_ok_rollback(VRT_CTX)
+{
+	struct vmod_priv *p;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	p = VRT_priv_task(ctx, &fail_rollback_token);
+	if (p == NULL) {
+		VRT_fail(ctx, "no priv task - out of ws?");
+		return;
+	}
+
+	p->priv = NULL;
+	p->free = NULL;
+}
