@@ -4,6 +4,8 @@
  *
  * Author: Dridi Boukelmoune <dridi@varnish-software.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -32,13 +34,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/socket.h>
-#include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
 #include "cache/cache.h"
 
 #include "vsa.h"
+#include "vss.h"
 #include "vcc_if.h"
 
 struct xyzzy_debug_dyn {
@@ -61,7 +63,6 @@ static void
 dyn_dir_init(VRT_CTX, struct xyzzy_debug_dyn *dyn,
      VCL_STRING addr, VCL_STRING port, VCL_PROBE probe)
 {
-	struct addrinfo hints, *res = NULL;
 	struct suckaddr *sa;
 	VCL_BACKEND dir, dir2;
 	struct vrt_backend vrt;
@@ -76,13 +77,7 @@ dyn_dir_init(VRT_CTX, struct xyzzy_debug_dyn *dyn,
 	vrt.hosthdr = addr;
 	vrt.probe = probe;
 
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = AF_UNSPEC;
-	hints.ai_socktype = SOCK_STREAM;
-	AZ(getaddrinfo(addr, port, &hints, &res));
-	XXXAZ(res->ai_next);
-
-	sa = VSA_Malloc(res->ai_addr, res->ai_addrlen);
+	sa = VSS_ResolveOne(NULL, addr, port, AF_UNSPEC, SOCK_STREAM, 0);
 	AN(sa);
 	if (VSA_Get_Proto(sa) == AF_INET) {
 		vrt.ipv4_addr = addr;
@@ -93,8 +88,6 @@ dyn_dir_init(VRT_CTX, struct xyzzy_debug_dyn *dyn,
 	} else
 		WRONG("Wrong proto family");
 
-	freeaddrinfo(res);
-
 	dir = VRT_new_backend(ctx, &vrt);
 	AN(dir);
 
@@ -144,21 +137,16 @@ xyzzy_dyn__init(VRT_CTX, struct xyzzy_debug_dyn **dynp,
 	*dynp = dyn;
 }
 
-VCL_VOID
+VCL_VOID v_matchproto_(td_xyzzy_debug_dyn__fini)
 xyzzy_dyn__fini(struct xyzzy_debug_dyn **dynp)
 {
 	struct xyzzy_debug_dyn *dyn;
 
-	AN(dynp);
-	if (*dynp == NULL)
-		return; /* failed initialization */
-
-	CAST_OBJ_NOTNULL(dyn, *dynp, VMOD_DEBUG_DYN_MAGIC);
+	TAKE_OBJ_NOTNULL(dyn, dynp, VMOD_DEBUG_DYN_MAGIC);
 	/* at this point all backends will be deleted by the vcl */
 	free(dyn->vcl_name);
 	AZ(pthread_mutex_destroy(&dyn->mtx));
 	FREE_OBJ(dyn);
-	*dynp = NULL;
 }
 
 VCL_BACKEND v_matchproto_()
@@ -194,6 +182,11 @@ dyn_uds_init(VRT_CTX, struct xyzzy_debug_dyn_uds *uds, VCL_STRING path)
 		VRT_fail(ctx, "path must be an absolute path: %s", path);
 		return (-1);
 	}
+
+	/* XXX: now that we accept that sockets may come after vcl.load
+	 * and change during the life of a VCL, do we still need to check
+	 * this? It looks like both if blocks can be retired.
+	 */
 	errno = 0;
 	if (stat(path, &st) != 0) {
 		VRT_fail(ctx, "Cannot stat path %s: %s", path, strerror(errno));
@@ -224,9 +217,9 @@ dyn_uds_init(VRT_CTX, struct xyzzy_debug_dyn_uds *uds, VCL_STRING path)
 	return (0);
 }
 
-VCL_VOID v_matchproto_(td_debug_dyn_uds__init)
+VCL_VOID v_matchproto_(td_xyzzy_debug_dyn_uds__init)
 xyzzy_dyn_uds__init(VRT_CTX, struct xyzzy_debug_dyn_uds **udsp,
-		    const char *vcl_name, VCL_STRING path)
+    const char *vcl_name, VCL_STRING path)
 {
 	struct xyzzy_debug_dyn_uds *uds;
 
@@ -246,6 +239,7 @@ xyzzy_dyn_uds__init(VRT_CTX, struct xyzzy_debug_dyn_uds **udsp,
 		FREE_OBJ(uds);
 		return;
 	}
+
 	*udsp = uds;
 }
 
@@ -254,14 +248,10 @@ xyzzy_dyn_uds__fini(struct xyzzy_debug_dyn_uds **udsp)
 {
 	struct xyzzy_debug_dyn_uds *uds;
 
-	if (udsp == NULL || *udsp == NULL)
-		return;
-	CHECK_OBJ(*udsp, VMOD_DEBUG_UDS_MAGIC);
-	uds = *udsp;
+	TAKE_OBJ_NOTNULL(uds, udsp, VMOD_DEBUG_UDS_MAGIC);
 	free(uds->vcl_name);
 	AZ(pthread_mutex_destroy(&uds->mtx));
 	FREE_OBJ(uds);
-	*udsp = NULL;
 }
 
 VCL_BACKEND v_matchproto_(td_debug_dyn_uds_backend)
