@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@FreeBSD.org>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -65,26 +67,25 @@ vmod_set_ip_tos(VRT_CTX, VCL_INT tos)
 }
 
 static const char *
-vmod_updown(VRT_CTX, int up, const char *s, va_list ap)
+vmod_updown(VRT_CTX, int up, VCL_STRANDS s)
 {
 	unsigned u;
 	char *b, *e;
 	const char *p;
+	int i;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	u = WS_Reserve(ctx->ws, 0);
+	u = WS_ReserveAll(ctx->ws);
 	e = b = ctx->ws->f;
 	e += u;
-	p = s;
-	while (p != vrt_magic_string_end && b < e) {
-		if (p != NULL) {
-			for (; b < e && *p != '\0'; p++)
-				if (up)
-					*b++ = (char)toupper(*p);
-				else
-					*b++ = (char)tolower(*p);
+	for (i = 0; i < s->n && b < e; i++) {
+		p = s->p[i];
+		while (p != NULL && *p != '\0' && b < e) {
+			if (up)
+				*b++ = (char)toupper(*p++);
+			else
+				*b++ = (char)tolower(*p++);
 		}
-		p = va_arg(ap, const char *);
 	}
 	if (b < e)
 		*b = '\0';
@@ -102,29 +103,19 @@ vmod_updown(VRT_CTX, int up, const char *s, va_list ap)
 }
 
 VCL_STRING v_matchproto_(td_std_toupper)
-vmod_toupper(VRT_CTX, const char *s, ...)
+vmod_toupper(VRT_CTX, VCL_STRANDS s)
 {
-	const char *p;
-	va_list ap;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	va_start(ap, s);
-	p = vmod_updown(ctx, 1, s, ap);
-	va_end(ap);
-	return (p);
+	return (vmod_updown(ctx, 1, s));
 }
 
 VCL_STRING v_matchproto_(td_std_tolower)
-vmod_tolower(VRT_CTX, const char *s, ...)
+vmod_tolower(VRT_CTX, VCL_STRANDS s)
 {
-	const char *p;
-	va_list ap;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	va_start(ap, s);
-	p = vmod_updown(ctx, 0, s, ap);
-	va_end(ap);
-	return (p);
+	return (vmod_updown(ctx, 0, s));
 }
 
 VCL_REAL v_matchproto_(td_std_random)
@@ -140,45 +131,26 @@ vmod_random(VRT_CTX, VCL_REAL lo, VCL_REAL hi)
 }
 
 VCL_VOID v_matchproto_(td_std_log)
-vmod_log(VRT_CTX, const char *fmt, ...)
+vmod_log(VRT_CTX, VCL_STRANDS s)
 {
-	const char *p;
-	va_list ap;
-	uintptr_t sn;
-
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
-	sn = WS_Snapshot(ctx->ws);
-	va_start(ap, fmt);
-	p = VRT_String(ctx->ws, NULL, fmt, ap);
-	va_end(ap);
 
-	if (p == NULL) {
-		WS_Reset(ctx->ws, sn);
-		WS_MarkOverflow(ctx->ws);
-		return;
-	}
-
-	AN(p);
 	if (ctx->vsl != NULL)
-		VSLb(ctx->vsl, SLT_VCL_Log, "%s", p);
+		VSLbs(ctx->vsl, SLT_VCL_Log, s);
 	else
-		VSL(SLT_VCL_Log, 0, "%s", p);
-	WS_Reset(ctx->ws, sn);
+		VSLs(SLT_VCL_Log, 0, s);
 }
 
 /* XXX use vsyslog() ? */
 VCL_VOID v_matchproto_(td_std_syslog)
-vmod_syslog(VRT_CTX, VCL_INT fac, const char *fmt, ...)
+vmod_syslog(VRT_CTX, VCL_INT fac, VCL_STRANDS s)
 {
 	const char *p;
-	va_list ap;
 	uintptr_t sn;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	sn = WS_Snapshot(ctx->ws);
-	va_start(ap, fmt);
-	p = VRT_String(ctx->ws, NULL, fmt, ap);
-	va_end(ap);
+	p = VRT_StrandsWS(ctx->ws, NULL, s);
 	if (p != NULL)
 		syslog((int)fac, "%s", p);
 	WS_Reset(ctx->ws, sn);
