@@ -4,6 +4,8 @@
  *
  * Author: Poul-Henning Kamp <phk@FreeBSD.org>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -40,6 +42,7 @@
 
 #include "vnum.h"
 #include "vsa.h"
+#include "vss.h"
 #include "vtim.h"
 #include "vcc_if.h"
 
@@ -189,39 +192,11 @@ vmod_integer(VRT_CTX, struct VARGS(integer) *a)
 	return (0);
 }
 
-static VCL_IP
-lookup(VCL_IP *p, const char *s, int resolve)
-{
-	struct addrinfo hints, *res0 = NULL;
-	const struct addrinfo *res;
-	int error;
-	VCL_IP retval = NULL;
-
-	if (s == NULL)
-		return (retval);
-
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = PF_UNSPEC;
-	hints.ai_socktype = SOCK_STREAM;
-	if (!resolve)
-		hints.ai_flags |= AI_NUMERICHOST;
-	error = getaddrinfo(s, "80", &hints, &res0);
-	if (!error) {
-		for (res = res0; res != NULL; res = res->ai_next) {
-			retval = VSA_Build(p, res->ai_addr, res->ai_addrlen);
-			if (retval != NULL)
-				break;
-		}
-		freeaddrinfo(res0);
-	}
-	return (retval);
-}
-
 VCL_IP
 vmod_ip(VRT_CTX, struct VARGS(ip) *a)
 {
 	void *p;
-	VCL_IP retval;
+	VCL_IP retval = NULL;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	if (a->valid_fallback)
@@ -229,12 +204,16 @@ vmod_ip(VRT_CTX, struct VARGS(ip) *a)
 
 	p = WS_Alloc(ctx->ws, vsa_suckaddr_len);
 	if (p == NULL) {
-		VSLb(ctx->vsl, SLT_VCL_Error,
-		    "vmod std.ip(): insufficient workspace");
+		VRT_fail(ctx, "std.ip: insufficient workspace");
 		return (NULL);
 	}
 
-	retval = lookup(p, a->s, a->resolve);
+	if (a->s != NULL)
+		retval = VSS_ResolveFirst(
+		    p, a->s, a->valid_p ? a->p : "80",
+		    AF_UNSPEC, SOCK_STREAM,
+		    a->resolve ? 0 : AI_NUMERICHOST|AI_NUMERICSERV);
+
 	if (retval != NULL)
 		return (retval);
 
@@ -243,7 +222,7 @@ vmod_ip(VRT_CTX, struct VARGS(ip) *a)
 	if (a->valid_fallback)
 		return (a->fallback);
 
-	VRT_fail(ctx, "std.integer: conversion failed");
+	VRT_fail(ctx, "std.ip: conversion failed");
 	return (NULL);
 }
 
