@@ -5,6 +5,8 @@
  * Author: Poul-Henning Kamp <phk@FreeBSD.org>
  * Author: Dridi Boukelmoune <dridi.boukelmoune@gmail.com>
  *
+ * SPDX-License-Identifier: BSD-2-Clause
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
@@ -35,6 +37,7 @@
 
 #include "cache/cache.h"
 
+#include "vsb.h"
 #include "vtcp.h"
 #include "vtim.h"
 
@@ -98,17 +101,14 @@ vmod_no_ip(VRT_CTX)
 
 /*--------------------------------------------------------------------*/
 
-VCL_VOID v_matchproto_(td_vtc_panic)
-vmod_panic(VRT_CTX, const char *str, ...)
+VCL_VOID v_noreturn_ v_matchproto_(td_vtc_panic)
+vmod_panic(VRT_CTX, VCL_STRANDS str)
 {
-	va_list ap;
 	const char *b;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 
-	va_start(ap, str);
-	b = VRT_String(ctx->ws, "PANIC: ", str, ap);
-	va_end(ap);
+	b = VRT_StrandsWS(ctx->ws, "PANIC:", str);
 	VAS_Fail("VCL", "", 0, b, VAS_VCL);
 }
 
@@ -118,25 +118,26 @@ VCL_VOID v_matchproto_(td_vtc_sleep)
 vmod_sleep(VRT_CTX, VCL_DURATION t)
 {
 
-	CHECK_OBJ_ORNULL(ctx, VRT_CTX_MAGIC);
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
 	VTIM_sleep(t);
 }
 
 /*--------------------------------------------------------------------*/
 
+// XXX this really should be PRIV_TASK state
 static uintptr_t vtc_ws_snapshot;
 
 static struct ws *
 vtc_ws_find(VRT_CTX, VCL_ENUM which)
 {
 
-	if (!strcmp(which, "client"))
+	if (which == VENUM(client))
 		return (ctx->ws);
-	if (!strcmp(which, "backend"))
+	if (which == VENUM(backend))
 		return (ctx->bo->ws);
-	if (!strcmp(which, "session"))
+	if (which == VENUM(session))
 		return (ctx->req->sp->ws);
-	if (!strcmp(which, "thread"))
+	if (which == VENUM(thread))
 		return (ctx->req->wrk->aws);
 	WRONG("vtc_ws_find Illegal enum");
 }
@@ -155,7 +156,7 @@ vmod_workspace_alloc(VRT_CTX, VCL_ENUM which, VCL_INT size)
 	WS_Assert(ws);
 
 	if (size < 0) {
-		size += WS_Reserve(ws, 0);
+		size += WS_ReserveAll(ws);
 		WS_Release(ws, 0);
 	}
 	if (size <= 0) {
@@ -169,6 +170,34 @@ vmod_workspace_alloc(VRT_CTX, VCL_ENUM which, VCL_INT size)
 		memset(p, '\0', size);
 }
 
+VCL_BYTES v_matchproto_(td_vtc_workspace_reserve)
+vmod_workspace_reserve(VRT_CTX, VCL_ENUM which, VCL_INT size)
+{
+	struct ws *ws;
+	unsigned r;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	ws = vtc_ws_find(ctx, which);
+	if (ws == NULL)
+		return (0);
+	WS_Assert(ws);
+
+	if (size < 0) {
+		size += WS_ReserveAll(ws);
+		WS_Release(ws, 0);
+	}
+	if (size <= 0) {
+		VRT_fail(ctx, "Attempted negative WS reservation");
+		return (0);
+	}
+	r = WS_ReserveSize(ws, size);
+	if (r == 0)
+		return (0);
+	WS_Release(ws, 0);
+	return (r);
+}
+
 VCL_INT v_matchproto_(td_vtc_workspace_free)
 vmod_workspace_free(VRT_CTX, VCL_ENUM which)
 {
@@ -182,7 +211,7 @@ vmod_workspace_free(VRT_CTX, VCL_ENUM which)
 		return(-1);
 	WS_Assert(ws);
 
-	u = WS_Reserve(ws, 0);
+	u = WS_ReserveAll(ws);
 	WS_Release(ws, 0);
 	return (u);
 }
@@ -208,19 +237,77 @@ VTC_WS_OP(VOID, , overflow, WS_MarkOverflow(ws))
 VTC_WS_OP(BOOL, (0), overflowed, return (WS_Overflowed(ws)))
 #undef VTC_WS_OP
 
+VCL_BLOB v_matchproto_(td_vtc_workspace_dump)
+vmod_workspace_dump(VRT_CTX, VCL_ENUM which, VCL_ENUM where,
+    VCL_BYTES off, VCL_BYTES len)
+{
+	struct ws *ws;
+	VCL_BYTES l, maxlen = 1024;
+	unsigned char buf[maxlen];
+	const char *p;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	ws = vtc_ws_find(ctx, which);
+	if (ws == NULL)
+		return (NULL);
+	WS_Assert(ws);
+
+	if (len > maxlen) {
+		VRT_fail(ctx, "workspace_dump: max length is %jd",
+		    (intmax_t)maxlen);
+		return (NULL);
+	}
+
+	if (where == VENUM(s))
+		p = ws->s;
+	else if (where == VENUM(f))
+		p = ws->f;
+	else if (where == VENUM(r))
+		p = ws->r;
+	else
+		INCOMPL();
+
+	if (p == NULL) {
+		VSLb(ctx->vsl, SLT_Error, "workspace_dump: NULL");
+		return (NULL);
+	}
+
+	p += off;
+	if (p >= ws->e) {
+		VSLb(ctx->vsl, SLT_Error, "workspace_dump: off limit");
+		return (NULL);
+	}
+
+	l = pdiff(p, ws->e);
+	if (len < l)
+		l = len;
+
+	assert(l < maxlen);
+	memcpy(buf, p, l);
+	p = WS_Copy(ctx->ws, buf, l);
+	if (p == NULL) {
+		VRT_fail(ctx, "workspace_dump: copy failed");
+		return (NULL);
+	}
+	return (VRT_blob(ctx, "workspace_dump", p, l, 0xd000d000));
+}
+
 /*--------------------------------------------------------------------*/
 
 VCL_INT v_matchproto_(td_vtc_typesize)
 vmod_typesize(VRT_CTX, VCL_STRING s)
 {
-	size_t i = 0;
-	const char *p;
+	size_t i = 0, l, a, p = 0;
 
 	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+	AN(s);
+	AN(*s);
 
-	for (p = s; *p; p++) {
-		switch (*p) {
-#define VTC_TYPESIZE(c, t) case c: i += sizeof(t); break;
+	for (; *s; s++) {
+		switch (*s) {
+#define VTC_TYPESIZE(c, t) case c: l = sizeof(t); break;
+		VTC_TYPESIZE('c', char)
 		VTC_TYPESIZE('d', double)
 		VTC_TYPESIZE('f', float)
 		VTC_TYPESIZE('i', int)
@@ -229,10 +316,59 @@ vmod_typesize(VRT_CTX, VCL_STRING s)
 		VTC_TYPESIZE('o', off_t)
 		VTC_TYPESIZE('p', void *)
 		VTC_TYPESIZE('s', short)
+		VTC_TYPESIZE('u', unsigned)
 		VTC_TYPESIZE('z', size_t)
 #undef VTC_TYPESIZE
 		default:	return (-1);
 		}
+		if (l > p)
+			p = l;
+		a = i % l;
+		if (a != 0)
+			i += (l - a); /* align */
+		i += l;
 	}
+	AN(p);
+	a = i % p;
+	if (a != 0)
+		i += (p - a); /* pad */
 	return ((VCL_INT)i);
 }
+
+/*--------------------------------------------------------------------*/
+
+#define BLOB_VMOD_PROXY_HEADER_TYPE	0xc8f34f78
+
+VCL_BLOB v_matchproto_(td_vtc_proxy_header)
+vmod_proxy_header(VRT_CTX, VCL_ENUM venum, VCL_IP client, VCL_IP server,
+    VCL_STRING authority)
+{
+	struct vsb *vsb;
+	const void *h;
+	int version;
+	size_t l;
+
+	CHECK_OBJ_NOTNULL(ctx, VRT_CTX_MAGIC);
+
+	if (venum == VENUM(v1))
+		version = 1;
+	else if (venum == VENUM(v2))
+		version = 2;
+	else
+		WRONG(venum);
+
+	vsb = VSB_new_auto();
+	AN(vsb);
+	VRT_Format_Proxy(vsb, version, client, server, authority);
+	l = VSB_len(vsb);
+	h = WS_Copy(ctx->ws, VSB_data(vsb), l);
+	VSB_delete(vsb);
+
+	if (h == NULL) {
+		VRT_fail(ctx, "proxy_header: out of workspace");
+		return (NULL);
+	}
+
+	return (VRT_blob(ctx, "proxy_header", h, l,
+	    BLOB_VMOD_PROXY_HEADER_TYPE));
+}
